


/*** Select Design ***/

(function () {
  const htmlElement = document.documentElement;
  const designChoiceWrap = document.querySelector('.design-choice-wrap');

  function applyDesign(design) {
    htmlElement.setAttribute('data-design', design);
    designChoiceWrap.querySelectorAll('.design-choice').forEach(el => {
      el.classList.toggle('active', el.dataset.value === design);
    });
  }

  const savedDesign = localStorage.getItem('designPreference') || 'clean';
  applyDesign(savedDesign);

  designChoiceWrap.addEventListener('click', function (event) {
    const target = event.target;
    if (target.classList.contains('design-choice')) {
      const selectedDesign = target.dataset.value;
      applyDesign(selectedDesign);
      localStorage.setItem('designPreference', selectedDesign);
    }
  });
})();


/*** Controling the Tabs ***/

(function () {
  const container = document.querySelector(".ss-tabs-wrap");
  const tabs = Array.from(container.querySelectorAll("[role=tab]"));
  const tabpanels = Array.from(container.querySelectorAll("[role=tabpanel]"));

  // Function to get the corresponding nav element for a given tab ID
  function getNavElement(id) {
    return document.querySelector(`#nav-for-${id}`);
  }

  function setActiveTabFocus(tab) {
    tabs.forEach((t) => t.setAttribute("aria-selected", "false"));
    tab.setAttribute("aria-selected", "true");
    tab.focus();

    // Save the active tab to localStorage
    localStorage.setItem("activeTab", tab.getAttribute("aria-controls"));
  }

  function setActiveTab(id) {
    // Scroll to the top of the page
    window.scrollTo(0, 0);

    // Update tabs and navs states
    tabs.forEach((tab) => tab.classList.remove("active"));
    document.querySelectorAll('.hovering-nav').forEach((nav) => nav.classList.remove('active-nav'));

    const newActiveTab = tabs.find((tab) => tab.getAttribute("aria-controls") === id);
    newActiveTab.classList.add("active");
    setActiveTabFocus(newActiveTab);

    // Update tabpanels
    tabpanels.forEach((panel) => panel.setAttribute("aria-expanded", panel.getAttribute("id") === id));

    // Add active class to the corresponding nav
    const navElement = getNavElement(id);
    if (navElement) {
      navElement.classList.add('active-nav');
    }
  }

  container.addEventListener("click", function (e) {
    const tab = e.target.closest("[role=tab]");
    if (tab) {
      e.preventDefault();
      setActiveTab(tab.getAttribute("aria-controls"));
    }
  });

  // Initialize active tab based on localStorage or default to the first tab
  const initialActiveTab = localStorage.getItem("activeTab");
  const defaultTabId = tabs[0].getAttribute("aria-controls");
  setActiveTab(initialActiveTab || defaultTabId);
})();


/*** Extend <textfield> by clicking the icon ***/

const togglesTA = document.querySelectorAll('.toggle-textarea-height');

togglesTA.forEach(toggle => {
  toggle.addEventListener('click', event => {
    // Toggle 'rotate' class on SVGs
    const expandTextElements = event.currentTarget.querySelectorAll('.expand-text');
    expandTextElements.forEach(el => el.classList.toggle('rotate'));

    const container = event.currentTarget.closest('.text-container');
    if (container) {
      const textarea = container.querySelector('textarea');
      if (textarea) {
        if (textarea.style.height === 'auto' || textarea.style.height === '') {
          // Set height to scrollHeight plus a 2px buffer when expanding
          textarea.style.height = `${textarea.scrollHeight + 2}px`; // Adding an additional 2px when expanding
        } else {
          // If currently expanded, reset to auto and ensure the toggle button is visible
          textarea.style.height = 'auto';
          event.currentTarget.scrollIntoView({ behavior: 'smooth', block: 'center', inline: 'nearest' });
        }
      }
    }
  });
});




/*** Remove Duplicate Lines ***/

function removeDuplicateLines(textarea) {
  var lines = textarea.value.split("\n");
  var uniqueLines = [...new Set(lines.map((line) => line.trim()))].filter(
    Boolean
  );

  textarea.value = uniqueLines.join("\n");
}

document.querySelectorAll(".remove-duplicate-btn").forEach((button) => {
  button.addEventListener("click", function () {
    var textarea = this.parentElement.querySelector("textarea");
    removeDuplicateLines(textarea);
  });
});

/*** Toggle textareas with radio buttons, including in newly created radio fields ***/

document.addEventListener("DOMContentLoaded", function () {
  // Select each group of radios and textboxes
  var groups = document.querySelectorAll(".radios-and-textboxes");

  groups.forEach(handleRadiosAndTextboxes);
});

function handleRadiosAndTextboxes(group) {
  var radioInputs = group.querySelectorAll('input[type="radio"]');

  // Check for checked inputs on page load
  radioInputs.forEach(function (input) {
    if (input.checked) {
      var containerId = input.getAttribute("data-show-container");
      // Ensure that the container is within this group
      var container = group.querySelector("#" + containerId);

      if (container) {
        container.classList.add("visible");
      }
    }
  });

  radioInputs.forEach(function (input) {
    input.addEventListener("change", function () {
      // Only select text containers within this group
      var textContainers = group.querySelectorAll(".text-container");

      textContainers.forEach(function (container) {
        container.classList.remove("visible");
      });

      // Add visible class to appropriate text container
      var containerId = this.getAttribute("data-show-container");
      // Ensure that the container is within this group
      var container = group.querySelector("#" + containerId);

      if (container) {
        container.classList.add("visible");
      }
    });
  });
}


/** Set the height of textboxes to show the content **/

document.addEventListener('DOMContentLoaded', () => {
  // Function to adjust textarea height
  const adjustHeight = (textarea) => {
    textarea.style.height = 'auto'; // Reset height to reduce when content is less
    const extraBuffer = 2; // Extra buffer in pixels
    textarea.style.height = `${textarea.scrollHeight + extraBuffer}px`;
  };

  // Debounce function to limit how often a function can fire
  const debounce = (func, delay) => {
    let debounceTimer;
    return function () {
      clearTimeout(debounceTimer);
      debounceTimer = setTimeout(() => func.apply(this, arguments), delay);
    };
  };

  // Add event listener to each textarea
  document.querySelectorAll('textarea').forEach(textarea => {
    adjustHeight(textarea); // Initially adjust height
    textarea.addEventListener('input', debounce(() => adjustHeight(textarea), 100));
  });
});





/** Save setting with keyboard, using CTRL+S **/
document.addEventListener('keydown', function (event) {
  // Check for 'CTRL + S' or equivalent in other OS systems
  if ((event.ctrlKey || event.metaKey) && event.key === 's') {
    event.preventDefault(); // Prevent the default save action

    // Simulate a click on the 'Save All Changes' button
    document.getElementById('save_wp_ss_button').click();
  }
});


/** Notify of unsaved changes **/
// Flag is reset in in save-settings-fetch-api.js
document.addEventListener('DOMContentLoaded', () => {

  // Function to mark settings as changed
  function markAsDirty() {
    sessionStorage.setItem('settingsChanged', 'true');
    document.getElementById('save_wp_ss_button').classList.add('unsaved-changes');
  }

  // Attach the markAsDirty function to changes in your settings elements
  document.querySelectorAll('.wp-ss-settings input, .wp-ss-settings textarea, .wp-ss-settings select').forEach(element => {
    element.addEventListener('change', markAsDirty);
  });

  // Confirmation dialog on page unload
  window.addEventListener('beforeunload', function (event) {
    if (sessionStorage.getItem('settingsChanged') === 'true') {
      var confirmationMessage = 'You have unsaved changes. Are you sure you want to leave this page?';
      (event || window).returnValue = confirmationMessage; // Cross-browser compatibility
      return confirmationMessage;
    }
  });
});

/* Set an overlay over options that need Image Data Collection */

document.addEventListener('DOMContentLoaded', () => {
  const wpbody = document.querySelector('#wpbody');
  const checkbox = document.querySelector('#image_data_collection');

  // Function to toggle tags based on checkbox state
  const toggleTags = () => {
    const elements = document.querySelectorAll('.needs-image-data');

    elements.forEach(element => {
      const newTag = checkbox.checked ? 'div' : 'a';
      const wrapper = document.createElement(newTag);
      const attributes = element.attributes;
      const fragment = document.createDocumentFragment();
      fragment.appendChild(wrapper);

      // Copy attributes to the new wrapper
      Array.from(attributes).forEach(attr => {
        wrapper.setAttribute(attr.nodeName, attr.nodeValue);
      });

      // Move all child nodes to the wrapper
      while (element.firstChild) {
        wrapper.appendChild(element.firstChild);
      }

      // Replace the element in the DOM
      element.parentNode.replaceChild(fragment, element);
    });
  };

  // Apply the correct tag on initial page load
  toggleTags();

  // Event delegation for checkbox changes
  wpbody.addEventListener('change', event => {
    if (event.target.matches('#image_data_collection')) {
      toggleTags();
    }
  });
});






