/*** Critical CSS repeater fields constructor ***/

(function () {
  const addFieldButton = document.getElementById("add-critical-css-btn");
  const repeaterFieldsContainer = document.getElementById("css-repeater-fields");

  function getCustomPostTypeOptions() {
    return new Promise((resolve, reject) => {
      // Create form data
      const formData = new FormData();
      formData.append('action', 'get_public_post_types');

      // Make the Fetch API request
      fetch(ajaxurl, {
        method: 'POST',
        body: formData
      }).then(response => response.json())
        .then(postTypes => resolve(postTypes))
        .catch(error => reject(error));
    });
  }

  function addField(fieldData) {
    // Generate a unique identifier for this repeater
    let repeaterId = "repeater_" + Math.random().toString(36).slice(2, 11);

    let woocommerceOptions = "";
    if (woo_active_conformation_data.is_woocommerce_active) {
      woocommerceOptions = `
				<option value="is_shop" data-name="shop-page">WooCommerce Shop Page</option>
				<option value="is_product_category" data-name="product-category-page">WooCommerce Product Category Page</option>
				<option value="is_product_tag" data-name="product-tag-page">WooCommerce Product Tag Page</option>
				<option value="is_product" data-name="single-product-page">WooCommerce Single Product Page</option>
				<option value="is_cart" data-name="cart-page">WooCommerce Cart Page</option>
				<option value="is_checkout" data-name="checkout-page">WooCommerce Checkout Page</option>
				<option value="is_account_page" data-name="account-page">WooCommerce Account Page</option>
			`;
    }

    // Fetch custom post type options
    getCustomPostTypeOptions().then(postTypes => {
      let postTypeOptions = '';
      for (let value in postTypes) {
        postTypeOptions += `<option value="${value}" data-name="${value}">${postTypes[value]}</option>`;
      }

      // Use 'postTypeOptions' in the 'fieldHtml' string to add it to the dropdown
      let fieldHtml = `
				<div class="css-repeater" data-repeater-id="${repeaterId}">
					<div class="css-repeater-first-row">
						<select class="css-dropdown" name="repeater_field_${repeaterId}[critical_css_page_template]">
							<option value="" data-name="custom-string">Select a page template</option>
							${postTypeOptions}
							<option value="is_front_page" data-name="front-page">Front Page</option>
							<option value="is_home" data-name="posts-page">Posts Page</option>
							<option value="ss_is_single_blog_post" data-name="single-post">Single Blog Post</option>
							<option value="ss_is_single_page" data-name="single-page">Single Page</option>
							<option value="is_archive" data-name="archive-page">Archive Page</option>
							<option value="is_category" data-name="category-archive">Category Archive Page</option>
							<option value="is_tag" data-name="tag-archive">Tag Archive Page</option>
							<option value="is_tax" data-name="custom-taxonomy-archive">Custom Taxonomy Archive Page</option>
							<option value="is_author" data-name="author-archive">Author Archive Page</option>
							<option value="is_search" data-name="search-results">Search Results Page</option>
							<option value="is_404" data-name="not-found">404 Not Found Page</option>
							<option value="is_post_type_archive" data-name="post-type-archive">Post Type Archive Page</option>
							${woocommerceOptions}
            </select>
            <input class="css-page-string" type="text" name="repeater_field_${repeaterId}[critical_css_page_url]" value="${fieldData.field || ''}" placeholder="Use a part of the URL">
            <button type="button" class="remove-css-field">Remove Field</button>
          </div>
          <textarea class="critical-css" name="repeater_field_${repeaterId}[critical_css]">${fieldData.textarea || ''}</textarea>

        	<div class="radios-and-textboxes">
						<div class="ss-radio-wrap">
							<input type="radio" id="critical_delay_all_links_css_${repeaterId}" class="critical_css_repeater_radio" name="critical_delay_css__links_radio_checked_${repeaterId}" value="critical_delay_all_links_css" data-show-container="show_critical_delay_all_links_css_${repeaterId}" ${fieldData.selectedRadio === 'critical_delay_all_links_css' ? 'checked' : ''}>
							<label for="critical_delay_all_links_css_${repeaterId}" class="ss-radio-label">All Styles</label>
						
							<input type="radio" id="critical_delay_exclude_links_css_${repeaterId}" class="critical_css_repeater_radio" name="critical_delay_css__links_radio_checked_${repeaterId}" value="critical_delay_exclude_links_css" data-show-container="show_critical_delay_exclude_links_css_${repeaterId}" ${fieldData.selectedRadio === 'critical_delay_exclude_links_css' ? 'checked' : ''}>
							<label for="critical_delay_exclude_links_css_${repeaterId}" class="ss-radio-label">Exclude Styles</label>
						
							<input type="radio" id="critical_delay_specific_links_css_${repeaterId}" class="critical_css_repeater_radio" name="critical_delay_css__links_radio_checked_${repeaterId}" value="critical_delay_specific_links_css" data-show-container="show_critical_delay_specific_links_css_${repeaterId}" ${fieldData.selectedRadio === 'critical_delay_specific_links_css' ? 'checked' : ''}>
							<label for="critical_delay_specific_links_css_${repeaterId}" class="ss-radio-label">Only These Styles</label>
						</div>
			
							<div class="text-container" id="show_critical_delay_all_links_css_${repeaterId}">
							<div class="ss-opt-desc">Option Description</div>
						</div>

						<div class="text-container" id="show_critical_delay_exclude_links_css_${repeaterId}">
							<div class="ss-opt-desc">Option Description</div>
							<textarea class="critical_delay_exclude_links_css_textarea" name="repeater_field_${repeaterId}[critical_delay_exclude_links_css_textarea]" id="critical_delay_exclude_links_css_textarea_${repeaterId}">${fieldData.critical_delay_exclude_links_css_textarea || ''}</textarea>
						</div>

						<div class="text-container" id="show_critical_delay_specific_links_css_${repeaterId}">
							<div class="ss-opt-desc">Option Description</div>
							<textarea class="critical_delay_specific_links_css_textarea" name="repeater_field_${repeaterId}[critical_delay_specific_links_css_textarea]" id="critical_delay_specific_links_css_textarea_${repeaterId}">${fieldData.critical_delay_specific_links_css_textarea || ''}</textarea>
						</div>
          </div>
        </div>
      `;

      // Add the new field to the DOM
      repeaterFieldsContainer.insertAdjacentHTML("beforeend", fieldHtml);

      // Add the 'shown' class after a delay to animate the element in
      let newField = repeaterFieldsContainer.lastElementChild;
      setTimeout(function () {
        newField.classList.add("shown");

        // Handle radios and textboxes in the new repeater field
        handleRadiosAndTextboxes(newField);
      }, 20); // Short delay before starting animation
    }).catch(e => {
      console.error(e);
    });
  }

  addFieldButton.addEventListener("click", function () {
    addField({
      selectedRadio: 'critical_delay_all_links_css', // Default value
      critical_delay_exclude_links_css_textarea: '', // Default value
      critical_delay_specific_links_css_textarea: '' // Default value
    });
  });


  repeaterFieldsContainer.addEventListener("click", function (e) {
    if (e.target && e.target.classList.contains("remove-css-field")) {
      // Animate the element out, then remove it
      let fieldToRemove = e.target.closest('.css-repeater');
      fieldToRemove.classList.remove("shown");
      setTimeout(function () {
        fieldToRemove.remove();
      }, 500); // Delay should match transition duration in CSS
    }
  });

  let myScript;

  // Add existing repeater fields
  if (myScript && Array.isArray(myScript.repeaterFields)) {
    myScript.repeaterFields.forEach(addField);
  }

})();


/*** UI - HIDE CUSTOM STRING FIELD in the repeater ***/

// Hide the custom string input area if the user chose a function from the dropdown
(function () {
  function handleDropdownChange(event) {
    // Check if the changed element is a dropdown within a repeater
    if (event.target.matches(".css-repeater .css-dropdown")) {
      toggleInputState(event.target);
    }
  }

  function toggleInputState(dropdown) {
    let input = dropdown
      .closest(".css-repeater")
      .querySelector("input.css-page-string");

    if (dropdown.selectedIndex === 0) {
      input.classList.remove("disabled-input");
    } else {
      input.classList.add("disabled-input");
    }
  }

  function initialCheck() {
    let dropdowns = document.querySelectorAll(".css-repeater .css-dropdown");
    dropdowns.forEach(toggleInputState);
  }

  document.addEventListener("DOMContentLoaded", initialCheck);
  // Attach the event listener to the css-repeater-fields div instead of the whole document
  document
    .querySelector("#css-repeater-fields")
    .addEventListener("change", handleDropdownChange);
})();

/*** UI - REMOVE DROPDOWN OPTIONS in the repeater select option ***/

// Remove from the dropdown list, options thta have already been selected
(function () {
  const dropdownContainer = document.querySelector("#css-repeater-fields");

  let dropdowns = [];

  const updateDropdowns = () => {
    dropdowns = Array.from(
      dropdownContainer.querySelectorAll(".css-dropdown")
    ).map((drop) => ({
      el: drop,
      selectedValue: drop.options[drop.selectedIndex].value,
    }));
  };

  const resetOptions = () => {
    dropdowns.forEach(({ el }) => {
      Array.from(el.options).forEach((opt, index) => {
        if (index !== 0) opt.disabled = false;
      });
    });
  };

  const removeSelectedOptions = () => {
    const selectedOptions = dropdowns.map(
      ({ el }) => el.options[el.selectedIndex].value
    );
    dropdowns.forEach(({ el }) => {
      Array.from(el.options).forEach((opt, index) => {
        if (
          index !== 0 &&
          selectedOptions.includes(opt.value) &&
          opt.value !== el.options[el.selectedIndex].value
        ) {
          opt.disabled = true;
        }
      });
    });
  };

  dropdownContainer.addEventListener("change", (e) => {
    if (e.target.classList.contains("css-dropdown")) {
      resetOptions();
      removeSelectedOptions();
    }
  });

  // create a MutationObserver instance
  const observer = new MutationObserver((mutationsList, observer) => {
    for (let mutation of mutationsList) {
      // if new nodes are added
      if (mutation.type === "childList") {
        updateDropdowns();
        resetOptions();
        removeSelectedOptions();
      }
    }
  });

  // start observing the '#css-repeater-fields' for child node changes
  observer.observe(dropdownContainer, { childList: true });

  updateDropdowns();
  resetOptions();
  removeSelectedOptions();
})();


/* Repeater fields for preloading images */

document.addEventListener('DOMContentLoaded', function () {
  const addButton = document.getElementById('add-preload-img-btn');
  addButton.addEventListener('click', function () {
    const repeaterContainer = document.getElementById('ss-repeater-fields');
    const isWooCommerceActive = repeaterContainer.getAttribute('data-woocommerce-active') === 'true';
    const newField = document.createElement('div');
    newField.classList.add('img-repeater', 'shown');

    // General options
    const options = {
      "": "Select a page template",
      "is_front_page": "Front Page",
      "is_home": "Posts Page",
      "ss_is_single_blog_post": "Single Blog Post",
      "ss_is_single_page": "Single Page",
      "is_archive": "Archive Page",
      "is_category": "Category Archive",
      "is_tag": "Tag Archive",
      "is_tax": "Custom Taxonomy Archive",
      "is_author": "Author Archive",
      "is_search": "Search Results",
      "is_404": "404 Not Found",
      "is_post_type_archive": "Post Type Archive"
    };

    // Initialize WooCommerce options as null or as an empty object
    let woocommerceOptions = {};

    if (isWooCommerceActive) {
      // Directly assign the object to the already declared `woocommerceOptions` variable
      woocommerceOptions = {
        "is_shop": "WooCommerce Shop Page",
        "is_product_category": "WooCommerce Product Category Page",
        "is_product_tag": "WooCommerce Product Tag Page",
        "is_product": "WooCommerce Single Product Page",
        "is_cart": "WooCommerce Cart Page",
        "is_checkout": "WooCommerce Checkout Page",
        "is_account_page": "WooCommerce Account Page",
      };
    }

    // Now, when merging, `woocommerceOptions` will either be an empty object or the defined options, depending on WooCommerce's active status
    const allOptions = { ...options, ...woocommerceOptions };


    const translations = {
      "preloadFirst": "Preload first",
      "imagesOn": "images on",
      // Any other translations you need
    };

    newField.innerHTML = `
  <div class="img-repeater shown">
      <div class="img-repeater-first-row">
          <div class="cbx-numeric-label-wrp">
              <div class="option-label">${translations.preloadFirst}</div>
              <select name="repeater_img_preload_first_x_select" id="repeater_img_preload_first_x_select">
                  ${[...Array(6).keys()].map(i => `<option value="${i}">${i}</option>`).join('')}
              </select>
              <div class="option-label">${translations.imagesOn}</div>
          </div>

          <select class="repeater-dropdown" name="preload_img_repeater_data[preload_img_page_templates]">
              ${Object.entries(allOptions).map(([value, display]) => `<option value="${value}">${display}</option>`).join('')}
          </select>

          <input class="repeater-page-string" type="text" name="preload_img_repeater_data[preload_img_page_url]" value="" placeholder="Use a part of the URL">

          <button type="button" class="remove-repeater-field">Remove Field</button>
      </div>
  </div>
`;
    repeaterContainer.appendChild(newField);

    // Event delegation to catch clicks on any "Remove Field" button, even for dynamically added fields
    repeaterContainer.addEventListener('click', function (e) {
      if (e.target.classList.contains('remove-repeater-field')) {
        e.preventDefault(); // Prevent form submission or navigation
        e.target.closest('.img-repeater').remove(); // Remove the closest parent `.img-repeater` container
      }
    });
  });
});




