/*** Saving With Fetch API Ajax ***/
document
  .getElementById("save_wp_ss_button")
  .addEventListener("click", handleSaveClick);

async function handleSaveClick() {
  try {
    const savingSuccess = document.querySelector(".saving-success");
    const savingError = document.querySelector(".saving-error");
    const savingInfo = document.querySelector(".saving-info");
    const savingOverlay = document.querySelector(".saving-overlay");

    savingSuccess.classList.remove("showing");
    savingError.classList.remove("showing");
    savingInfo.classList.add("showing");
    savingOverlay.classList.add("showing");

    const formData = createFormData();

    const response = await fetch(wpss_ajax_object.ajax_url, {
      method: "POST",
      body: formData,
    });

    // Log the response text
    const responseText = await response.text();
    console.log("Server response text:", responseText);

    if (!response.ok) {
      throw new Error(`HTTP error! status: ${response.status}`);
    }

    // Parse the response as JSON
    const jsonResponse = JSON.parse(responseText);

    //const jsonResponse = await response.json();
    console.log("Server response:", jsonResponse);

    savingInfo.classList.remove("showing");
    savingOverlay.classList.remove("showing");
    if (jsonResponse.success) {
      savingSuccess.classList.add("showing");
      setTimeout(function () {
        savingSuccess.classList.remove("showing");
      }, 4000); // remove after 4 seconds

      // Reset the 'Notify of unsaved changes' flag from admin-script.js
      // Remove the session storage item upon successful save
      sessionStorage.removeItem('settingsChanged');

      // Also remove the 'unsaved-changes' class from the button
      document.getElementById('save_wp_ss_button').classList.remove('unsaved-changes');

    } else {
      savingError.classList.add("showing");
    }
  } catch (error) {
    console.error("Fetch error:", error);
  }
}

function createFormData() {
  const formData = new FormData();

  formData.append("action", "save_wp_ss");
  formData.append("security", document.getElementById("wp_ss_nonce").value);

  /// JS ///

  // Dequeue Scripts
  formData.append(
    "ss_enable_dequeue",
    document.getElementById("ss_enable_dequeue").checked ? "1" : "0"
  );

  let checkedDeqLinksJSRadio = document.querySelector(
    'input[name="ss_dequeue_js_links_radio_checked"]:checked'
  ).value;
  formData.append("ss_dequeue_js_links_radio_checked", checkedDeqLinksJSRadio); // selected radio button
  formData.append(
    "ss_dequeue_exclude_links_js_textarea",
    document.getElementById("ss_dequeue_exclude_links_js_textarea").value // textarea to exclude pages
  );
  formData.append(
    "ss_dequeue_spacific_links_js_textarea",
    document.getElementById("ss_dequeue_spacific_links_js_textarea").value // textarea to run only on pages
  );

  let DequeueRadioButtons = document.querySelector(
    'input[name="ss_deq_radio_checked"]:checked'
  ).value;
  formData.append("ss_deq_radio_checked", DequeueRadioButtons); // selected radio button
  formData.append(
    "ss_exc_deq_js_textarea",
    document.getElementById("ss_exc_deq_js_textarea").value // textarea to exclude pages
  );
  formData.append(
    "ss_deq_spec_js_textarea",
    document.getElementById("ss_deq_spec_js_textarea").value // textarea to run only on pages
  );

  formData.append(
    "ss_dequeue_js_exclude_home",
    document.getElementById("ss_dequeue_js_exclude_home").checked ? "1" : "0"
  );
  formData.append(
    "ss_dequeue_js_include_home",
    document.getElementById("ss_dequeue_js_include_home").checked ? "1" : "0"
  );
  formData.append(
    "dequeue_scripts_exclude_3rd_party",
    document.getElementById("dequeue_scripts_exclude_3rd_party").checked ? "1" : "0"
  );

  // Defer JS
  formData.append(
    "ss_enable_defer_ext_js",
    document.getElementById("ss_enable_defer_ext_js").checked ? "1" : "0"
  );
  let DeferRadioButtons = document.querySelector(
    'input[name="ss_defer_radio_checked"]:checked'
  ).value;
  formData.append("ss_defer_radio_checked", DeferRadioButtons); // selected radio button
  formData.append(
    "defer_all_scripts",
    document.getElementById("defer_all_scripts").value // textarea exclude from defer
  );
  formData.append(
    "ss_exc_defer_textarea",
    document.getElementById("ss_exc_defer_textarea").value // textarea exclude from defer
  );
  formData.append(
    "ss_defer_specific_textarea",
    document.getElementById("ss_defer_specific_textarea").value // textarea defer specific pages
  );
  formData.append(
    "ss_defer_exclude_dependent",
    document.getElementById("ss_defer_exclude_dependent").checked ? "1" : "0"
  );
  formData.append(
    "ss_defer_exclude_support",
    document.getElementById("ss_defer_exclude_support").checked ? "1" : "0"
  );

  // Delay scripts
  formData.append(
    "ss_enable_delay_ext_js",
    document.getElementById("ss_enable_delay_ext_js").checked ? "1" : "0"
  );
  let DelayExRadioLinks = document.querySelector(
    'input[name="ss_delay_js_links_radio_checked"]:checked'
  ).value;
  formData.append("ss_delay_js_links_radio_checked", DelayExRadioLinks); // selected radio button
  formData.append(
    "ss_delay_ext_exclude_links_textarea",
    document.getElementById("ss_delay_ext_exclude_links_textarea").value
  );
  formData.append(
    "ss_delay_ext_specific_links_textarea",
    document.getElementById("ss_delay_ext_specific_links_textarea").value
  );
  let DelayExRadioPages = document.querySelector(
    'input[name="ss_delay_ext_pages_radio_checked"]:checked'
  ).value;
  formData.append("ss_delay_ext_pages_radio_checked", DelayExRadioPages);
  formData.append(
    "ss_delay_ext_excl_pages_textarea",
    document.getElementById("ss_delay_ext_excl_pages_textarea").value
  );
  formData.append(
    "ss_delay_ext_specific_pages_textarea",
    document.getElementById("ss_delay_ext_specific_pages_textarea").value
  );

  formData.append(
    "ss_delay_js_exclude_home",
    document.getElementById("ss_delay_js_exclude_home").checked ? "1" : "0"
  );
  formData.append(
    "ss_delay_js_include_home",
    document.getElementById("ss_delay_js_include_home").checked ? "1" : "0"
  );

  /// CSS ///

  // Preload CSS
  formData.append(
    "preload_styles",
    document.getElementById("preload_styles").checked ? "1" : "0"
  );

  // Used CSS
  formData.append(
    "enable_used_css",
    document.getElementById("enable_used_css").checked ? "1" : "0"
  );
  formData.append(
    "used_css_file_info",
    document.getElementById("used_css_file_info").checked ? "1" : "0"
  );
  formData.append(
    "used_css_different_post_layout",
    document.getElementById("used_css_different_post_layout").checked ? "1" : "0"
  );
  formData.append(
    "used_css_different_cpt_layout",
    document.getElementById("used_css_different_cpt_layout").checked ? "1" : "0"
  );
  let loadUsedCssOnPagesRadio = document.querySelector(
    'input[name="load_used_css_on_pages_radio"]:checked'
  ).value;
  formData.append("load_used_css_on_pages_radio", loadUsedCssOnPagesRadio);
  formData.append(
    "used_css_exclude_home",
    document.getElementById("used_css_exclude_home").checked ? "1" : "0"
  );
  formData.append(
    "used_css_exclude_pages_textarea",
    document.getElementById("used_css_exclude_pages_textarea").value // textarea to exclude pages
  );
  formData.append(
    "used_css_include_home",
    document.getElementById("used_css_include_home").checked ? "1" : "0"
  );
  formData.append(
    "used_css_spacific_pages_textarea",
    document.getElementById("used_css_spacific_pages_textarea").value // textarea to exclude pages
  );

  // Dequeue CSS
  formData.append(
    "enable_dequeue_styles",
    document.getElementById("enable_dequeue_styles").checked ? "1" : "0"
  );

  let checkedDeqLinksCSSRadio = document.querySelector(
    'input[name="dequeue_css_links_radio_checked"]:checked'
  ).value;
  formData.append("dequeue_css_links_radio_checked", checkedDeqLinksCSSRadio); // selected radio button
  formData.append(
    "dequeue_exclude_links_css_textarea",
    document.getElementById("dequeue_exclude_links_css_textarea").value // textarea to exclude pages
  );
  formData.append(
    "dequeue_spacific_links_css_textarea",
    document.getElementById("dequeue_spacific_links_css_textarea").value // textarea to run only on pages
  );

  let checkedDeqCSSPagesRadio = document.querySelector(
    'input[name="remove_pages_css_radio_checked"]:checked'
  ).value;
  formData.append("remove_pages_css_radio_checked", checkedDeqCSSPagesRadio); // selected radio button
  formData.append(
    "dequeue_exclude_pages_css_textarea",
    document.getElementById("dequeue_exclude_pages_css_textarea").value // textarea to exclude pages
  );
  formData.append(
    "dequeue_spacific_pages_css_textarea",
    document.getElementById("dequeue_spacific_pages_css_textarea").value // textarea to run only on pages
  );

  formData.append(
    "dequeue_css_exclude_home",
    document.getElementById("dequeue_css_exclude_home").checked ? "1" : "0"
  );
  formData.append(
    "dequeue_css_include_home",
    document.getElementById("dequeue_css_include_home").checked ? "1" : "0"
  );
  formData.append(
    "dequeue_css_exclude_3rd_party",
    document.getElementById("dequeue_css_exclude_3rd_party").checked ? "1" : "0"
  );
  formData.append(
    "used_css_exclude_google",
    document.getElementById("used_css_exclude_google").checked ? "1" : "0"
  );

  // Delay CSS
  formData.append(
    "enable_ext_delay_css",
    document.getElementById("enable_ext_delay_css").checked ? "1" : "0"
  );

  let checkedCSSDelayLinksRadio = document.querySelector(
    'input[name="delay_css_links_radio_checked"]:checked'
  ).value;
  formData.append("delay_css_links_radio_checked", checkedCSSDelayLinksRadio); // selected radio button
  formData.append(
    "delay_exclude_links_css_textarea",
    document.getElementById("delay_exclude_links_css_textarea").value // textarea to exclude pages
  );
  formData.append(
    "delay_spacific_links_css_textarea",
    document.getElementById("delay_spacific_links_css_textarea").value // textarea to run only on pages
  );

  let checkedCSSDelayPagesRadio = document.querySelector(
    'input[name="delay_css_pages_radio_checked"]:checked'
  ).value;
  formData.append("delay_css_pages_radio_checked", checkedCSSDelayPagesRadio); // selected radio button
  formData.append(
    "delay_exclude_pages_css_textarea",
    document.getElementById("delay_exclude_pages_css_textarea").value // textarea to exclude pages
  );
  formData.append(
    "delay_specific_pages_css_textarea",
    document.getElementById("delay_specific_pages_css_textarea").value // textarea to run only on pages
  );

  formData.append(
    "delay_css_exclude_home",
    document.getElementById("delay_css_exclude_home").checked ? "1" : "0"
  );
  formData.append(
    "delay_css_include_home",
    document.getElementById("delay_css_include_home").checked ? "1" : "0"
  );

  // Critical CSS repateer
  formData.append(
    "critical_css_enable",
    document.getElementById("critical_css_enable").checked ? "1" : "0"
  );


  /// IMAGE DATA COLLECTION ///
  formData.append(
    "image_data_collection",
    document.getElementById("image_data_collection").checked ? "1" : "0"
  ); 
  formData.append(
    "image_data_collection_exclude_home",
    document.getElementById("image_data_collection_exclude_home").checked ? "1" : "0"
  ); 
  formData.append(
    "image_data_collection_exclude_pages_cbx",
    document.getElementById("image_data_collection_exclude_pages_cbx").checked ? "1" : "0"
  );
  formData.append(
    "image_data_collection_exclude_pages_textarea",
    document.getElementById("image_data_collection_exclude_pages_textarea").value
  );
  /* Check if these dynamically added options are present */
  if (document.getElementById("image_data_collection_exclude_posts")) {
    formData.append(
      "image_data_collection_exclude_posts",
      document.getElementById("image_data_collection_exclude_posts").checked ? "1" : "0"
    );
  }
  if (document.getElementById("image_data_collection_exclude_products")) {
    formData.append(
      "image_data_collection_exclude_products",
      document.getElementById("image_data_collection_exclude_products").checked ? "1" : "0"
    );
  }
  if (document.getElementById("image_data_collection_exclude_cpts")) {
    formData.append(
      "image_data_collection_exclude_cpts",
      document.getElementById("image_data_collection_exclude_cpts").checked ? "1" : "0"
    );
  }
  if (document.getElementById("image_data_collection_exclude_cpts_textarea")) {
    formData.append(
      "image_data_collection_exclude_cpts_textarea",
      document.getElementById("image_data_collection_exclude_cpts_textarea").value
    );
  }


  /// LAZY LOAD ///

  // Images
  formData.append(
    "lazy_load_images",
    document.getElementById("lazy_load_images").checked ? "1" : "0"
  );
  formData.append(
    "lazy_load_images_checkbox",
    document.getElementById("lazy_load_images_checkbox").checked ? "1" : "0"
  );
  formData.append(
    "lazyloaded_img_visually_mark",
    document.getElementById("lazyloaded_img_visually_mark").checked ? "1" : "0"
  );
  formData.append(
    "lazy_img_preload_first_enable",
    document.getElementById("lazy_img_preload_first_enable").checked ? "1" : "0"
  );
  formData.append(
    "lazy_img_preload_first_x_select",
    document.getElementById("lazy_img_preload_first_x_select").value
  );


  formData.append(
    "lazy_img_preload_selector_enable",
    document.getElementById("lazy_img_preload_selector_enable").checked ? "1" : "0"
  );
  formData.append(
    "preload_featured_img_blog",
    document.getElementById("preload_featured_img_blog").checked ? "1" : "0"
  );
  formData.append(
    "preload_featured_img_product",
    document.getElementById("preload_featured_img_product").checked ? "1" : "0"
  );
  formData.append(
    "lazy_img_preload_selector",
    document.getElementById("lazy_img_preload_selector").checked ? "1" : "0"
  );
  formData.append(
    "lazy_img_selector_textarea",
    document.getElementById("lazy_img_selector_textarea").value
  );
  formData.append(
    "lazy_img_preload_parent_selector",
    document.getElementById("lazy_img_preload_parent_selector").checked
      ? "1"
      : "0"
  );
  formData.append(
    "lazy_img_parent_selector_textarea",
    document.getElementById("lazy_img_parent_selector_textarea").value
  );
  formData.append(
    "lazy_img_fadein",
    document.getElementById("lazy_img_fadein").checked ? "1" : "0"
  );
  formData.append(
    "lazy_img_fadein_duration",
    document.getElementById("lazy_img_fadein_duration").value
  );
  formData.append(
    "lazy_img_fadein_exclude_selector",
    document.getElementById("lazy_img_fadein_exclude_selector").checked ? "1" : "0"
  );
  formData.append(
    "lazy_img_fadein_exclude_selector_textarea",
    document.getElementById("lazy_img_fadein_exclude_selector_textarea").value
  );
  formData.append(
    "lazy_img_fadein_exclude_parent_selector",
    document.getElementById("lazy_img_fadein_exclude_parent_selector").checked ? "1" : "0"
  );
  formData.append(
    "lazy_img_fadein_exclude_parent_selector_textarea",
    document.getElementById("lazy_img_fadein_exclude_parent_selector_textarea").value
  );

  formData.append(
    "lazy_img_exclude_home_cbx",
    document.getElementById("lazy_img_exclude_home_cbx").checked ? "1" : "0"
  );
  formData.append(
    "lazy_img_include_home_cbx",
    document.getElementById("lazy_img_include_home_cbx").checked ? "1" : "0"
  );
  formData.append(
    "img_first_x_exlude_selector",
    document.getElementById("img_first_x_exlude_selector").checked ? "1" : "0"
  );
  formData.append(
    "img_first_x_exlude_selector_textarea",
    document.getElementById("img_first_x_exlude_selector_textarea").value
  );
  formData.append(
    "img_first_x_exlude_parent_selector",
    document.getElementById("img_first_x_exlude_parent_selector").checked ? "1" : "0"
  );
  formData.append(
    "img_first_x_exlude_selector_parent_textarea",
    document.getElementById("img_first_x_exlude_selector_parent_textarea").value
  );
  formData.append(
    "preloaded_img_visually_mark",
    document.getElementById("preloaded_img_visually_mark").checked ? "1" : "0"
  );

  let Radio_img_first_x_select_pages = document.querySelector(
    'input[name="img_first_x_select_pages_radio"]:checked'
  ).value;
  formData.append("img_first_x_select_pages_radio", Radio_img_first_x_select_pages); // selected radio button

  formData.append(
    "img_first_x_exlude_pages_textarea",
    document.getElementById("img_first_x_exlude_pages_textarea").value
  );
  formData.append(
    "img_first_x_include_pages_textarea",
    document.getElementById("img_first_x_include_pages_textarea").value
  );

  formData.append(
    "image_auto_set_thumbnails",
    document.getElementById("image_auto_set_thumbnails").checked ? "1" : "0"
  );
  formData.append(
    "image_custom_pixel_density_desktop",
    document.getElementById("image_custom_pixel_density_desktop").checked ? "1" : "0"
  );
  formData.append(
    "image_custom_pixel_density_tablet",
    document.getElementById("image_custom_pixel_density_tablet").checked ? "1" : "0"
  );
  formData.append(
    "image_custom_pixel_density_mobile",
    document.getElementById("image_custom_pixel_density_mobile").checked ? "1" : "0"
  );
  formData.append(
    "image_auto_set_fixed_pixels_desktop",
    document.getElementById("image_auto_set_fixed_pixels_desktop").value
  );
  formData.append(
    "image_auto_set_fixed_pixels_tablet",
    document.getElementById("image_auto_set_fixed_pixels_tablet").value
  );
  formData.append(
    "image_auto_set_fixed_pixels_mobile",
    document.getElementById("image_auto_set_fixed_pixels_mobile").value
  );
  formData.append(
    "image_auto_set_exclude_selector",
    document.getElementById("image_auto_set_exclude_selector").checked ? "1" : "0"
  );
  formData.append(
    "image_auto_set_exclude_selector_textarea",
    document.getElementById("image_auto_set_exclude_selector_textarea").value
  );
  formData.append(
    "image_auto_set_exclude_parent_selector",
    document.getElementById("image_auto_set_exclude_parent_selector").checked ? "1" : "0"
  );
  formData.append(
    "image_auto_set_exclude_parent_selector_textarea",
    document.getElementById("image_auto_set_exclude_parent_selector_textarea").value
  );
  formData.append(
    "image_auto_set_thumbs_info",
    document.getElementById("image_auto_set_thumbs_info").checked ? "1" : "0"
  );

  formData.append(
    "lazy_image_exclude_home",
    document.getElementById("lazy_image_exclude_home").checked ? "1" : "0"
  );
  formData.append(
    "lazy_image_exclude_pages_cbx",
    document.getElementById("lazy_image_exclude_pages_cbx").checked ? "1" : "0"
  );
  formData.append(
    "lazy_image_exclude_pages_textarea",
    document.getElementById("lazy_image_exclude_pages_textarea").value
  );
  formData.append(
    "lazy_image_exclude_selector",
    document.getElementById("lazy_image_exclude_selector").checked ? "1" : "0"
  );
  formData.append(
    "lazy_image_exclude_selector_textarea",
    document.getElementById("lazy_image_exclude_selector_textarea").value
  );
  formData.append(
    "lazy_image_exclude_parent_selector",
    document.getElementById("lazy_image_exclude_parent_selector").checked ? "1" : "0"
  );
  formData.append(
    "lazy_image_exclude_parent_selector_textarea",
    document.getElementById("lazy_image_exclude_parent_selector_textarea").value
  );

  let imagesPlaceholderRadio = document.querySelector(
    'input[name="imagesPlaceholderRadio"]:checked'
  ).value;
  formData.append("imagesPlaceholderRadio", imagesPlaceholderRadio);
  formData.append(
    "img_load_smallest_thumbnail_exclude_selector",
    document.getElementById("img_load_smallest_thumbnail_exclude_selector").checked ? "1" : "0"
  );
  formData.append(
    "img_load_smallest_thumbnail_exclude_selector_textarea",
    document.getElementById("img_load_smallest_thumbnail_exclude_selector_textarea").value
  );
  formData.append(
    "img_load_smallest_thumbnail_exclude_parent_selector",
    document.getElementById("img_load_smallest_thumbnail_exclude_parent_selector").checked ? "1" : "0"
  );
  formData.append(
    "img_load_smallest_thumbnail_exclude_parent_selector_textarea",
    document.getElementById("img_load_smallest_thumbnail_exclude_parent_selector_textarea").value
  );


  formData.append(
    "lazy_img_placeholder_exclude_selector",
    document.getElementById("lazy_img_placeholder_exclude_selector").checked ? "1" : "0"
  );
  formData.append(
    "lazy_img_placeholder_exclude_selector_textarea",
    document.getElementById("lazy_img_placeholder_exclude_selector_textarea").value
  );
  formData.append(
    "lazy_img_placeholder_exclude_parent_selector",
    document.getElementById("lazy_img_placeholder_exclude_parent_selector").checked ? "1" : "0"
  );
  formData.append(
    "lazy_img_placeholder_exclude_parent_selector_textarea",
    document.getElementById("lazy_img_placeholder_exclude_parent_selector_textarea").value
  );


  /// Background Images ///
  formData.append(
    "lazy_load_background",
    document.getElementById("lazy_load_background").checked ? "1" : "0"
  );
  formData.append(
    "lazy_background_checkbox",
    document.getElementById("lazy_background_checkbox").checked ? "1" : "0"
  );
  formData.append(
    "lazy_background_exclude_selector",
    document.getElementById("lazy_background_exclude_selector").checked
      ? "1"
      : "0"
  );
  formData.append(
    "lazy_background_exclude_selector_textarea",
    document.getElementById("lazy_background_exclude_selector_textarea").value
  );
  formData.append(
    "lazy_background_exclude_parent_selector",
    document.getElementById("lazy_background_exclude_parent_selector").checked
      ? "1"
      : "0"
  );
  formData.append(
    "lazy_background_exclude_parent_selector_textarea",
    document.getElementById("lazy_background_exclude_parent_selector_textarea").value
  );
  formData.append(
    "lazy_background_exclude_pages_cbx",
    document.getElementById("lazy_background_exclude_pages_cbx").checked
      ? "1"
      : "0"
  );
  formData.append(
    "lazy_background_exclude_pages_textarea",
    document.getElementById("lazy_background_exclude_pages_textarea").value
  );

  formData.append(
    "lazy_background_exclude_home",
    document.getElementById("lazy_background_exclude_home").checked ? "1" : "0"
  );
  formData.append(
    "lazy_bg_fadein",
    document.getElementById("lazy_bg_fadein").checked ? "1" : "0"
  );
  formData.append(
    "lazy_bg_fadein_duration",
    document.getElementById("lazy_bg_fadein_duration").value
  );
  formData.append(
    "preloaded_bg_visually_mark",
    document.getElementById("preloaded_bg_visually_mark").checked ? "1" : "0"
  );
  formData.append(
    "preloaded_bg_lazyloaded_mark",
    document.getElementById("preloaded_bg_lazyloaded_mark").checked ? "1" : "0"
  );

  formData.append(
    "preload_background_images",
    document.getElementById("preload_background_images").checked ? "1" : "0"
  );
  formData.append(
    "preload_background_selector",
    document.getElementById("preload_background_selector").checked ? "1" : "0"
  );
  formData.append(
    "preload_background_selector_textarea",
    document.getElementById("preload_background_selector_textarea").value
  );
  formData.append(
    "preload_background_parent_selector",
    document.getElementById("preload_background_parent_selector").checked ? "1" : "0"
  );
  formData.append(
    "preload_background_parent_selector_textarea",
    document.getElementById("preload_background_parent_selector_textarea").value
  );
  let Radio_back_img_placeholder = document.querySelector(
    'input[name="back_img_placeholder_radio"]:checked'
  ).value;
  formData.append("back_img_placeholder_radio", Radio_back_img_placeholder);

  formData.append(
    "background_auto_set_thumbnails",
    document.getElementById("background_auto_set_thumbnails").checked ? "1" : "0"
  );
  formData.append(
    "background_auto_set_thumbs_info",
    document.getElementById("background_auto_set_thumbs_info").checked ? "1" : "0"
  );
  formData.append(
    "background_custom_pixel_density_desktop",
    document.getElementById("background_custom_pixel_density_desktop").checked ? "1" : "0"
  );
  formData.append(
    "background_custom_pixel_density_tablet",
    document.getElementById("background_custom_pixel_density_tablet").checked ? "1" : "0"
  );
  formData.append(
    "background_custom_pixel_density_mobile",
    document.getElementById("background_custom_pixel_density_mobile").checked ? "1" : "0"
  );
  formData.append(
    "backgound_auto_set_fixed_pixels_desktop",
    document.getElementById("backgound_auto_set_fixed_pixels_desktop").value
  );
  formData.append(
    "backgound_auto_set_fixed_pixels_tablet",
    document.getElementById("backgound_auto_set_fixed_pixels_tablet").value
  );
  formData.append(
    "backgound_auto_set_fixed_pixels_mobile",
    document.getElementById("backgound_auto_set_fixed_pixels_mobile").value
  );
  formData.append(
    "backgound_auto_set_exclude_selector",
    document.getElementById("backgound_auto_set_exclude_selector").checked ? "1" : "0"
  );
  formData.append(
    "backgound_auto_set_exclude_selector_textarea",
    document.getElementById("backgound_auto_set_exclude_selector_textarea").value
  );
  formData.append(
    "backgound_auto_set_exclude_parent_selector",
    document.getElementById("backgound_auto_set_exclude_parent_selector").checked ? "1" : "0"
  );
  formData.append(
    "backgound_auto_set_exclude_parent_selector_textarea",
    document.getElementById("backgound_auto_set_exclude_parent_selector_textarea").value
  );

  /// LCP ///
  formData.append(
    "lcp_enable",
    document.getElementById("lcp_enable").checked ? "1" : "0"
  );
  formData.append(
    "visually_mark_lcp",
    document.getElementById("visually_mark_lcp").checked ? "1" : "0"
  );
  formData.append(
    "display_lcp_info",
    document.getElementById("display_lcp_info").checked ? "1" : "0"
  );
  formData.append(
    "auto_preload_lcp",
    document.getElementById("auto_preload_lcp").checked ? "1" : "0"
  );
  formData.append(
    "lcp_prelod_exclude_selector",
    document.getElementById("lcp_prelod_exclude_selector").checked ? "1" : "0"
  );
  formData.append(
    "lcp_prelod_exclude_selector_textarea",
    document.getElementById("lcp_prelod_exclude_selector_textarea").value
  );
  formData.append(
    "lcp_prelod_exclude_parent_selector",
    document.getElementById("lcp_prelod_exclude_parent_selector").checked ? "1" : "0"
  );
  formData.append(
    "lcp_prelod_exclude_parent_selector_textarea",
    document.getElementById("lcp_prelod_exclude_parent_selector_textarea").value
  );
  formData.append(
    "lcp_prelod_exclude_home_cbx",
    document.getElementById("lcp_prelod_exclude_home_cbx").checked ? "1" : "0"
  );
  formData.append(
    "lcp_prelod_exclude_pages_cbx",
    document.getElementById("lcp_prelod_exclude_pages_cbx").checked ? "1" : "0"
  );
  formData.append(
    "lcp_prelod_exclude_pages_textarea",
    document.getElementById("lcp_prelod_exclude_pages_textarea").value
  );
  /* Check if these dynamically added options are present */
  if (document.getElementById("lcp_prelod_exclude_posts")) {
    formData.append(
      "lcp_prelod_exclude_posts",
      document.getElementById("lcp_prelod_exclude_posts").checked ? "1" : "0"
    );
  }
  if (document.getElementById("lcp_prelod_exclude_products")) {
    formData.append(
      "lcp_prelod_exclude_products",
      document.getElementById("lcp_prelod_exclude_products").checked ? "1" : "0"
    );
  }
  if (document.getElementById("lcp_prelod_exclude_cpts")) {
    formData.append(
      "lcp_prelod_exclude_cpts",
      document.getElementById("lcp_prelod_exclude_cpts").checked ? "1" : "0"
    );
  }
  if (document.getElementById("lcp_prelod_exclude_cpts_textarea")) {
    formData.append(
      "lcp_prelod_exclude_cpts_textarea",
      document.getElementById("lcp_prelod_exclude_cpts_textarea").value
    );
  }

  /// iFrames ///
  formData.append(
    "lazy_load_iframes",
    document.getElementById("lazy_load_iframes").checked ? "1" : "0"
  );
  formData.append(
    "lazy_iframe_exclude_selector",
    document.getElementById("lazy_iframe_exclude_selector").checked ? "1" : "0"
  );
  formData.append(
    "lazy_iframe_exclude_selector_textarea",
    document.getElementById("lazy_iframe_exclude_selector_textarea").value
  );
  formData.append(
    "lazy_iframe_exclude_parent_selector",
    document.getElementById("lazy_iframe_exclude_parent_selector").checked ? "1" : "0"
  );
  formData.append(
    "lazy_iframe_exclude_parent_selector_textarea",
    document.getElementById("lazy_iframe_exclude_parent_selector_textarea")
      .value
  );
  formData.append(
    "lazy_iframe_exclude_pages_cbx",
    document.getElementById("lazy_iframe_exclude_pages_cbx").checked ? "1" : "0"
  );
  formData.append(
    "lazy_iframe_exclude_pages_textarea",
    document.getElementById("lazy_iframe_exclude_pages_textarea").value
  );
  formData.append(
    "lazy_iframe_placeholder",
    document.getElementById("lazy_iframe_placeholder").checked ? "1" : "0"
  );
  formData.append(
    "lazy_iframe_exclude_home_cbx",
    document.getElementById("lazy_iframe_exclude_home_cbx").checked ? "1" : "0"
  );
  formData.append(
    "lazyloaded_iframe_visually_mark",
    document.getElementById("lazyloaded_iframe_visually_mark").checked ? "1" : "0"
  );


  /// Videos ///
  formData.append(
    "enable_lazy_load_videos",
    document.getElementById("enable_lazy_load_videos").checked ? "1" : "0"
  );
  formData.append(
    "lazy_video_thumb_preload_selector",
    document.getElementById("lazy_video_thumb_preload_selector").checked ? "1" : "0"
  );
  formData.append(
    "lazy_video_thumb_preload_selector_textarea",
    document.getElementById("lazy_video_thumb_preload_selector_textarea").value
  );
  formData.append(
    "lazy_video_thumb_preload_parent_selector",
    document.getElementById("lazy_video_thumb_preload_parent_selector").checked ? "1" : "0"
  );
  formData.append(
    "lazy_video_thumb_preload_parent_selector_textarea",
    document.getElementById("lazy_video_thumb_preload_parent_selector_textarea").value
  );
  formData.append(
    "lazy_video_exclude_pages_cbx",
    document.getElementById("lazy_video_exclude_pages_cbx").checked ? "1" : "0"
  );
  formData.append(
    "lazy_video_exclude_pages_textarea",
    document.getElementById("lazy_video_exclude_pages_textarea").value
  );
  formData.append(
    "lazy_video_exclude_home_cbx",
    document.getElementById("lazy_video_exclude_home_cbx").checked ? "1" : "0"
  );
  formData.append(
    "lazy_video_exclude_selector",
    document.getElementById("lazy_video_exclude_selector").checked ? "1" : "0"
  );
  formData.append(
    "lazy_video_exclude_selector_textarea",
    document.getElementById("lazy_video_exclude_selector_textarea").value
  );
  formData.append(
    "lazy_video_exclude_parent_selector",
    document.getElementById("lazy_video_exclude_parent_selector").checked ? "1" : "0"
  );
  formData.append(
    "lazy_video_exclude_parent_selector_textarea",
    document.getElementById("lazy_video_exclude_parent_selector_textarea").value
  );
  formData.append(
    "visually_mark_non_embeded_yt",
    document.getElementById("visually_mark_non_embeded_yt").checked ? "1" : "0"
  );
  formData.append(
    "preloaded_thumbnail_visually_mark",
    document.getElementById("preloaded_thumbnail_visually_mark").checked ? "1" : "0"
  );
  formData.append(
    "lazyloaded_video_visually_mark_js",
    document.getElementById("lazyloaded_video_visually_mark_js").checked ? "1" : "0"
  );

  /// Preload Links ///
  formData.append(
    "enable_preload_links",
    document.getElementById("enable_preload_links").checked ? "1" : "0"
  );
  formData.append(
    "preload_links_enable_mobile",
    document.getElementById("preload_links_enable_mobile").checked ? "1" : "0"
  );
  formData.append(
    "preload_links_mobile_width",
    document.getElementById("preload_links_mobile_width").value
  );
  formData.append(
    "preload_links_max_retries",
    document.getElementById("preload_links_max_retries").value
  );
  formData.append(
    "preload_links_retry_delay",
    document.getElementById("preload_links_retry_delay").value
  );

  formData.append(
    "preload_stylesheets_desktop",
    document.getElementById("preload_stylesheets_desktop").checked ? "1" : "0"
  );
  formData.append(
    "preload_javaScript_desktop",
    document.getElementById("preload_javaScript_desktop").checked ? "1" : "0"
  );
  formData.append(
    "preload_images_desktop",
    document.getElementById("preload_images_desktop").checked ? "1" : "0"
  );
  formData.append(
    "preload_links_fetch_timeout_desktop",
    document.getElementById("preload_links_fetch_timeout_desktop").value
  );
  formData.append(
    "preload_links_server_response_desktop",
    document.getElementById("preload_links_server_response_desktop").value
  );

  formData.append(
    "preload_links_for_logged_in_users",
    document.getElementById("preload_links_for_logged_in_users").checked ? "1" : "0"
  );
  formData.append(
    "preload_links_test_mode",
    document.getElementById("preload_links_test_mode").checked ? "1" : "0"
  );
  formData.append(
    "preload_stylesheets_mobile",
    document.getElementById("preload_stylesheets_mobile").checked ? "1" : "0"
  );
  formData.append(
    "preload_javaScript_mobile",
    document.getElementById("preload_javaScript_mobile").checked ? "1" : "0"
  );
  formData.append(
    "preload_images_mobile",
    document.getElementById("preload_images_mobile").checked ? "1" : "0"
  );
  formData.append(
    "preload_links_initial_delay_mobile",
    document.getElementById("preload_links_initial_delay_mobile").value
  );
  formData.append(
    "preload_links_next_file_delay_mobile",
    document.getElementById("preload_links_next_file_delay_mobile").value
  );
  formData.append(
    "preload_links_fetch_timeout_mobile",
    document.getElementById("preload_links_fetch_timeout_mobile").value
  );
  formData.append(
    "preload_links_server_response_mobile",
    document.getElementById("preload_links_server_response_mobile").value
  );

  formData.append(
    "preload_links_color_coding_enable",
    document.getElementById("preload_links_color_coding_enable").checked ? "1" : "0"
  );
  formData.append(
    "preload_links_color_coding_visibility",
    document.getElementById("preload_links_color_coding_visibility").value
  );


  // Critical CSS - Append repeater fields data to formData
/*
  const repeaterFieldsData = Array.from(
    document.querySelectorAll("#css-repeater-fields .css-repeater")
  );
  repeaterFieldsData.forEach((item, index) => {
    // iterate over all the radio buttons in the repeater field and get the value of the one that's checked
    let radioValue;
    const radios = item.querySelectorAll(".critical_css_repeater_radio");
    radios.forEach((radio) => {
      if (radio.checked) {
        radioValue = radio.value;
      }
    });

    // Get the values of the text areas
    const excludeLinksTextarea = item.querySelector(
      ".critical_delay_exclude_links_css_textarea"
    );
    const specificLinksTextarea = item.querySelector(
      ".critical_delay_specific_links_css_textarea"
    );

    // Check if the textareas exist before trying to access their value property
    const critical_exclude_links = excludeLinksTextarea
      ? excludeLinksTextarea.value
      : "";
    const critical_delay_specific_links = specificLinksTextarea
      ? specificLinksTextarea.value
      : "";


    const fieldData = {
      critical_css_page_template: item.querySelector(".css-dropdown").value,
      critical_css_page_url: item.querySelector(".css-page-string").value,
      critical_css: item.querySelector(".critical-css").value,
      critical_css_radio: radioValue,
      critical_exclude_links: critical_exclude_links,
      critical_delay_specific_links: critical_delay_specific_links,
    };

    formData.append(`repeater_field_${index}`, JSON.stringify(fieldData));
  });
*/

  // log the formData to ensure it is correctly populated
  for (let pair of formData.entries()) {
    console.log(pair[0] + ", " + pair[1]);
  }

  return formData;
}


