<?php

namespace SuperSpeed\Admin;

use SuperSpeed\Utility;

class Settings
{
  private $main_plugin_file;

  public function __construct($main_plugin_file)
  {
    // Exit if we could not get the current URL. Skipping static resource because it is not a page.
    $current_url = Utility::get_current_page_url();
    if (!$current_url) {
      return;
    }

    // Register the settings
    add_action('admin_init', array($this, 'register_settings'));

    // Save setting
    add_action('wp_ajax_save_wp_ss', array($this, 'save_settings'));

    // register a new REST API route to use saved settings in JavaScript
    add_action('rest_api_init', array($this, 'register_routes'));

    // Add link to settings page in plugin list, next to Activate / Deactivate
    $this->main_plugin_file = $main_plugin_file;
    add_filter('plugin_action_links_' . plugin_basename($this->main_plugin_file), array($this, 'add_settings_link'));
  }

  public function add_settings_link($links)
  {
    $settings_link = '<a href="options-general.php?page=' . \SuperSpeed\Config::PLUGIN_SLUG . '">' . __('Settings', 'wpssxtdomain') . '</a>';
    array_push($links, $settings_link);
    return $links;
  }

  public function register_settings()
  {
    $settings = [
      /// JS ///

      // Dequeue Scripts
      'ss_enable_dequeue' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'ss_dequeue_js_links_radio_checked' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => 'dequeue_all_links_js'],
      'ss_dequeue_exclude_links_js_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'ss_dequeue_spacific_links_js_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'ss_deq_radio_checked' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => 'dequeue_all_pages'],
      'ss_exc_deq_js_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'ss_deq_spec_js_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'ss_dequeue_js_exclude_home' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'ss_dequeue_js_include_home' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'dequeue_scripts_exclude_3rd_party' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],

      // Defer scripts
      'ss_enable_defer_ext_js' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'ss_defer_radio_checked' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => 'defer_all_scripts'],
      'ss_exc_defer_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'ss_defer_specific_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'ss_defer_exclude_dependent' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'ss_defer_exclude_support' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],

      // Delay scripts
      'ss_enable_delay_ext_js' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'ss_delay_js_links_radio_checked' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => 'delay_ext_all_links_radio'],
      'ss_delay_ext_exclude_links_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'ss_delay_ext_specific_links_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'ss_delay_ext_pages_radio_checked' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => 'delay_ext_all_pages_radio'],
      'ss_delay_ext_excl_pages_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'ss_delay_ext_specific_pages_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'ss_delay_js_exclude_home' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'ss_delay_js_include_home' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],

      /// CSS ///

      // Preload CSS
      'preload_styles' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],

      // Used CSS
      'enable_used_css' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'used_css_file_info' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'used_css_different_post_layout' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'used_css_different_cpt_layout' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'load_used_css_on_pages_radio' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => 'used_css_load_all_pages'],
      'used_css_exclude_home' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'used_css_exclude_pages_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'used_css_include_home' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'used_css_spacific_pages_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],

      // Dequeue CSS
      'enable_dequeue_styles' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'dequeue_css_links_radio_checked' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => 'dequeue_all_links_css'],
      'dequeue_exclude_links_css_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'dequeue_spacific_links_css_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'remove_pages_css_radio_checked' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => 'dequeue_all_pages_css'],
      'dequeue_exclude_pages_css_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'dequeue_spacific_pages_css_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'dequeue_css_exclude_home' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'dequeue_css_include_home' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'dequeue_css_exclude_3rd_party' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'used_css_exclude_google' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],

      // Delay CSS
      'enable_ext_delay_css' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'delay_css_links_radio_checked' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => 'delay_all_links_css'],
      'delay_exclude_links_css_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'delay_spacific_links_css_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'delay_css_pages_radio_checked' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => 'delay_all_pages_css_radio'],
      'delay_exclude_pages_css_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'delay_specific_pages_css_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'delay_css_exclude_home' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'delay_css_include_home' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],

      // Critical CSS repeater
      'critical_css_enable' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],


      /// IMAGE DATA COLLECTION ///
      'image_data_collection' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'image_data_collection_exclude_home' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'image_data_collection_exclude_pages_cbx' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'image_data_collection_exclude_pages_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'image_data_collection_exclude_posts' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'image_data_collection_exclude_products' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'image_data_collection_exclude_cpts' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'image_data_collection_exclude_cpts_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],

      /// LAZY LOAD ///

      // Images
      'lazy_load_images' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_load_images_checkbox' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazyloaded_img_visually_mark' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],

      'lazy_img_fadein_exclude_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_img_fadein_exclude_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lazy_img_fadein_exclude_parent_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_img_fadein_exclude_parent_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],

      'img_first_x_exlude_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'img_first_x_exlude_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'img_first_x_exlude_parent_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'img_first_x_exlude_selector_parent_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'preloaded_img_visually_mark' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],

      'lazy_img_exclude_home_cbx' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_img_include_home_cbx' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'img_first_x_select_pages_radio' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => 'img_first_x_all_pages'],
      'img_first_x_exlude_pages_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'img_first_x_include_pages_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],

      'lazy_img_fadein' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_img_fadein_duration' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '1'],

      'image_auto_set_thumbnails' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'image_auto_set_fixed_pixels_desktop' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '1.5'],
      'image_auto_set_fixed_pixels_tablet' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '2'],
      'image_auto_set_fixed_pixels_mobile' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '2.5'],
      'image_auto_set_exclude_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'image_auto_set_exclude_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'image_auto_set_exclude_parent_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'image_auto_set_exclude_parent_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'image_auto_set_thumbs_info' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],

      'lazy_image_exclude_home' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_image_exclude_pages_cbx' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_image_exclude_pages_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lazy_image_exclude_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_image_exclude_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lazy_image_exclude_parent_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_image_exclude_parent_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],

      'imagesPlaceholderRadio' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => 'lazy_image_placeholder_gray'],

      'lazy_img_placeholder_exclude_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_img_placeholder_exclude_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lazy_img_placeholder_exclude_parent_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_img_placeholder_exclude_parent_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],

      'img_load_smallest_thumbnail_exclude_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'img_load_smallest_thumbnail_exclude_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'img_load_smallest_thumbnail_exclude_parent_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'img_load_smallest_thumbnail_exclude_parent_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],

      'lazy_img_preload_first_enable' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_img_preload_first_x_select' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '1'],

      'lazy_img_preload_selector_enable' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_featured_img_blog' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_featured_img_product' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_img_preload_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_img_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lazy_img_preload_parent_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_img_parent_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],

      // Background Images
      'lazy_load_background' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_background_checkbox' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_background_exclude_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_background_exclude_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lazy_background_exclude_parent_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_background_exclude_parent_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lazy_background_exclude_pages_cbx' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_background_exclude_pages_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lazy_background_exclude_home' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_bg_fadein' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_bg_fadein_duration' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '3'],
      'preloaded_bg_visually_mark' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preloaded_bg_lazyloaded_mark' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_background_images' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_background_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_background_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'preload_background_parent_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_background_parent_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'back_img_placeholder_radio' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => 'back_img_placeholder_gray'],

      'background_auto_set_thumbnails' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'background_auto_set_thumbs_info' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'image_custom_pixel_density_desktop' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'image_custom_pixel_density_tablet' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'image_custom_pixel_density_mobile' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'background_custom_pixel_density_tablet' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'background_custom_pixel_density_mobile' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'backgound_auto_set_fixed_pixels_desktop' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '1.5'],
      'backgound_auto_set_fixed_pixels_tablet' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '2'],
      'backgound_auto_set_fixed_pixels_mobile' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '2.5'],
      'backgound_auto_set_exclude_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'backgound_auto_set_exclude_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'backgound_auto_set_exclude_parent_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'backgound_auto_set_exclude_parent_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],

      // LCP
      'lcp_enable' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'visually_mark_lcp' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'display_lcp_info' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'auto_preload_lcp' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lcp_prelod_exclude_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lcp_prelod_exclude_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lcp_prelod_exclude_parent_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lcp_prelod_exclude_parent_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lcp_prelod_exclude_home_cbx' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lcp_prelod_exclude_pages_cbx' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lcp_prelod_exclude_pages_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lcp_prelod_exclude_posts' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lcp_prelod_exclude_products' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lcp_prelod_exclude_cpts' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lcp_prelod_exclude_pages_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],

      // iFrames
      'lazy_load_iframes' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_iframe_exclude_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_iframe_exclude_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lazy_iframe_exclude_parent_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_iframe_exclude_parent_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lazy_iframe_exclude_pages_cbx' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_iframe_exclude_pages_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lazy_iframe_placeholder' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_iframe_exclude_home_cbx' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazyloaded_iframe_visually_mark' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],

      // Videos
      'enable_lazy_load_videos' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_video_thumb_preload_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_video_thumb_preload_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lazy_video_thumb_preload_parent_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_video_thumb_preload_parent_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lazy_video_exclude_pages_cbx' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_video_exclude_pages_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lazy_video_exclude_home_cbx' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_video_exclude_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_video_exclude_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'lazy_video_exclude_parent_selector' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazy_video_exclude_parent_selector_textarea' => ['type' => 'string', 'sanitize_callback' => 'sanitize_textarea_field', 'default' => ''],
      'visually_mark_non_embeded_yt' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preloaded_thumbnail_visually_mark' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'lazyloaded_video_visually_mark_js' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],

      // Preload Links
      'enable_preload_links' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_links_enable_mobile' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_links_mobile_width' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '800'],
      'preload_links_max_retries' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '3'],
      'preload_links_retry_delay' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '1'],

      'preload_stylesheets_desktop' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_javaScript_desktop' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_images_desktop' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_links_fetch_timeout_desktop' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '5'],
      'preload_links_server_response_desktop' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '250'],

      'preload_links_for_logged_in_users' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_links_test_mode' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_stylesheets_mobile' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_javaScript_mobile' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_images_mobile' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_links_initial_delay_mobile' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '5'],
      'preload_links_next_file_delay_mobile' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '5'],
      'preload_links_fetch_timeout_mobile' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '7'],
      'preload_links_server_response_mobile' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '250'],

      'preload_links_color_coding_enable' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => '0'],
      'preload_links_color_coding_visibility' => ['type' => 'string', 'sanitize_callback' => 'sanitize_text_field', 'default' => 'editing_permissions'],

    ];



    foreach ($settings as $option_name => $args) {
      register_setting('wp_ss-group', $option_name, $args);
      if (false === get_option($option_name)) {
        add_option($option_name, $args['default'], '', 'no');
      }
    }
  }




  /*
  // Critial CSS repeater sanitization
  public function sanitize_critical_css_repeater_data($data)
  {
    return array_map(function ($item) {
      return [
        'critical_css_page_template' => sanitize_text_field($item['critical_css_page_template']),
        'field' => sanitize_text_field($item['field']),
        'textarea' => sanitize_textarea_field($item['textarea']),
      ];
    }, $data);
  }
*/


  public function save_settings()
  {
    if (!current_user_can('manage_options')) {
      wp_die(__('You do not have sufficient permissions to access this page.', 'wpssxtdomain'));
    }

    check_ajax_referer(\SuperSpeed\Config::PLUGIN_SLUG, 'security');

    // Declaring variables that are used for Lazy Loading of imgaes and fixing CLS
    $lazy_img_preload_first_x_select = intval(filter_input(INPUT_POST, 'lazy_img_preload_first_x_select', FILTER_DEFAULT) ?: '1');
    $current_fadein_value = floatval(filter_input(INPUT_POST, 'lazy_img_fadein_duration', FILTER_DEFAULT) ?: '1');
    $current_fadein_value_bg = floatval(filter_input(INPUT_POST, 'lazy_bg_fadein_duration', FILTER_DEFAULT) ?: '1');

    $preload_links_mobile_width = floatval(filter_input(INPUT_POST, 'preload_links_mobile_width', FILTER_DEFAULT) ?: '800');
    $preload_links_max_retries = floatval(filter_input(INPUT_POST, 'preload_links_max_retries', FILTER_DEFAULT) ?: '3');
    $preload_links_retry_delay = floatval(filter_input(INPUT_POST, 'preload_links_retry_delay', FILTER_DEFAULT) ?: '1');
    $preload_links_fetch_timeout_desktop = floatval(filter_input(INPUT_POST, 'preload_links_fetch_timeout_desktop', FILTER_DEFAULT) ?: '5');
    $preload_links_server_response_desktop = floatval(filter_input(INPUT_POST, 'preload_links_server_response_desktop', FILTER_DEFAULT) ?: '250');
    $preload_links_initial_delay_mobile = floatval(filter_input(INPUT_POST, 'preload_links_initial_delay_mobile', FILTER_DEFAULT) ?: '5');
    $preload_links_next_file_delay_mobile = floatval(filter_input(INPUT_POST, 'preload_links_next_file_delay_mobile', FILTER_DEFAULT) ?: '5');
    $preload_links_fetch_timeout_mobile = floatval(filter_input(INPUT_POST, 'preload_links_fetch_timeout_mobile', FILTER_DEFAULT) ?: '7');
    $preload_links_server_response_mobile = floatval(filter_input(INPUT_POST, 'preload_links_server_response_mobile', FILTER_DEFAULT) ?: '250');
    $preload_links_color_coding_visibility = filter_input(INPUT_POST, 'preload_links_color_coding_visibility', FILTER_UNSAFE_RAW) ?: 'editing_permissions';

    $image_auto_set_fixed_pixels_desktop = floatval(filter_input(INPUT_POST, 'image_auto_set_fixed_pixels_desktop', FILTER_DEFAULT) ?: '1.5');
    $image_auto_set_fixed_pixels_tablet = floatval(filter_input(INPUT_POST, 'image_auto_set_fixed_pixels_tablet', FILTER_DEFAULT) ?: '2');
    $image_auto_set_fixed_pixels_mobile = floatval(filter_input(INPUT_POST, 'image_auto_set_fixed_pixels_mobile', FILTER_DEFAULT) ?: '2.5');

    $backgound_auto_set_fixed_pixels_desktop = floatval(filter_input(INPUT_POST, 'backgound_auto_set_fixed_pixels_desktop', FILTER_DEFAULT) ?: '1.5');
    $backgound_auto_set_fixed_pixels_tablet = floatval(filter_input(INPUT_POST, 'backgound_auto_set_fixed_pixels_tablet', FILTER_DEFAULT) ?: '2');
    $backgound_auto_set_fixed_pixels_mobile = floatval(filter_input(INPUT_POST, 'backgound_auto_set_fixed_pixels_mobile', FILTER_DEFAULT) ?: '2.5');

    // Define an associative array to map option names to POST variables and sanitization functions
    $options = [

      /// JS ///

      // Dequeue Scripts
      'ss_enable_dequeue' => [filter_input(INPUT_POST, 'ss_enable_dequeue', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'ss_dequeue_js_links_radio_checked' => [sanitize_text_field(filter_input(INPUT_POST, 'ss_dequeue_js_links_radio_checked', FILTER_DEFAULT)) ?: 'dequeue_all_links_js', 'sanitize_text_field'],
      'ss_dequeue_exclude_links_js_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'ss_dequeue_exclude_links_js_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'ss_dequeue_spacific_links_js_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'ss_dequeue_spacific_links_js_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'ss_deq_radio_checked' => [sanitize_text_field(filter_input(INPUT_POST, 'ss_deq_radio_checked', FILTER_DEFAULT)) ?: 'dequeue_all_pages', 'sanitize_text_field'],
      'ss_exc_deq_js_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'ss_exc_deq_js_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'ss_deq_spec_js_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'ss_deq_spec_js_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'ss_dequeue_js_exclude_home' => [filter_input(INPUT_POST, 'ss_dequeue_js_exclude_home', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'ss_dequeue_js_include_home' => [filter_input(INPUT_POST, 'ss_dequeue_js_include_home', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'dequeue_scripts_exclude_3rd_party' => [filter_input(INPUT_POST, 'dequeue_scripts_exclude_3rd_party', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],

      // Defer Scripts
      'ss_enable_defer_ext_js' => [filter_input(INPUT_POST, 'ss_enable_defer_ext_js', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'ss_defer_radio_checked' => [filter_input(INPUT_POST, 'ss_defer_radio_checked', FILTER_DEFAULT) ?: 'defer_all_scripts', 'sanitize_text_field'],
      'ss_exc_defer_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'ss_exc_defer_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'ss_defer_specific_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'ss_defer_specific_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'ss_defer_exclude_dependent' => [filter_input(INPUT_POST, 'ss_defer_exclude_dependent', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'ss_defer_exclude_support' => [filter_input(INPUT_POST, 'ss_defer_exclude_support', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],

      // Delay Scripts
      'ss_enable_delay_ext_js' => [filter_input(INPUT_POST, 'ss_enable_delay_ext_js', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'ss_delay_js_links_radio_checked' => [sanitize_text_field(filter_input(INPUT_POST, 'ss_delay_js_links_radio_checked', FILTER_DEFAULT)) ?: 'delay_ext_all_links_radio', 'sanitize_text_field'],
      'ss_delay_ext_exclude_links_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'ss_delay_ext_exclude_links_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'ss_delay_ext_specific_links_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'ss_delay_ext_specific_links_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'ss_delay_ext_pages_radio_checked' => [sanitize_text_field(filter_input(INPUT_POST, 'ss_delay_ext_pages_radio_checked', FILTER_DEFAULT)) ?: 'delay_ext_all_pages_radio', 'sanitize_text_field'],
      'ss_delay_ext_excl_pages_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'ss_delay_ext_excl_pages_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'ss_delay_ext_specific_pages_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'ss_delay_ext_specific_pages_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'ss_delay_js_exclude_home' => [filter_input(INPUT_POST, 'ss_delay_js_exclude_home', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'ss_delay_js_include_home' => [filter_input(INPUT_POST, 'ss_delay_js_include_home', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],

      /// CSS ///

      // Preload CSS
      'preload_styles' => [filter_input(INPUT_POST, 'preload_styles', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],

      // Used CSS 
      'enable_used_css' => [filter_input(INPUT_POST, 'enable_used_css', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'used_css_file_info' => [filter_input(INPUT_POST, 'used_css_file_info', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'used_css_different_post_layout' => [filter_input(INPUT_POST, 'used_css_different_post_layout', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'used_css_different_cpt_layout' => [filter_input(INPUT_POST, 'used_css_different_cpt_layout', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'load_used_css_on_pages_radio' => [sanitize_text_field(filter_input(INPUT_POST, 'load_used_css_on_pages_radio', FILTER_DEFAULT)) ?: 'used_css_load_all_pages', 'sanitize_text_field'],
      'used_css_exclude_home' => [filter_input(INPUT_POST, 'used_css_exclude_home', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'used_css_exclude_pages_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'used_css_exclude_pages_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'used_css_include_home' => [filter_input(INPUT_POST, 'used_css_include_home', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'used_css_spacific_pages_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'used_css_spacific_pages_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],

      // Dequeue CSS
      'enable_dequeue_styles' => [filter_input(INPUT_POST, 'enable_dequeue_styles', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'dequeue_css_links_radio_checked' => [sanitize_text_field(filter_input(INPUT_POST, 'dequeue_css_links_radio_checked', FILTER_DEFAULT)) ?: 'dequeue_all_links_css', 'sanitize_text_field'],
      'dequeue_exclude_links_css_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'dequeue_exclude_links_css_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'dequeue_spacific_links_css_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'dequeue_spacific_links_css_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'remove_pages_css_radio_checked' => [sanitize_text_field(filter_input(INPUT_POST, 'remove_pages_css_radio_checked', FILTER_DEFAULT)) ?: 'dequeue_all_pages_css', 'sanitize_text_field'],
      'dequeue_exclude_pages_css_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'dequeue_exclude_pages_css_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'dequeue_spacific_pages_css_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'dequeue_spacific_pages_css_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'dequeue_css_exclude_home' => [filter_input(INPUT_POST, 'dequeue_css_exclude_home', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'dequeue_css_include_home' => [filter_input(INPUT_POST, 'dequeue_css_include_home', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'dequeue_css_exclude_3rd_party' => [filter_input(INPUT_POST, 'dequeue_css_exclude_3rd_party', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'used_css_exclude_google' => [filter_input(INPUT_POST, 'used_css_exclude_google', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],

      // Delay CSS
      'enable_ext_delay_css' => [filter_input(INPUT_POST, 'enable_ext_delay_css', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'delay_css_links_radio_checked' => [sanitize_text_field(filter_input(INPUT_POST, 'delay_css_links_radio_checked', FILTER_DEFAULT)) ?: 'delay_all_links_css', 'sanitize_text_field'],
      'delay_exclude_links_css_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'delay_exclude_links_css_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'delay_spacific_links_css_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'delay_spacific_links_css_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'delay_css_pages_radio_checked' => [sanitize_text_field(filter_input(INPUT_POST, 'delay_css_pages_radio_checked', FILTER_DEFAULT)) ?: 'delay_all_pages_css_radio', 'sanitize_text_field'],
      'delay_exclude_pages_css_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'delay_exclude_pages_css_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'delay_specific_pages_css_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'delay_specific_pages_css_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'delay_css_exclude_home' => [filter_input(INPUT_POST, 'delay_css_exclude_home', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'delay_css_include_home' => [filter_input(INPUT_POST, 'delay_css_include_home', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],

      // Critical CSS
      'critical_css_enable' => [filter_input(INPUT_POST, 'critical_css_enable', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],


      /// IMAGE DATA COLLECTION ///

      'image_data_collection' => [filter_input(INPUT_POST, 'image_data_collection', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'image_data_collection_exclude_home' => [filter_input(INPUT_POST, 'image_data_collection_exclude_home', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'image_data_collection_exclude_pages_cbx' => [filter_input(INPUT_POST, 'image_data_collection_exclude_pages_cbx', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'image_data_collection_exclude_pages_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'image_data_collection_exclude_pages_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'image_data_collection_exclude_posts' => [filter_input(INPUT_POST, 'image_data_collection_exclude_posts', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'image_data_collection_exclude_products' => [filter_input(INPUT_POST, 'image_data_collection_exclude_products', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'image_data_collection_exclude_cpts' => [filter_input(INPUT_POST, 'image_data_collection_exclude_cpts', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'image_data_collection_exclude_cpts_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'image_data_collection_exclude_cpts_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],


      /// LAZY LOAD ///

      // Image Data Collection
      'image_data_collection' => [filter_input(INPUT_POST, 'image_data_collection', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],

      // Images
      'lazy_load_images' => [filter_input(INPUT_POST, 'lazy_load_images', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_img_fadein' => [filter_input(INPUT_POST, 'lazy_img_fadein', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_img_fadein_duration' => [$current_fadein_value, 'floatval'],

      'lazy_img_fadein_exclude_selector' => [filter_input(INPUT_POST, 'lazy_img_fadein_exclude_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_img_fadein_exclude_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_img_fadein_exclude_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lazy_img_fadein_exclude_parent_selector' => [filter_input(INPUT_POST, 'lazy_img_fadein_exclude_parent_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_img_fadein_exclude_parent_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_img_fadein_exclude_parent_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],

      'img_first_x_exlude_selector' => [filter_input(INPUT_POST, 'img_first_x_exlude_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'img_first_x_exlude_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'img_first_x_exlude_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'img_first_x_exlude_parent_selector' => [filter_input(INPUT_POST, 'img_first_x_exlude_parent_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'img_first_x_exlude_selector_parent_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'img_first_x_exlude_selector_parent_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'preloaded_img_visually_mark' => [filter_input(INPUT_POST, 'preloaded_img_visually_mark', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazyloaded_img_visually_mark' => [filter_input(INPUT_POST, 'lazyloaded_img_visually_mark', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_load_images_checkbox' => [filter_input(INPUT_POST, 'lazy_load_images_checkbox', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],

      'lazy_img_include_home_cbx' => [filter_input(INPUT_POST, 'lazy_img_include_home_cbx', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'img_first_x_select_pages_radio' => [sanitize_text_field(filter_input(INPUT_POST, 'img_first_x_select_pages_radio', FILTER_DEFAULT)) ?: 'img_first_x_all_pages', 'sanitize_text_field'],
      'lazy_img_include_home_cbx' => [filter_input(INPUT_POST, 'lazy_img_include_home_cbx', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'img_first_x_exlude_pages_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'img_first_x_exlude_pages_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lazy_img_exclude_home_cbx' => [filter_input(INPUT_POST, 'lazy_img_exclude_home_cbx', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'img_first_x_include_pages_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'img_first_x_include_pages_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],

      'image_auto_set_thumbnails' => [filter_input(INPUT_POST, 'image_auto_set_thumbnails', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'image_auto_set_thumbs_info' => [filter_input(INPUT_POST, 'image_auto_set_thumbs_info', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'image_custom_pixel_density_desktop' => [filter_input(INPUT_POST, 'image_custom_pixel_density_desktop', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'image_custom_pixel_density_tablet' => [filter_input(INPUT_POST, 'image_custom_pixel_density_tablet', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'image_custom_pixel_density_mobile' => [filter_input(INPUT_POST, 'image_custom_pixel_density_mobile', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'image_auto_set_fixed_pixels_desktop' => [$image_auto_set_fixed_pixels_desktop, 'floatval'],
      'image_auto_set_fixed_pixels_tablet' => [$image_auto_set_fixed_pixels_tablet, 'floatval'],
      'image_auto_set_fixed_pixels_mobile' => [$image_auto_set_fixed_pixels_mobile, 'floatval'],
      'image_auto_set_exclude_selector' => [filter_input(INPUT_POST, 'image_auto_set_exclude_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'image_auto_set_exclude_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'image_auto_set_exclude_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'image_auto_set_exclude_parent_selector' => [filter_input(INPUT_POST, 'image_auto_set_exclude_parent_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'image_auto_set_exclude_parent_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'image_auto_set_exclude_parent_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],

      'lazy_image_exclude_home' => [filter_input(INPUT_POST, 'lazy_image_exclude_home', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_image_exclude_pages_cbx' => [filter_input(INPUT_POST, 'lazy_image_exclude_pages_cbx', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_image_exclude_pages_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_image_exclude_pages_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lazy_image_exclude_selector' => [filter_input(INPUT_POST, 'lazy_image_exclude_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_image_exclude_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_image_exclude_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lazy_image_exclude_parent_selector' => [filter_input(INPUT_POST, 'lazy_image_exclude_parent_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_image_exclude_parent_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_image_exclude_parent_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],

      'imagesPlaceholderRadio' => [sanitize_text_field(filter_input(INPUT_POST, 'imagesPlaceholderRadio', FILTER_DEFAULT)) ?: 'lazy_image_placeholder_gray', 'sanitize_text_field'],
      'lazy_img_placeholder_exclude_selector' => [filter_input(INPUT_POST, 'lazy_img_placeholder_exclude_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_img_placeholder_exclude_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_img_placeholder_exclude_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lazy_img_placeholder_exclude_parent_selector' => [filter_input(INPUT_POST, 'lazy_img_placeholder_exclude_parent_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_img_placeholder_exclude_parent_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_img_placeholder_exclude_parent_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],

      'img_load_smallest_thumbnail_exclude_selector' => [filter_input(INPUT_POST, 'img_load_smallest_thumbnail_exclude_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'img_load_smallest_thumbnail_exclude_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'img_load_smallest_thumbnail_exclude_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'img_load_smallest_thumbnail_exclude_parent_selector' => [filter_input(INPUT_POST, 'img_load_smallest_thumbnail_exclude_parent_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'img_load_smallest_thumbnail_exclude_parent_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'img_load_smallest_thumbnail_exclude_parent_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],

      'lazy_img_preload_first_enable' => [filter_input(INPUT_POST, 'lazy_img_preload_first_enable', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_img_preload_first_x_select' => [$lazy_img_preload_first_x_select, 'floatval'],

      'lazy_img_preload_selector_enable' => [filter_input(INPUT_POST, 'lazy_img_preload_selector_enable', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preload_featured_img_blog' => [filter_input(INPUT_POST, 'preload_featured_img_blog', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preload_featured_img_product' => [filter_input(INPUT_POST, 'preload_featured_img_product', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_img_preload_selector' => [filter_input(INPUT_POST, 'lazy_img_preload_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_img_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_img_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lazy_img_preload_parent_selector' => [filter_input(INPUT_POST, 'lazy_img_preload_parent_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_img_parent_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_img_parent_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],

      /// Background Images ///
      'lazy_load_background' => [filter_input(INPUT_POST, 'lazy_load_background', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_background_checkbox' => [filter_input(INPUT_POST, 'lazy_background_checkbox', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_background_exclude_pages_cbx' => [filter_input(INPUT_POST, 'lazy_background_exclude_pages_cbx', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_background_exclude_pages_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_background_exclude_pages_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],

      'lazy_background_exclude_selector' => [filter_input(INPUT_POST, 'lazy_background_exclude_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_background_exclude_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_background_exclude_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lazy_background_exclude_parent_selector' => [filter_input(INPUT_POST, 'lazy_background_exclude_parent_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_background_exclude_parent_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_background_exclude_parent_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],

      'lazy_background_exclude_home' => [filter_input(INPUT_POST, 'lazy_background_exclude_home', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_bg_fadein' => [filter_input(INPUT_POST, 'lazy_bg_fadein', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_bg_fadein_duration' => [$current_fadein_value_bg, 'floatval'],
      'preloaded_bg_visually_mark' => [filter_input(INPUT_POST, 'preloaded_bg_visually_mark', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preloaded_bg_lazyloaded_mark' => [filter_input(INPUT_POST, 'preloaded_bg_lazyloaded_mark', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],

      'preload_background_images' => [filter_input(INPUT_POST, 'preload_background_images', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preload_background_selector' => [filter_input(INPUT_POST, 'preload_background_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preload_background_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'preload_background_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'preload_background_parent_selector' => [filter_input(INPUT_POST, 'preload_background_parent_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preload_background_parent_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'preload_background_parent_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'back_img_placeholder_radio' => [sanitize_text_field(filter_input(INPUT_POST, 'back_img_placeholder_radio', FILTER_DEFAULT)) ?: 'back_img_placeholder_gray', 'sanitize_text_field'],

      'background_auto_set_thumbnails' => [filter_input(INPUT_POST, 'background_auto_set_thumbnails', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'background_auto_set_thumbs_info' => [filter_input(INPUT_POST, 'background_auto_set_thumbs_info', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'background_custom_pixel_density_desktop' => [filter_input(INPUT_POST, 'background_custom_pixel_density_desktop', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'background_custom_pixel_density_tablet' => [filter_input(INPUT_POST, 'background_custom_pixel_density_tablet', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'background_custom_pixel_density_mobile' => [filter_input(INPUT_POST, 'background_custom_pixel_density_mobile', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'backgound_auto_set_fixed_pixels_desktop' => [$backgound_auto_set_fixed_pixels_desktop, 'floatval'],
      'backgound_auto_set_fixed_pixels_tablet' => [$backgound_auto_set_fixed_pixels_tablet, 'floatval'],
      'backgound_auto_set_fixed_pixels_mobile' => [$backgound_auto_set_fixed_pixels_mobile, 'floatval'],
      'backgound_auto_set_exclude_selector' => [filter_input(INPUT_POST, 'backgound_auto_set_exclude_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'backgound_auto_set_exclude_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'backgound_auto_set_exclude_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'backgound_auto_set_exclude_parent_selector' => [filter_input(INPUT_POST, 'backgound_auto_set_exclude_parent_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'backgound_auto_set_exclude_parent_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'backgound_auto_set_exclude_parent_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],

      /// LCP ///
      'lcp_enable' => [filter_input(INPUT_POST, 'lcp_enable', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'visually_mark_lcp' => [filter_input(INPUT_POST, 'visually_mark_lcp', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'display_lcp_info' => [filter_input(INPUT_POST, 'display_lcp_info', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'auto_preload_lcp' => [filter_input(INPUT_POST, 'auto_preload_lcp', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lcp_prelod_exclude_selector' => [filter_input(INPUT_POST, 'lcp_prelod_exclude_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lcp_prelod_exclude_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lcp_prelod_exclude_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lcp_prelod_exclude_parent_selector' => [filter_input(INPUT_POST, 'lcp_prelod_exclude_parent_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lcp_prelod_exclude_parent_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lcp_prelod_exclude_parent_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lcp_prelod_exclude_home_cbx' => [filter_input(INPUT_POST, 'lcp_prelod_exclude_home_cbx', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lcp_prelod_exclude_pages_cbx' => [filter_input(INPUT_POST, 'lcp_prelod_exclude_pages_cbx', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lcp_prelod_exclude_pages_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lcp_prelod_exclude_pages_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lcp_prelod_exclude_posts' => [filter_input(INPUT_POST, 'lcp_prelod_exclude_posts', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lcp_prelod_exclude_products' => [filter_input(INPUT_POST, 'lcp_prelod_exclude_products', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lcp_prelod_exclude_cpts' => [filter_input(INPUT_POST, 'lcp_prelod_exclude_cpts', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lcp_prelod_exclude_cpts_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lcp_prelod_exclude_cpts_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],

      /// iFrames ///
      'lazy_load_iframes' => [filter_input(INPUT_POST, 'lazy_load_iframes', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_iframe_exclude_selector' => [filter_input(INPUT_POST, 'lazy_iframe_exclude_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_iframe_exclude_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_iframe_exclude_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lazy_iframe_exclude_parent_selector' => [filter_input(INPUT_POST, 'lazy_iframe_exclude_parent_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_iframe_exclude_parent_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_iframe_exclude_parent_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lazy_iframe_exclude_pages_cbx' => [filter_input(INPUT_POST, 'lazy_iframe_exclude_pages_cbx', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_iframe_exclude_pages_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_iframe_exclude_pages_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lazy_iframe_placeholder' => [filter_input(INPUT_POST, 'lazy_iframe_placeholder', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_iframe_exclude_home_cbx' => [filter_input(INPUT_POST, 'lazy_iframe_exclude_home_cbx', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazyloaded_iframe_visually_mark' => [filter_input(INPUT_POST, 'lazyloaded_iframe_visually_mark', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],

      /// Videos ///
      'enable_lazy_load_videos' => [filter_input(INPUT_POST, 'enable_lazy_load_videos', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_video_thumb_preload_selector' => [sanitize_text_field(filter_input(INPUT_POST, 'lazy_video_thumb_preload_selector', FILTER_DEFAULT)) ?: '0', 'sanitize_text_field'],
      'lazy_video_thumb_preload_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_video_thumb_preload_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lazy_video_thumb_preload_parent_selector' => [sanitize_text_field(filter_input(INPUT_POST, 'lazy_video_thumb_preload_parent_selector', FILTER_DEFAULT)) ?: '0', 'sanitize_text_field'],
      'lazy_video_thumb_preload_parent_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_video_thumb_preload_parent_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lazy_video_exclude_pages_cbx' => [filter_input(INPUT_POST, 'lazy_video_exclude_pages_cbx', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_video_exclude_pages_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_video_exclude_pages_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lazy_video_exclude_home_cbx' => [filter_input(INPUT_POST, 'lazy_video_exclude_home_cbx', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_video_exclude_selector' => [filter_input(INPUT_POST, 'lazy_video_exclude_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_video_exclude_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_video_exclude_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'lazy_video_exclude_parent_selector' => [filter_input(INPUT_POST, 'lazy_video_exclude_parent_selector', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazy_video_exclude_parent_selector_textarea' => [sanitize_textarea_field(filter_input(INPUT_POST, 'lazy_video_exclude_parent_selector_textarea', FILTER_DEFAULT)) ?: '', 'sanitize_textarea_field'],
      'visually_mark_non_embeded_yt' => [filter_input(INPUT_POST, 'visually_mark_non_embeded_yt', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preloaded_thumbnail_visually_mark' => [filter_input(INPUT_POST, 'preloaded_thumbnail_visually_mark', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'lazyloaded_video_visually_mark_js' => [filter_input(INPUT_POST, 'lazyloaded_video_visually_mark_js', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],

      // Preload Links
      'enable_preload_links' => [filter_input(INPUT_POST, 'enable_preload_links', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preload_links_enable_mobile' => [filter_input(INPUT_POST, 'preload_links_enable_mobile', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preload_links_mobile_width' => [$preload_links_mobile_width, 'floatval'],
      'preload_links_max_retries' => [$preload_links_max_retries, 'floatval'],
      'preload_links_retry_delay' => [$preload_links_retry_delay, 'floatval'],

      'preload_stylesheets_desktop' => [filter_input(INPUT_POST, 'preload_stylesheets_desktop', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preload_javaScript_desktop' => [filter_input(INPUT_POST, 'preload_javaScript_desktop', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preload_images_desktop' => [filter_input(INPUT_POST, 'preload_images_desktop', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preload_links_fetch_timeout_desktop' => [$preload_links_fetch_timeout_desktop, 'floatval'],
      'preload_links_server_response_desktop' => [$preload_links_server_response_desktop, 'floatval'],

      'preload_links_for_logged_in_users' => [filter_input(INPUT_POST, 'preload_links_for_logged_in_users', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preload_links_test_mode' => [filter_input(INPUT_POST, 'preload_links_test_mode', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preload_stylesheets_mobile' => [filter_input(INPUT_POST, 'preload_stylesheets_mobile', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preload_javaScript_mobile' => [filter_input(INPUT_POST, 'preload_javaScript_mobile', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preload_images_mobile' => [filter_input(INPUT_POST, 'preload_images_mobile', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preload_links_initial_delay_mobile' => [$preload_links_initial_delay_mobile, 'floatval'],
      'preload_links_next_file_delay_mobile' => [$preload_links_next_file_delay_mobile, 'floatval'],
      'preload_links_fetch_timeout_mobile' => [$preload_links_fetch_timeout_mobile, 'floatval'],
      'preload_links_server_response_mobile' => [$preload_links_server_response_mobile, 'floatval'],

      'preload_links_color_coding_enable' => [filter_input(INPUT_POST, 'preload_links_color_coding_enable', FILTER_SANITIZE_NUMBER_INT) ? '1' : '0', 'intval'],
      'preload_links_color_coding_visibility' => [$preload_links_color_coding_visibility, 'sanitize_text_field'],
    ];
    /*
    // Loop through each option and update it
    foreach ($options as $option => $new_value) {
      $sanitized_value = call_user_func($new_value[1], $new_value[0]);
      update_option($option, $sanitized_value, 'no'); // Added 'no' to avoid autoloading
    }
*/

    // Loop through each option and update it
    // Checks if the $new_value is an array with a callable second element. If it is, it calls the function. If not, it uses the value as-is.
    foreach ($options as $option => $new_value) {
      if (is_array($new_value) && isset($new_value[1]) && is_callable($new_value[1])) {
        $sanitized_value = call_user_func($new_value[1], $new_value[0]);
      } else {
        $sanitized_value = $new_value;
      }
      update_option($option, $sanitized_value, 'no'); // Added 'no' to avoid autoloading
    }


    // Handling repeater fields
    global $wpdb;
    $table_name = $wpdb->prefix . 'critical_css_repeater_fields';

    try {
      // Retrieve and sanitize the repeater data from POST
      $repeater_data = [];
      foreach ($_POST as $key => $value) {
        if (strpos($key, 'repeater_field_') === 0) { // Check if the key starts with 'repeater_field_'
          try {
            $repeater_data[] = json_decode(stripslashes($value), true);
          } catch (\Exception $e) {
            // Invalid JSON, handle the error
            error_log("Invalid JSON in $key: " . $e->getMessage());
            wp_send_json_error(array('message' => "Invalid data provided for $key. Please check the input and try again."));
            exit; // Stop execution
          }
        }
      }
      error_log(print_r($repeater_data, true));

      // Create an array to store the IDs
      $present_ids = [];

      // Loop through the sanitized repeater data
      foreach ($repeater_data as $key => $fields) {
        // Don't save the field if all values are empty
        if (
          empty($fields['critical_css_page_template'])
          && empty($fields['critical_css_page_url'])
          && empty($fields['critical_css'])
          // && empty($fields['critical_css_radio']) // this is never empty because it has a default value
          && empty($fields['critical_exclude_links'])
          && empty($fields['critical_delay_specific_links'])
        ) {
          continue;
        }
        $data = array(
          'critical_css_page_templates' => isset($fields['critical_css_page_template']) ? sanitize_text_field($fields['critical_css_page_template']) : '',
          'critical_css_page_url' => isset($fields['critical_css_page_url']) ? sanitize_text_field($fields['critical_css_page_url']) : '',
          'critical_css' => isset($fields['critical_css']) ? sanitize_textarea_field($fields['critical_css']) : '',
          'critical_css_radio' => isset($fields['critical_css_radio']) ? sanitize_text_field($fields['critical_css_radio']) : 'critical_delay_all_links_css', // Default value
          'critical_exclude_links' => isset($fields['critical_exclude_links']) ? sanitize_textarea_field($fields['critical_exclude_links']) : '',
          'critical_delay_specific_links' => isset($fields['critical_delay_specific_links']) ? sanitize_textarea_field($fields['critical_delay_specific_links']) : '',
        );

        if (isset($fields['id'])) {
          // Row exists, so update it
          $where = array('id' => $fields['id']);
          $wpdb->update($table_name, $data, $where);

          // Add the ID to the array
          $present_ids[] = $fields['id'];
        } else {
          // Row does not exist, so insert it
          $wpdb->insert($table_name, $data);

          // Add the newly created row's ID to the array
          $present_ids[] = $wpdb->insert_id;
        }
      }

      // Delete from the table, if it is removed from the frontend
      if (!empty($present_ids)) {
        $wpdb->query("DELETE FROM {$table_name} WHERE id NOT IN (" . implode(',', array_map('intval', $present_ids)) . ")");
      } else {
        // No repeater fields in POST data, so remove all rows from the table
        $wpdb->query("DELETE FROM {$table_name}");
      }

      // Respond with success
      wp_send_json_success();
    } catch (\Exception $e) {
      // Log error for debugging (during development)
      error_log($e->getMessage());

      // Respond with error
      wp_send_json_error(array('message' => 'Something went wrong, please try again later.'));
    }


    exit; // Ensure no further execution
  }



  // register a new REST API route to use saved settings in JavaScript
  public function register_routes()
  {
    register_rest_route('wp-ss/v1', '/get-delays', [
      'methods' => \WP_REST_Server::READABLE,
      'callback' => array($this, 'get_delays_callback'),
      'permission_callback' => array($this, 'custom_permission_callback') // Added custom permission callback
    ]);
  }

  public function custom_permission_callback()
  {
    return is_user_logged_in();
  }

  // Callback for settings that will be used in JavaScript
  public function get_delays_callback()
  {
    return array(
      'test_tool_delay_length' => get_option('test_tool_delay_length'),
      'return_visitor_delay_length' => get_option('return_visitor_delay_length'),
      'first_visit_delay_length' => get_option('first_visit_delay_length'),
    );
  }
}
