// Working, but only gets about 20 - 30 Kb


(function () {

  const uniqueRules = new Set();

  async function fetchCSSRules() {

    // Process the CSS text to extract rules
    function handleCssText(cssText) {
      cssText = cssText.replace(/\/\*[\s\S]*?\*\//g, ''); // Remove comments
      let currentRule = "", braceDepth = 0;

      cssText.split('\n').forEach(line => {
        line = line.trim();
        if (!line) return; // Skip empty lines

        if (line.includes('{')) braceDepth++;
        if (line.includes('}')) braceDepth--;

        currentRule += line + ' ';
        if (braceDepth === 0 && currentRule) {
          uniqueRules.add(currentRule.trim());
          currentRule = "";
        }
      });
    }

    // Iterate through all stylesheets
    for (let sheet of Array.from(document.styleSheets)) {
      if (sheet.href) { // Only external stylesheets have a href
        try {
          // console.log('Fetching stylesheet:', sheet.href);
          const response = await fetch(sheet.href);
          if (response.ok) {
            const cssText = await response.text();
            handleCssText(cssText);
            // console.log('Processed stylesheet:', sheet.href);
          } else {
            console.error('Failed to load stylesheet:', sheet.href);
          }
        } catch (error) {
          console.error('Error fetching stylesheet:', sheet.href, '; Error:', error);
        }
      }
    }

    console.log("Unique CSS Rules Array:", Array.from(uniqueRules));
  }

  const essentialSelectors = ['*', '*::before', '*:before', '*::after', '*:after', 'html', 'body', 'h1', 'h2', 'h3', 'h4', 'h5', 'h6', 'p', 'ul', 'ol', 'li', 'a', 'header', 'footer', 'nav', 'section', 'article', 'aside', 'button', 'input', 'optgroup', 'select', 'textarea'];

  function isGenericTagSelector(selector) {
    return /^[a-z]+$/i.test(selector) && !essentialSelectors.includes(selector);
  }

  function isWpAdminBarSelector(selector) {
    return selector.includes('#wpadminbar');
  }

  function extractSelectors(ruleText) {
    // Extract and clean up the CSS selectors from the given rule text
    const selectorGroups = ruleText.split(',');
    const selectors = new Set(
      selectorGroups
        .map(selector => selector.trim()) // Trim each selector group
        .flatMap(selector =>
          // Split complex selectors into individual parts
          selector.split(/(?<!\([^)]*)(?<!\[[^\]]*)(?<!\s)(?=[>+~ ])/)
        )
        .filter(selector =>
          // Filter out generic, admin bar, and empty selectors
          !isGenericTagSelector(selector) && !isWpAdminBarSelector(selector) && selector.trim().length > 0
        )
    );
    return selectors;
  }

  function extractDomSelectors() {
    const allElements = document.querySelectorAll('*');
    const selectors = new Set(essentialSelectors);

    allElements.forEach(element => {
      if (element.id) selectors.add('#' + element.id);
      element.classList.forEach(cls => selectors.add('.' + cls));
      selectors.add(element.tagName.toLowerCase());
    });

    return selectors;
  }


  function isAnySelectorMatching(selectors) {
    const uniqueSelectors = new Set(selectors);

    return Array.from(uniqueSelectors).some(selector => {
      if (selector.includes(':root')) {
        return true;
      }
      if (!selector.trim() || /^[\s>+~]+$/.test(selector)) {
        return false;
      }

      const isComplex = /[:\[\(]/.test(selector);
      if (isComplex) {
        const hasUnbalancedBrackets = (selector.match(/\[/g) || []).length !== (selector.match(/\]/g) || []).length;
        const hasUnbalancedParentheses = (selector.match(/\(/g) || []).length !== (selector.match(/\)/g) || []).length;
        if (hasUnbalancedBrackets || hasUnbalancedParentheses) {
          return false;
        }
      }

      try {
        return document.querySelectorAll(selector).length > 0;
      } catch (e) {
        return false;
      }
    });
  }


  function filterUsedCssRules(uniqueRules, domSelectors) {
    const usedCssRules = new Set();
  
    uniqueRules.forEach(rule => {
      const ruleSelectorPart = rule.split('{')[0].trim();
      // Extract selectors from the rule part using the previously defined extractSelectors function
      const extractedSelectors = Array.from(extractSelectors(ruleSelectorPart));
  
      // Check if any extracted selector is present in the DOM selectors set
      if (extractedSelectors.some(selector => domSelectors.has(selector))) {
        usedCssRules.add(rule);
      }
    });
  
    console.log("Extracted DOM Selectors:", Array.from(domSelectors));
    console.log("Filtered Used CSS Rules:", Array.from(usedCssRules));
  
    return Array.from(usedCssRules);
  }
  


  function minifyCss(rules) {
    return rules.join(' ');
  }

  // Integrating with the fetchCSSRules function
  async function processAndSendUsedCss() {
    await fetchCSSRules();
    const domSelectors = extractDomSelectors(); // Extract selectors from DOM elements
    const usedCssRules = filterUsedCssRules(uniqueRules, domSelectors); // Filter CSS rules using DOM selectors
    const minifiedCss = minifyCss(usedCssRules);
    console.log("Minified CSS Data:", minifiedCss); // This logs the final used CSS data
    sendCssToServer(minifiedCss);
  }

  processAndSendUsedCss();


  // Send the gathered and minified CSS to the server
  async function sendCssToServer(cssData) {
    const nonce = window.dynamicallyLoadedFilesDataCss.cssNonce;
    const url = window.dynamicallyLoadedFilesDataCss.getUsedCssUrl;
    const pageContext = document.getElementById('used-css-info');
    const templateType = pageContext.getAttribute('data-template-or-slug');

    try {
      const response = await fetch(url, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'X-WP-Nonce': nonce
        },
        credentials: 'same-origin',
        body: JSON.stringify({
          template_type: templateType,
          css: cssData
        })
      });

      if (!response.ok) {
        throw new Error(`Network response was not ok: ${response.statusText}`);
      }

      const data = await response.json();
      console.log('CSS successfully sent to server:', data);
    } catch (error) {
      console.error('Error sending CSS data to server:', error);
    }
  }

})();
