/* Get Critical CSS Function */
/*
    // Disable scrolling
    document.body.classList.add('no-scroll');

    // ... existing code for gatherCriticalCSS function ...

    // Enable scrolling after the process
    document.body.classList.remove('no-scroll');
*/


(function () {
	let GenCriticalCSSLoader; // Define loader at a higher scope

	// Create the loader ahead of time
	GenCriticalCSSLoader = document.createElement("div");
	GenCriticalCSSLoader.id = "gen-critical-css-loader";
	GenCriticalCSSLoader.textContent = "Generating Critical CSS...";
	GenCriticalCSSLoader.style.display = "none"; // Hide it initially
	document.body.appendChild(GenCriticalCSSLoader);

	// Generate critical CSS
	function gatherAndDisplayCriticalCSS() {
		// Constants
		const NOTIFICATION_DISPLAY_TIME = 3 * 1000;
		const EXTRA_VIEWPORT_HEIGHT = 250;
		const ADMIN_BAR_SELECTOR = "#wpadminbar";
		const WRAPPER_ID = "critical-css-wrap";
		const COPY_BUTTON_ID = "copy-critical-css";
		const NOTIFICATION_ID = "copy-critical-css-notification";
		const CLOSE_BUTTON_ID = "close-critical-css";
		const CONTAINER_ID = "critical-css-container";
		const BUTTONS_CONTAINER_ID = "ss-btn-cont";
		const NEW_DIV_ID = "page-template-n-slug";

		// Create the HTML elements
		const wrap = document.createElement("div");
		wrap.id = WRAPPER_ID;

		const copyButton = document.createElement("button");
		copyButton.id = COPY_BUTTON_ID;
		copyButton.textContent = "Copy CSS To Clipboard";

		const notification = document.createElement("div");
		notification.id = NOTIFICATION_ID;
		notification.style.display = "none";

		const container = document.createElement("pre");
		container.id = CONTAINER_ID;

		// Create the new div and move the page info into it
		const pageInfo = document.createElement("div");
		pageInfo.id = NEW_DIV_ID;
		const pageTemplate = document.getElementById("ss-page-template");
		const pageSlug = document.getElementById("ss-page-slug");
		if (pageTemplate) {
			pageTemplate.removeAttribute("style");
			pageInfo.appendChild(pageTemplate);
		}
		if (pageSlug) {
			pageSlug.removeAttribute("style");
			pageInfo.appendChild(pageSlug);
		}

		// create the close button
		const closeButton = document.createElement("button");
		closeButton.id = CLOSE_BUTTON_ID;
		closeButton.textContent = "Close";
		pageInfo.appendChild(closeButton);

		// Create the buttons container
		const buttonsContainer = document.createElement("div");
		buttonsContainer.id = BUTTONS_CONTAINER_ID;

		// Append the buttons and notification to the buttons container
		buttonsContainer.appendChild(copyButton);
		buttonsContainer.appendChild(notification);

		// Append the buttons container, the page info and the container to the wrapper
		wrap.appendChild(pageInfo);
		wrap.appendChild(buttonsContainer);
		wrap.appendChild(container);

		// Add the wrapper to the document body
		document.body.appendChild(wrap);

		// Function to show notifications and hide them after a set time
		function showNotification(message) {
			notification.textContent = message;
			notification.style.display = "inline-block";
			setTimeout(() => {
				notification.style.display = "none";
			}, NOTIFICATION_DISPLAY_TIME);
		}

		// Function to gather critical CSS
		function gatherCriticalCSS() {
			// Clear the container
			container.textContent = "";

			// Get all stylesheets on the page
			const stylesheets = Array.from(document.styleSheets);

			// Loop through each stylesheet
			for (let stylesheet of stylesheets) {
				try {
					const cssRules = Array.from(stylesheet.cssRules || stylesheet.rules);

					// Loop through each CSS rule
					for (let cssRule of cssRules) {
						// Handle media queries
						if (cssRule instanceof CSSMediaRule) {
							// Loop through each rule inside the media rule
							for (let rule of cssRule.cssRules) {
								if (isInViewport(rule)) {
									// Append the CSS rule to the container
									container.textContent += `@media ${cssRule.media.mediaText} { ${rule.cssText} }\n`;
								}
							}
						} else if (isInViewport(cssRule)) {
							// Append the CSS rule to the container
							container.textContent += cssRule.cssText + "\n";
						}
					}
				} catch (error) {
					// Display the error notification
					//showNotification('Stylesheet could not be processed: ' + error.message);
					//console.error(error);
				}
			}

			// Minify the CSS in the container
      container.textContent = container.textContent
    .replace(/\s*{\s*/g, '{')
    .replace(/\s*}\s*/g, '}')
    .replace(/\s*:\s*/g, ':')
    .replace(/\s*;\s*/g, ';')
    .trim();

		}

		// Function to check if an element is within the viewport or has display: none
		function isInViewport(cssRule) {
			// Get the elements matching the CSS rule selector
			const elements = document.querySelectorAll(cssRule.selectorText);

			// Loop through each element
			for (let i = 0; i < elements.length; i++) {
				const element = elements[i];

				// Check if the element is a descendant of the admin bar
				if (element.closest(ADMIN_BAR_SELECTOR)) {
					continue; // Skip this element
				}

				// Check if the element is within the viewport or 100px below it, or if it has display: none
				const rect = element.getBoundingClientRect();
				const style = window.getComputedStyle(element);
				if (
					(rect.top < window.innerHeight + EXTRA_VIEWPORT_HEIGHT &&
						rect.bottom > 0) ||
					style.display === "none"
				) {
					return true;
				}
			}

			return false;
		}

		// Add event listeners to the buttons
		copyButton.addEventListener("click", async () => {
			// Copy the content of the container to the clipboard using the Clipboard API
			try {
				await navigator.clipboard.writeText(container.textContent);
				showNotification("Critical CSS copied to clipboard!");
			} catch (error) {
				showNotification("Failed to copy text: " + error.message);
				console.error(error);
			}
		});

		closeButton.addEventListener("click", () => {
			// Hide the wrapper
			wrap.style.display = "none";
		});

		// Call the gatherCriticalCSS function
		gatherCriticalCSS();

		// Hide the loader
		GenCriticalCSSLoader.style.display = "none"; // hide loader
	}


		let element = document.querySelector("#wp-admin-bar-get-critical-css .ab-item");
		if (element) {

			// If the element exists, attach two event listeners to it
			element.addEventListener("click", function () {
				// Scroll to the top of the page
				window.scrollTo(0, 0);

				// Show the loader
				GenCriticalCSSLoader.style.display = "block";
			});

			element.addEventListener("click", function () {
				setTimeout(gatherAndDisplayCriticalCSS, 100);
			});
		}

})();
