/*** LAZY LOAD VIDEOS ***/

(function () {
  // Function to detect if the user is on a mobile device
  function isMobileDevice() {
    return (window.matchMedia("(max-width: 767px)").matches ||
            ('ontouchstart' in window) ||
            (navigator.maxTouchPoints > 0) ||
            (navigator.msMaxTouchPoints > 0));
  }

  // If the user is on a mobile device, use the Intersection Observer
  if (isMobileDevice()) {
    // Initialize Intersection Observer to detect when the placeholder is in the viewport
    var observer = new IntersectionObserver(function (entries) {
      entries.forEach(function (entry) {
        if (entry.isIntersecting) {
          // Get the target element
          var target = entry.target;

          // Create an iframe and set its attributes based on the placeholder's data attributes
          replacePlaceholderWithIframe(target);

          // Unobserve the placeholder element as it has been replaced
          observer.unobserve(target);
        }
      });
    });

    // Get all elements with the class 'ss-video-placeholder' and observe them
    var placeholders = document.querySelectorAll('.ss-video-placeholder');
    placeholders.forEach(function (placeholder) {
      observer.observe(placeholder);
    });
  } else {
    // For non-mobile devices, use mouseover event
    document.addEventListener('mouseover', function (event) {
      var target = event.target;
      while (target && !target.classList.contains('ss-video-placeholder')) {
        target = target.parentElement;
      }
      if (!target) return;

      // Create an iframe and set its attributes based on the placeholder's data attributes
      replacePlaceholderWithIframe(target);
    });
  }

  // Function to replace a placeholder with an iframe
  function replacePlaceholderWithIframe(target) {
    // Read data attributes from the placeholder
    var src = target.getAttribute('data-src');
    var title = target.getAttribute('data-title');
    var width = target.getAttribute('data-width');
    var height = target.getAttribute('data-height');
    var style = target.getAttribute('data-style');
    var classes = target.getAttribute('class');

    classes = classes.replace(/\bss-allow-bg\b|\bss-video-placeholder|\bbg-lazy-loaded\b/g, '').trim();
    
    // Create a new iframe element
    var iframe = document.createElement('iframe');

    // Set ARIA roles and attributes for accessibility
    iframe.setAttribute('role', 'application');
    iframe.setAttribute('aria-label', title);

    // Set the iframe's attributes
    iframe.setAttribute('src', src);
    iframe.setAttribute('title', title);
    iframe.setAttribute('width', width);
    iframe.setAttribute('height', height);
    iframe.setAttribute('style', style);
    iframe.setAttribute('class', classes);
    iframe.setAttribute('frameborder', '0');
    iframe.setAttribute('allow', 'accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture; web-share');
    iframe.setAttribute('allowfullscreen', '');
    iframe.setAttribute('video-lazyloaded', 'super-speed');
    // Check if the 'yt-bg-preloaded' attribute exists on the target element
    if (target.hasAttribute('yt-bg-preloaded')) {
      // Set the 'yt-bg-preloaded' attribute on the iframe element
      iframe.setAttribute('yt-bg-preloaded', '');
    }

    // Replace the placeholder with the new iframe
    target.parentNode.replaceChild(iframe, target);
  }
})();

