// Remove image data when the adminbar links are clicked, and reload the page

(async function () {
  'use strict';

  async function sha256(str) {
    const buffer = new TextEncoder().encode(str);
    const digest = await crypto.subtle.digest('SHA-256', buffer);
    return Array.from(new Uint8Array(digest)).map(val => val.toString(16).padStart(2, '0')).join('');
  }

  var messageText;

  function showLoadingMessage() {
    // Check if a loading message already exists
    if (!document.querySelector('#loadingMessage')) {
      const messageDiv = document.createElement('div');
      messageDiv.setAttribute('id', 'loadingMessage');
      messageDiv.style.cssText = `
      position: fixed;
      top: 50%;
      left: 50%;
      transform: translate(-50%, -50%);
      color: white;
      font-size: clamp(13px, 2.5vw, 22px);
      padding: 0.5em 1em;
      border-radius: 0.5em;
      box-shadow: 10px 11px 50px black;
      opacity: 0.6;
      background: #003b87;
      z-index: 2147483647;
      font-family: system-ui;
  `;
      messageDiv.innerText = messageText;
      document.body.appendChild(messageDiv);
    }
  }

  // function hideLoadingMessage() {
  //   const messageDiv = document.querySelector('#loadingMessage');
  //   if (messageDiv) {
  //     document.body.removeChild(messageDiv);
  //   }
  // }

  // Use nonce in two functions
  const nonce = window.dynamicallyLoadedFilesDataImages.imagesNonce;

  async function removeData() {
    
    messageText = 'Refreshing images data for this page...';

    showLoadingMessage();

    const url = window.dynamicallyLoadedFilesDataImages.deleteImageDataUrl;
    const normalizedUrl = window.location.href.toLowerCase().split('#')[0].split('?')[0];
    const hash = await sha256(normalizedUrl);

    try {
      const response = await fetch(url, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'X-WP-Nonce': nonce
        },
        credentials: 'same-origin',
        body: JSON.stringify({ currentUrl: hash })
      });

      if (!response.ok) throw new Error('Network response was not ok.');

      const data = await response.json();

      if (data.success && data.message === 'Image data removed successfully') {
        console.log('Images data file deleted successfully:', data);
        window.location.reload();
      } else {
        console.error('Failed to delete images data file:', data);
        window.location.reload();
      }
    } catch (error) {
      console.error('Error during fetch operation:', error);
      window.location.reload();
    } finally {
      window.location.reload();
      // hideLoadingMessage();
    }
  }



  async function removeAllData() {

    messageText = 'Deleting images data for all pages...';

    showLoadingMessage();

    const url = window.dynamicallyLoadedFilesDataImages.deleteAllImagesDataUrl;

    try {
      const response = await fetch(url, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'X-WP-Nonce': nonce
        },
        credentials: 'same-origin',
        body: JSON.stringify({
          action: 'delete_all_image_data',
        }),
      });

      if (!response.ok) {
        throw new Error(`HTTP error! status: ${response.status}`);
      }

      const data = await response.json();
      console.log('Response data:', data);

      if (data.success) {
        console.log('All CSS files deleted successfully:', data);
        window.location.reload();
      } else {
        console.error('Failed to delete all CSS files:', data);
        // hideLoadingMessage();
        window.location.reload();
      }
    } catch (error) {
      console.error('Error during fetch operation:', error);
      // hideLoadingMessage();
      window.location.reload();
    }
  }


  // Event listener for removing data of a specific page
  const refreshImgBtn = document.querySelector('#wp-admin-bar-refresh-img-data .ab-item');
  if (refreshImgBtn) {
    refreshImgBtn.addEventListener('click', removeData);
  }

  // Event listener for removing all image data
  const deleteAllImgDataBtn = document.querySelector('#wp-admin-bar-refresh-all-img-data .ab-item');
  if (deleteAllImgDataBtn) {
    deleteAllImgDataBtn.addEventListener('click', removeAllData);
  }
})();


