
// Remove image used CSS files when the adminbar links are clicked, and reload the page

(async function () {
  // Make sure dynamicallyLoadedFilesDataCss is defined
  if (!window.dynamicallyLoadedFilesDataCss) {
    console.error("Configuration data is missing.");
    return;
  }

  function showLoadingMessage() {
    if (!document.querySelector('#loadingMessage')) {
      const messageDiv = document.createElement('div');
      messageDiv.setAttribute('id', 'loadingMessage');
      messageDiv.style.cssText = `
      position: fixed;
      top: 50%;
      left: 50%;
      transform: translate(-50%, -50%);
      color: white;
      font-size: clamp(13px, 2.5vw, 22px);
      padding: 0.5em 1em;
      border-radius: 0.5em;
      box-shadow: 10px 11px 50px black;
      opacity: 0.6;
      background: #003b87;
      z-index: 2147483647;
      font-family: system-ui;
  `;
      messageDiv.innerText = messageText;
      document.body.appendChild(messageDiv);
    }
  }

  function hideLoadingMessage() {
    const messageDiv = document.querySelector('#loadingMessage');
    if (messageDiv) {
      document.body.removeChild(messageDiv);
    }
  }

  // Use nonce in two functions
  const nonce = window.dynamicallyLoadedFilesDataCss.cssNonce;

  async function deleteUsedCSS() {
    const deleteCssUrl = window.dynamicallyLoadedFilesDataCss.deleteCssUrl;

    messageText = 'Refreshing used CSS for this page...';
    showLoadingMessage();
    const pageContext = document.getElementById('used-css-info');
    const cssFileLocation = pageContext.getAttribute('data-used-css-file-location');

    try {
      const response = await fetch(deleteCssUrl, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'X-WP-Nonce': nonce
        },
        credentials: 'same-origin',
        body: JSON.stringify({
          css_file_url: cssFileLocation,
        })
      });

      const data = await response.json();
      console.log('Response data:', data); // Detailed log of the response

      if (response.ok && data.message === 'CSS file deleted successfully') {
        console.log('CSS file deleted successfully:', data);
      } else {
        // This assumes your WP_Error objects are correctly converted to JSON
        console.error('Failed to delete CSS file:', data);
        // hideLoadingMessage();
      }

    } catch (error) {
      console.error('Error during fetch operation:', error);
      // hideLoadingMessage();
    }
    window.location.reload();
  }


  async function deleteAllUsedCSS() {
    const deleteAllCssUrl = window.dynamicallyLoadedFilesDataCss.deleteAllCssUrl;

    messageText = 'Deleting used CSS for all pages...';
    showLoadingMessage();

    try {
      const response = await fetch(deleteAllCssUrl, {
        method: 'POST',
        headers: {
          'Content-Type': 'application/json',
          'X-WP-Nonce': nonce
        },
        credentials: 'same-origin',
        body: JSON.stringify({
          action: 'delete_all_used_css',
        }),
      });

      if (!response.ok) {
        throw new Error(`HTTP error! status: ${response.status}`);
      }

      const data = await response.json();
      console.log('Response data:', data);

      if (data.success) {
        console.log('All CSS files deleted successfully:', data);
      } else {
        console.error('Failed to delete all CSS files:', data);
        // hideLoadingMessage();
      }
    } catch (error) {
      console.error('Error during fetch operation:', error);
      // hideLoadingMessage();
    }
    window.location.reload();
  }



  // Event listener for deleting the used CSS file for the current page
  const refreshUsedCssBtn = document.querySelector('#wp-admin-bar-refresh-used-css .ab-item');
  if (refreshUsedCssBtn) {
    refreshUsedCssBtn.addEventListener('click', deleteUsedCSS);
  }

  // Event listener for deleting all used CSS files
  const deleteAllUsedCssBtn = document.querySelector('#wp-admin-bar-refresh-all-used-css .ab-item');
  if (deleteAllUsedCssBtn) {
    deleteAllUsedCssBtn.addEventListener('click', deleteAllUsedCSS);
  }

})();
