<?php

namespace SuperSpeed;

use SuperSpeed\Utility;

class FixThemesAndPlugins
{

  public function __construct()
  {
    // Exit if we could not get the current URL. Skipping static resource because it is not a page.
    if (!Utility::get_current_page_url()) {
      return;
    }

    // If we are in the backend
    if (Utility::is_page_editor_or_wp_backend()) {
      return;
    }

    // Add CSS to fix WooCommerce
    add_action('wp_head', array($this, 'css_to_fix_woocommerce'), -1 * PHP_INT_MAX);

    // Add classes to the body
    add_filter('body_class', array($this, 'add_classes_to_body'));
  }



  // Add CSS to fix WooCommerce
  public function css_to_fix_woocommerce()
  {
?>
    <style id="superspeed-fix-woocommerce" type="text/css">
      /* WooCommerce Product page */

      /* Main product image on the product page */
      .woocommerce-product-gallery.woocommerce-product-gallery--with-images {
        opacity: unset !important;
      }

      /* Tabs were open untill JS loads */
      .woocommerce div.product .woocommerce-tabs #tab-reviews.panel {
        display: none;
      }
    </style>
<?php
  }

  // Add classes to the body
  public function add_classes_to_body($classes)
  {

    // Some styles are dependant on this class in the <body>
    // We need to add it, because there is no JS during a delay
    $classes[] = 'woocommerce-js';
    return $classes;
  }
}
