<?php

namespace SuperSpeed;

use SuperSpeed\Utility;

class Main
{

  public function __construct()
  {
    // Exit if we could not get the current URL. Skipping static resource because it is not a page.
    if (!Utility::get_current_page_url()) {
      return;
    }

    // Add pages to the admin menu
    add_action('admin_menu', array($this, 'add_admin_menu'));

    // Load translation files
    add_action('init', array($this, 'load_translation'));

    // Add scripts and stylesheets for the backend page
    add_action('admin_enqueue_scripts', array($this, 'enqueue_backend_scripts_and_styles'));

    // Admin bar tools
    add_action('admin_bar_menu', array($this, 'customize_my_wp_admin_bar'), 41);

    // Add classes to the body to improve CSS targeting, and show elements only to admins and editors
    add_filter('body_class', array($this, 'add_role_to_body_class'), PHP_INT_MAX);

    // Create an element to place into data from JS tools
    add_filter('wp_head', array($this, 'store_page_info_above_header'), 0);

    // Inline the JS files delay script
    add_action('wp_footer', array($this, 'inlined_js_files_delay_script'), 99);

    // Delay the plugin's inline styles
    add_action('wp_footer', array($this, 'delay_plugins_inline_style'), 99);

    // Set a Global Preload variable
    add_action('init', array($this, 'set_global_variable_for_preloading'));
    add_action('template_redirect', array($this, 'buffer_start_global_preload'), 10);
    add_action('shutdown', array($this, 'buffer_end_global_preload'), 20);
  }



  // adds a sub-menu under the Settins menu
  public function add_admin_menu()
  {
    add_options_page(
      esc_html__('SuperSpeed', 'wpssxtdomain'),
      esc_html__('SuperSpeed', 'wpssxtdomain'),
      'manage_options',
      \SuperSpeed\Config::PLUGIN_SLUG,
      array($this, 'settings_page')
    );
  }

  public function load_translation()
  {
    load_plugin_textdomain('wpssxtdomain', false, dirname(plugin_basename(__FILE__)) . '/languages');
  }


  // Backend scripts and styles
  public function enqueue_backend_scripts_and_styles($hook)
  {
    if ($hook === 'settings_page_' . \SuperSpeed\Config::PLUGIN_SLUG || $hook === 'toplevel_page_' . \SuperSpeed\Config::PLUGIN_SLUG) {

      global $ss_global_variable_for_version_control;

      if ($ss_global_variable_for_version_control) {
        $version_file_1 = $ss_global_variable_for_version_control;
        $version_file_2 = $ss_global_variable_for_version_control;
        $version_file_3 = $ss_global_variable_for_version_control;
      } else {

        // Define the base directory for JS files on the system file path
        $ansolute_base_dir = ABSPATH . 'wp-content/plugins/superspeed/admin/assets/';

        // Define file system paths for the JavaScript files
        $file_path_1 = $ansolute_base_dir . 'js/admin-script.js';
        $version_file_1 = filemtime($file_path_1);

        $file_path_2 = $ansolute_base_dir . 'js/save-settings-fetch-api.js';
        $version_file_2 = filemtime($file_path_2);

        $file_path_3 = $ansolute_base_dir . 'css/admin-style.css';
        $version_file_3 = filemtime($file_path_3);
      }

      $base_dir_js = plugin_dir_url(dirname(__FILE__, 2)) . 'admin/assets/js/';
      $base_dir_css = plugin_dir_url(dirname(__FILE__, 2)) . 'admin/assets/css/';



      // Main settings UI script
      wp_enqueue_script('ss-backend-script', $base_dir_js . 'admin-script.js', array(), $version_file_1, array('in_footer' => true, 'strategy' => 'defer'));

      // Save setting with Fetch API Ajax
      wp_enqueue_script('save-settings-fetch-api', $base_dir_js . 'save-settings-fetch-api.js', array(), $version_file_2, array('in_footer' => true, 'strategy' => 'defer'));
      // Localize the script
      wp_localize_script('save-settings-fetch-api', 'wpss_ajax_object', array(
        'ajax_url' => admin_url('admin-ajax.php'),
        'security' => wp_create_nonce('wp_ss_nonce')
      ));

      /*
      // CSS repeater UI handler
      wp_enqueue_script('critical-css-ui', plugin_dir_url(dirname(__FILE__, 2)) . 'admin/assets/js/critical-css-ui.js', array(), $version, array('in_footer' => true, 'strategy' => 'defer'));
      // using is_plugin_active_for_network() function to pass data to script
      include_once(ABSPATH . 'wp-admin/includes/plugin.php');
      $script_data = array(
        'is_woocommerce_active' => is_plugin_active('woocommerce/woocommerce.php')
      );
      // Localize the script
      wp_localize_script('critical-css-ui', 'woo_active_conformation_data', $script_data);
      */

      wp_enqueue_style('ss-backend-style', $base_dir_css . 'admin-style.css', array(), $version_file_3);
    }
  }

  /*** Admin bar tools ***/

  public function customize_my_wp_admin_bar($wp_admin_bar)
  {
    if (!is_admin()) {
      $args = array(
        'id'    => 'ss-tools',
        'title' => '<span class="ab-icon dashicons dashicons-image-rotate" style="translate: 0 2px;"></span> ' . esc_html__("No-Cache Reload", "wpssxtdomain"),
        'href'  => '#',
        'meta'  => array('class' => 'ss-tools')
      );
      $wp_admin_bar->add_node($args);

      // Add the submenu node

      // Go to plugin settings page
      $args = array(
        'id'     => 'go-to-settings',
        'title'  => '<span class="ab-icon dashicons dashicons-admin-generic"></span> ' . esc_html__("Go To SuperSpeed Settings", "wpssxtdomain"),
        'href'   => admin_url('options-general.php?page=superspeed'), // Dynamic admin URL for staging and multisites
        'meta'   => array('target' => '_blank',),
        'parent' => 'ss-tools'
      );
      $wp_admin_bar->add_node($args);


      // List JS and CSS
      $args = array(
        'id'     => 'list-js-css',
        'title'  => '<span class="ab-icon dashicons dashicons-code-standards"></span> ' . esc_html__("List JS & CSS", "wpssxtdomain"),
        'href'   => '#',
        'parent' => 'ss-tools'
      );
      $wp_admin_bar->add_node($args);

      // Get Image Data
      if (Utility::get_images_info_on_this_page()) {
        $args = array(
          'id'     => 'refresh-img-data',
          'title'  => '<span class="ab-icon dashicons dashicons-image-rotate-right"></span> ' . esc_html__("Refresh Image Data For Page", "wpssxtdomain"),
          'href'   => '#',
          'parent' => 'ss-tools'
        );
        $wp_admin_bar->add_node($args);

        $args = array(
          'id'     => 'refresh-all-img-data',
          'title'  => '<span class="ab-icon dashicons dashicons-trash"></span> ' . esc_html__("Delete Image Data From All Pages", "wpssxtdomain"),
          'href'   => '#',
          'parent' => 'ss-tools'
        );
        $wp_admin_bar->add_node($args);
      } else {
        $args = array(
          'id'     => 'img-data-notification',
          'title'  => '<span class="ab-icon disabled dashicons-image-rotate-right"></span> ' . esc_html__("DISABLED - Image Data Collecting", "wpssxtdomain"),
          'href'   => '#',
          'parent' => 'ss-tools'
        );
        $wp_admin_bar->add_node($args);
      }

      // Used CSS - remove file of the page
      if (Utility::getting_used_css_on_page()) {
        $args = array(
          'id'     => 'refresh-used-css',
          'title'  => '<span class="ab-icon dashicons dashicons-editor-code"></span> ' . esc_html__("Refresh Used CSS For Page", "wpssxtdomain"),
          'href'   => '#',
          'parent' => 'ss-tools'
        );
        $wp_admin_bar->add_node($args);

        $args = array(
          'id'     => 'refresh-all-used-css',
          'title'  => '<span class="ab-icon dashicons dashicons-trash"></span> ' . esc_html__("Delete Used CSS For All Pages", "wpssxtdomain"),
          'href'   => '#',
          'parent' => 'ss-tools'
        );
        $wp_admin_bar->add_node($args);
      } else {
        $args = array(
          'id'     => 'used-css-notification',
          'title'  => '<span class="ab-icon disabled dashicons dashicons-editor-code"></span> ' . esc_html__("DISABLED - Used CSS Collecting", "wpssxtdomain"),
          'href'   => '#',
          'parent' => 'ss-tools'
        );
        $wp_admin_bar->add_node($args);
      }
    }
  }

  public function add_role_to_body_class($classes)
  {
    if (is_user_logged_in()) {
      $current_user = wp_get_current_user();
      $editable_roles = ['administrator', 'editor', 'author', 'contributor'];
      $editable_roles_lookup = array_flip($editable_roles); // Convert to lookup array

      foreach ($current_user->roles as $role) {
        if (isset($editable_roles_lookup[$role])) {
          $classes[] = 'can-edit';
          break; // Break the loop after adding 'can-edit'
        }
      }
    }
    return $classes;
  }


  // Create an element to place into data from JS tools
  public function store_page_info_above_header()
  {
    echo '<div id="page-info-wrap" style="display: none;">
            <input class="accordion-page-info-input" type="checkbox" id="page-info-accordion" tabindex="0">
            <label for="page-info-accordion" id="accordion-page-info-label">
              SuperSpeed Page Info<span id="page-info-chevron">▲</span>
            </label>
            <div id="superspeed-page-info">
            </div>
          </div>';
  }

  public function settings_page()
  {
    if (!current_user_can('manage_options')) {
      wp_die(__('You do not have sufficient permissions to access this page.', 'wpssxtdomain'));
    }
?>
    <div class="wp-ss-settings">
      <?php wp_nonce_field(\SuperSpeed\Config::PLUGIN_SLUG, 'wp_ss_nonce'); ?>
      <div class="ss-heading-wrap">
        <div class="ss-heading-left"></div>
        <div class="ss-heading-right">
          <h1>SuperSpeed</h1>
          <span id="design-choice">Design
            <div class="design-choice-wrap">
              <div class="design-choice" data-value="clean">Clean</div>
              <div class="design-choice" data-value="stylized">Stylized</div>
            </div>
          </span>
          <a class="website-link" href="https://wpsuperspeed.com/" target="_blank">Website</a>
          <a class="fb-group-link" href="https://www.facebook.com/groups/wpsuperspeed/" target="_blank">FB Group</a>
        </div>
      </div>

      <div class="ss-wrap">
        <div class="ss-tabs-container">
          <div aria-controls="tab-list" role="tablist">
            <div class="ss-tabs-wrap">
              <div class="ss-wrap-menu">
                <ul aria-controls="tab-list" role="tablist">
                  <!-- <li id="ss-tab1" class="active" role="tab" aria-controls="ss-tab-1" tabindex="0" aria-selected="true">
                    <span class="tab-label"><?php esc_html_e('Instructions', 'wpssxtdomain'); ?></span>
                  </li> -->
                  <li id="ss-tab2" role="tab" aria-controls="ss-tab-2" tabindex="0" aria-selected="false">
                    <span class="tab-label"><?php esc_html_e('JavaScript', 'wpssxtdomain'); ?></span>
                  </li>
                  <li id="ss-tab3" role="tab" aria-controls="ss-tab-3" tabindex="0" aria-selected="false">
                    <span class="tab-label"><?php esc_html_e('CSS', 'wpssxtdomain'); ?></span>
                  </li>
                  <li id="ss-tab4" role="tab" aria-controls="ss-tab-4" tabindex="0" aria-selected="false">
                    <span class="tab-label"><?php esc_html_e('Media', 'wpssxtdomain'); ?></span>
                  </li>
                  <li id="ss-tab7" role="tab" aria-controls="ss-tab-5" tabindex="0" aria-selected="false">
                    <span class="tab-label"><?php esc_html_e('Preload Links', 'wpssxtdomain'); ?></span>
                  </li>
                  <!-- <li id="ss-tab5" role="tab" aria-controls="ss-tab-6" tabindex="0" aria-selected="false">
                    <span class="tab-label"><?php esc_html_e('Page Builders', 'wpssxtdomain'); ?></span>
                  </li> -->
                  <!-- <li id="ss-tab6" role="tab" aria-controls="ss-tab-7" tabindex="0" aria-selected="false">
                    <span class="tab-label"><?php esc_html_e('Debloat', 'wpssxtdomain'); ?></span>
                  </li> -->
                  <!-- <li id="ss-tab7" role="tab" aria-controls="ss-tab-8" tabindex="0" aria-selected="false">
                    <span class="tab-label"><?php esc_html_e('Extras', 'wpssxtdomain'); ?></span>
                  </li> -->
                </ul>

              </div>

              <div class="ss-tab-content-container">
                <?php
                function get_template_path($filename)
                {
                  $plugin_root_dir = plugin_dir_path(dirname(__FILE__, 2)); // get the root directory of the plugin
                  return $plugin_root_dir . 'admin/templates/tabs/' . $filename;
                }

                include get_template_path('instructions.php');
                include get_template_path('javascript.php');
                include get_template_path('css.php');
                include get_template_path('media.php');
                include get_template_path('preload-links.php');
                include get_template_path('page-builders.php');
                include get_template_path('debloat.php');
                include get_template_path('extras.php');

                ?>
              </div>
            </div>
          </div>
        </div>
      </div>
      <span class="floating-submit">
        <button id="save_wp_ss_button" class="button button-primary"><?php esc_html_e('Save All Changes', 'wpssxtdomain'); ?></button>
        <span class="saving-message saving-info"><?php esc_html_e('Saving...', 'wpssxtdomain'); ?></span>
        <span class="saving-message saving-success"><?php esc_html_e('Saved', 'wpssxtdomain'); ?></span>
        <span class="saving-message saving-error"><?php esc_html_e('Error saving settings. Please, try again later.', 'wpssxtdomain'); ?></span>
      </span>
      <div class="saving-overlay"></div>
    </div>
  <?php
  }

  /*** Run any of our scripts, only if a specific element is in the DOM ***/
  // Scripts are loaded if the user is human, and at the 'load' event

  public function inlined_js_files_delay_script()
  {

    if (\SuperSpeed\Utility::is_page_editor_or_wp_backend()) {
      return;
    }

  ?>
    <script defer id="superspeed-delayed-scripts">
      (function() {

        let hasRun = false; // Flag to check if MainDelayScripts has already run
        let userInteracted = false; // Flag to check if the user has interacted with the page

        // Check for localStorage item
        const hasVisitedBefore = localStorage.getItem('hasVisitedBefore');

        /*
        // Function to detect known testing tools
        function isPossibleTestingTool() {
          if (hasVisitedBefore) return true;

          // Get the user agent string
          const userAgent = window.navigator.userAgent.toLowerCase();

          // Known testing tool user agents or patterns
          const testingPatterns = [
            'gtmetrix',
            'ptst/',
            'dmbrowser/',
            'chrome-lighthouse',
            'debugbear'
          ];

          // Detect user agent
          for (const pattern of testingPatterns) {
            if (userAgent.includes(pattern)) {
              return true;
            }
          }

          // Get the OS platform
          const platform = window.navigator.platform.toLowerCase();

          // Detect platform
          const platformPatterns = [
            "linux x86_64",
            "linux amd64"
          ];

          if (platformPatterns.some(p => platform.includes(p))) {
            return true;
          }

          return false;
        }
        */

        // Function to load scripts based on the presence of specific DOM elements
        async function loadScripts() {
          // Define scripts and their respective DOM selectors
          const scripts = {
            '/delay-styles.js': ['link[delay-href]'], // not on first open
            '/delay-scripts.js': ['script[type="ss-delay-script"]'],
            '/lazy-load-images.js': ['[lazy_loaded_img]', '[ss-srcset]'],
            '/lazy-load-backgrouds.js': ['style#superspeed-lazy-load-backgrounds', '[lazy_loaded_bg]', '[bg-srcset-all]', '[bg-srcset-desktop]', '[bg-srcset-tablet]', '[bg-srcset-mobile]'], // not on first open if for selector 'style#superspeed-lazy-load-backgrounds'
            '/lazy-load-iframes.js': ['iframe[lazy_loaded_iframe]:not([src*="youtu"])'],
            '/lazy-load-videos.js': ['.ss-video-placeholder'],
            '/preload-links.js': ['script#SSpreFetchConfig'],
            '/original-url-reset-backgrounds.js': ['[bg_preloaded]', '[bg_preloaded_desktop]', '[bg_preloaded_tablet]', '[bg_preloaded_mobile]', '[bg_placeholder]'],
            '/original-url-reset-images.js': ['[ss-original-src-img]', '[data-srcset]'],
            '/display-lcp-info.js': ['body.can-edit style#superspeed-display-lcp-info'],

            // Admin bar tools
            '/no-cache-reload.js': ['li#wp-admin-bar-ss-tools'],
            '/list-js-and-css.js': ['li#wp-admin-bar-ss-tools'],
          };

          <?php
          global $ss_global_variable_for_version_control;

          if ($ss_global_variable_for_version_control) {
            $dynamically_loaded_file_versions = $ss_global_variable_for_version_control;
          } else {
            $dynamically_loaded_file_versions = time();
          }

          // Base URL for the scripts, important to not hard-code the path for multisites and staging sites
          $script_base_url = plugins_url('assets/js/', dirname(__FILE__, 2));
          ?>

          // Non-hard coded path
          const scriptBaseUrl = '<?php echo $script_base_url; ?>';

          // Declare the JavaScript 'version' variable with proper scope and initialization
          const version = <?php echo $dynamically_loaded_file_versions ?>;

          // Load each script if at least one of its respective DOM elements exists
          const scriptPromises = Object.entries(scripts)
            .filter(([scrips, selectors]) => selectors.some(selector => document.querySelector(selector)))
            .map(([scripts, selectors]) => {
              return new Promise((resolve, reject) => {
                let scriptElement = document.createElement('script');
                scriptElement.src = `${scriptBaseUrl + scripts}?v=${version}`; // Cache-busting implemented
                scriptElement.onload = resolve;
                scriptElement.onerror = reject;
                document.body.appendChild(scriptElement);
              });
            });

          // Wait for all script promises to resolve
          await Promise.all(scriptPromises);
        }

        // Your MainDelayScripts function
        async function MainDelayScripts() {
          if (hasRun) return; // If the function has already run, return immediately

          await loadScripts();

          hasRun = true;
        }

        // Event listener to set user interaction flag
        function setUserInteracted() {
          userInteracted = true;
        }

        // Event types to listen for user interaction
        const userEvents = ["mouseover", "keydown", "touchmove", "touchstart", "scroll"];

        for (const event of userEvents) {
          window.addEventListener(event, setUserInteracted, {
            once: true
          });
        }

        function shouldRunMainDelayScripts() {
          // If the user has visited before, return true immediately
          if (hasVisitedBefore) {
            return true;
          }

          // If the user has interacted, return true
          if (userInteracted) {
            return true;
          }

          // If it's not a possible testing tool, return true
          // if (!isPossibleTestingTool()) {
          //   return true;
          // }

          // If none of the above conditions are met, return false
          return false;
        }

        window.addEventListener('load', () => {
          if (shouldRunMainDelayScripts()) {
            MainDelayScripts();
          }
        });


        // Set the localStorage item to indicate the user has visited before
        localStorage.setItem('hasVisitedBefore', 'true');

      })();
    </script>
    <?php


    // Load CSS on the frontend if the admin bar tools are in the tool bar
    ?>
    <script defer id="superspeed-frontend-tools-css">
      (function() {
        function initialize() {
          const isEditor = document.querySelector('body.can-edit');
          const isAdminBar = document.getElementById('wpadminbar');
          if (!isEditor || !isAdminBar) {
            return; // Exit if conditions are not met
          }

          <?php
          global $ss_global_variable_for_version_control;

          if ($ss_global_variable_for_version_control) {
            $dynamically_loaded_file_versions = $ss_global_variable_for_version_control;
          } else {
            $dynamically_loaded_file_versions = time();
          }

          // Base URL for the styles, important to not hard-code the path for multisites and staging sites
          $style_base_url = plugins_url('assets/css/', dirname(__FILE__, 2));
          ?>

          // Non-hard coded path
          const styleBaseUrl = '<?php echo $style_base_url; ?>';

          // Declare the stylesheet 'version' variable with proper scope and initialization
          const version = <?php echo $dynamically_loaded_file_versions ?>;

          var link = document.createElement("link");
          link.href = styleBaseUrl + "wp-ss-tools.css?v=" + version;
          link.type = "text/css";
          link.rel = "stylesheet";
          link.id = "superspeed-wp-ss-tools";
          document.head.appendChild(link);
        }

        if (document.readyState === "complete") {
          initialize();
        } else {
          window.addEventListener('load', initialize);
        }
      })();
    </script>


    <?php
  }


  // Delay the plugin's inline styles
  public function delay_plugins_inline_style()
  {

    if (current_user_can('edit_others_posts')) {
    ?>
      <script id="superspeed-delay-inline-styles" defer>
        (function() {
          function activateStyles() {
            const isEditor = document.querySelector('body.can-edit');
            if (isEditor) {
              const delayedStyles = document.querySelectorAll('style[type="delayed-style"]');
              delayedStyles.forEach(function(style) {
                style.type = "text/css"; // Activate the styles
              });
            }
          }

          if (document.readyState === "complete") {
            // If the document has already loaded, activate styles immediately
            activateStyles();
          } else {
            // Otherwise, add an event listener for when the load event completes
            window.addEventListener('load', activateStyles);
          }
        })();
      </script>
<?php
    }
  }

  /*** SET GLOBAL PRELOAD VARIABLE ***/

  // Define the global variable that will be used to insert preload links into the <head>
  public function set_global_variable_for_preloading()
  {
    global $ss_global_variable_for_preloading_images;
    $ss_global_variable_for_preloading_images;

    global $ss_global_variable_for_inlining_css;
    $ss_global_variable_for_inlining_css;

    global $ss_global_variable_for_version_control;
    $ss_global_variable_for_version_control = false;
  }

  // Flag to check if output buffering started
  private $buffering_started_global_preload = false;

  public function buffer_start_global_preload()
  {

    if (\SuperSpeed\Utility::is_page_editor_or_wp_backend()) {
      return;
    }

    if ($_SERVER['REQUEST_METHOD'] !== 'GET') {
      return;
    }

    ob_start(array($this, "set_global_preload_variable"));
    $this->buffering_started_global_preload = true;
  }

  public function set_global_preload_variable($html)
  {

    // Check if the HTML is empty or does not start with '<'
    if (empty(trim($html)) || trim($html)[0] !== "<") {
      return $html;
    }

    // Check for XML declaration
    if (strcasecmp(substr($html, 0, 5), '<?xml') === 0) {
      return $html;
    }

    global $ss_global_variable_for_preloading_images;
    global $ss_global_variable_for_inlining_css;

    $img_parse_break = '<div id="img-parse-break" style="display:none"></div>';

    $viewport_meta = '<meta name="viewport" content="width=device-width, initial-scale=1">';

    $first_in_head = $ss_global_variable_for_preloading_images . "\n" .
      $img_parse_break . "\n" .
      $viewport_meta . "\n" .
      $ss_global_variable_for_inlining_css;

    // Ensure content in the variable
    if (isset($first_in_head) && !empty(trim($first_in_head))) {

      // Use a more flexible pattern for the <head> tag, allowing for possible attributes or spaces
      $html = preg_replace('/<head.*?>/', '$0' . $first_in_head, $html, 1);
    }

    return apply_filters('buffer_output', $html);
  }

  public function buffer_end_global_preload()
  {
    if ($this->buffering_started_global_preload) {
      if (ob_get_length() !== false) {
        ob_end_flush();
      }
      $this->buffering_started_global_preload = false;
    }
  }
}
