<?php

namespace SuperSpeed;

use SuperSpeed\Utility;

class PreloadLinks
{

  public function __construct()
  {
    // Exit if we could not get the current URL. Skipping static resource because it is not a page.
    $current_url = Utility::get_current_page_url();
    if (!$current_url) {
      return;
    }

    // If it's a frontend HTML page
    if (!Utility::is_page_editor_or_wp_backend()) {
      // Preload Fetch Links
      add_action('wp_footer', array($this, 'preload_fetch_links'), 100);

      // Prefetch links color coding
      add_action('wp_footer', array($this, 'prefetch_links_color_coding'));
    }
  }

  /*** PRELOAD FETCH LINKS ***/

  public function preload_fetch_links()
  {

    if (get_option('enable_preload_links') !== '1') {
      return;
    }

    // If we are not caching logged in users
    // If link preloading is disabled for logged-in users, stop execution.
    if (get_option('preload_links_for_logged_in_users') !== '1' && is_user_logged_in()) {
      return;
    }

    // Common setting
    $preload_links_test_mode = get_option('preload_links_test_mode') === '1';
    $preload_links_enable_mobile = get_option('preload_links_enable_mobile') === '1';
    $preload_links_mobile_width = get_option('preload_links_mobile_width', '800');
    $preload_links_max_retries = get_option('preload_links_max_retries', '3');
    $preload_links_retry_delay = get_option('preload_links_retry_delay', '1');

    // Desktop settings
    $preload_stylesheets_desktop = get_option('preload_stylesheets_desktop') === '1';
    $preload_javaScript_desktop = get_option('preload_javaScript_desktop') === '1';
    $preload_images_desktop = get_option('preload_images_desktop') === '1';
    $preload_links_fetch_timeout_desktop = get_option('preload_links_fetch_timeout_desktop', '5');
    $preload_links_server_response_desktop = get_option('preload_links_server_response_desktop', '2');

    // Mobile setings
    $preload_stylesheets_mobile = get_option('preload_stylesheets_mobile') === '1';
    $preload_javaScript_mobile = get_option('preload_javaScript_mobile') === '1';
    $preload_images_mobile = get_option('preload_images_mobile') === '1';
    $preload_links_initial_delay_mobile = get_option('preload_links_initial_delay_mobile', '3');
    $preload_links_next_file_delay_mobile = get_option('preload_links_next_file_delay_mobile', '5');
    $preload_links_fetch_timeout_mobile = get_option('preload_links_fetch_timeout_mobile', '7');
    $preload_links_server_response_mobile = get_option('preload_links_server_response_mobile', '3');


    // Adding info and configuration to the HTML
    // Using the specific image URL for server load check
    $image_url = plugins_url('assets/images/blank_transparent_webp_20x20.webp', dirname(__FILE__, 2));
?>
    <script id="SSpreFetchConfig" ss-fetch-config='{
  "COMMON": {
      "ENABLE_TEST_MODE": <?php echo $preload_links_test_mode ? "true" : "false"; ?>,
      "ENABLE_ON_MOBILE": <?php echo $preload_links_enable_mobile ? "true" : "false"; ?>,
      "MOBILE_WIDTH_THRESHOLD": <?php echo floatval($preload_links_mobile_width); ?>,
      "MAX_RETRIES": <?php echo floatval($preload_links_max_retries); ?>,
      "RETRY_DELAY_MS": <?php echo floatval($preload_links_retry_delay) * 1000; ?>,
      "TEST_FILE_URL": "<?php echo $image_url; ?>"
  },
  "MOBILE": {
      "PRELOAD_CSS": <?php echo $preload_stylesheets_mobile ? "true" : "false"; ?>,
      "PRELOAD_JS": <?php echo $preload_javaScript_mobile ? "true" : "false"; ?>,
      "PRELOAD_IMAGE": <?php echo $preload_images_mobile ? "true" : "false"; ?>,
      "MOBILE_DELAY_MS": <?php echo floatval($preload_links_initial_delay_mobile) * 1000; ?>,
      "MOBILE_DELAY_BETWEEN_LINKS_MS": <?php echo floatval($preload_links_next_file_delay_mobile) * 1000; ?>,
      "MOBILE_FETCH_TIMEOUT": <?php echo floatval($preload_links_fetch_timeout_mobile) * 1000; ?>,
      "RTT_THRESHOLD": <?php echo floatval($preload_links_server_response_mobile) * 1; ?>
  },
  "DESKTOP": {
      "PRELOAD_CSS": <?php echo $preload_stylesheets_desktop ? "true" : "false"; ?>,
      "PRELOAD_JS": <?php echo $preload_javaScript_desktop ? "true" : "false"; ?>,
      "PRELOAD_IMAGE": <?php echo $preload_images_desktop ? "true" : "false"; ?>,
      "DESKTOP_FETCH_TIMEOUT": <?php echo floatval($preload_links_fetch_timeout_desktop) * 1000; ?>,
      "RTT_THRESHOLD": <?php echo floatval($preload_links_server_response_desktop) * 1; ?>
  }
}'></script>

  <?php
  }


  // Prefetch links color coding
  public function prefetch_links_color_coding()
  {
    if (get_option('enable_preload_links') !== '1') {
      return;
    }

    if (get_option('preload_links_color_coding_enable') !== '1') {
      return;
    }

    $preload_links_color_coding_visibility = get_option('preload_links_color_coding_visibility', 'editing_permissions');

    $canEdit = '';
    if ($preload_links_color_coding_visibility === 'editing_permissions') {
      $canEdit = '.can-edit';
    }

    echo '<style id="super-speed-prefetch-links-color-coding">
      /* Preload Liks */

      '.$canEdit.'a[link-preload="aborted"] {
        animation: pulsate-aborted 1.5s infinite !important;
      }

      @keyframes pulsate-aborted {
        0% {
          outline: solid;
          outline-color: rgba(255, 0, 0, 0);
          outline-width: 0px;
          outline-offset: 0px;
        }

        50% {
          outline-color: rgba(255, 0, 0, 0.5);
        }

        100% {
          outline: solid;
          outline-color: rgba(255, 0, 0, 0);
          outline-width: 3px;
          outline-offset: 5px;
        }
      }

      '.$canEdit.'a[link-preload="waiting"] {
        animation: pulsate-waiting 1.5s infinite !important;
      }

      @keyframes pulsate-waiting {
        0% {
          outline: solid;
          outline-color: rgba(0, 0, 255, 0);
          outline-width: 0px;
          outline-offset: 0px;
        }

        50% {
          outline-color: rgba(0, 0, 255, 0.5);
        }

        100% {
          outline: solid;
          outline-color: rgba(0, 0, 255, 0);
          outline-width: 3px;
          outline-offset: 5px;
        }
      }

      '.$canEdit.'a[link-preload="loading"] {
        animation: pulsate-loading 1.5s infinite !important;
      }

      @keyframes pulsate-loading {
        0% {
          outline: solid;
          outline-color: rgba(255, 165, 0, 0);
          outline-width: 0px;
          outline-offset: 0px;
        }

        50% {
          outline-color: rgba(255, 165, 0, 0.5);
        }

        100% {
          outline: solid;
          outline-color: rgba(255, 165, 0, 0);
          outline-width: 3px;
          outline-offset: 5px;
        }
      }

      '.$canEdit.'a[link-preload="loaded"] {
        outline: solid rgba(0, 128, 0, 0.5) 3px !important;
      }

      '.$canEdit.'a[link-preload]:not([href*="http"]) {
        animation: unset !important;
      }

      '.$canEdit.'a[link-preload]:not([href*="http"]):hover {
        outline: dashed 2px orange !important;
        outline-offset: 2px !important;
      }

      '.$canEdit.'a[link-preload="current-page"]:hover {
        outline: dashed 2px green !important;
        outline-offset: 2px !important;
      }
    </style>';

  }
}
