<?php

namespace SuperSpeed;

class SSElementor
{
  public function __construct()
  {
    // If Elementor is active, return early
    if (!did_action('elementor/loaded')) {
      return;
    }

    // Add inline styles to the editor
    add_action('elementor/editor/after_enqueue_styles', array($this, 'add_custom_styles_to_elementor'));

    // Advanced Tab Background Settings
    add_action('elementor/element/common/_section_background/after_section_end', array($this, 'section_below_background'), 10, 2);

    // Settings Background Image Section
    add_action('elementor/element/section/section_background/after_section_end', array($this, 'background_image_settings__section_widget'), 10, 2);

    // Settings Background Overlay Image Section
    add_action('elementor/element/section/section_background_overlay/after_section_end', array($this, 'background_overlay_image_settings__section_widget'), 10, 2);

    // Settings Background Image Column
    add_action('elementor/element/column/section_style/after_section_end', array($this, 'background_image_settings__column_widget'), 10, 2);



    // Functions For Advanced Tab Background
    add_action('elementor/frontend/section/before_render', array($this, 'background_image_functions__advanced_tab'));

    // Functions For Background Image Section
    add_action('elementor/frontend/section/before_render', array($this, 'background_image_functions__section_widget'));

    // Functions For Background Image Overlay Section
    add_action('elementor/frontend/section/before_render', array($this, 'background_image_overlay_functions__section_widget'));

    // Functions For Background Image Column
    add_action('elementor/frontend/column/before_render', array($this, 'background_image_functions__column_widget'));
  }


  // Add inline styles to the editor
  public function add_custom_styles_to_elementor()
  {
    // Remove Elementor's native Image Size option for background images
    $custom_styles = '	
      .elementor-control-media .elementor-control-type-select.e-control-image-size {
         display: none;
      }
    ';

    // Add inline style to the edotor
    wp_add_inline_style('elementor-editor', $custom_styles);
  }


  // Advanced Tab Background Settings
  public function section_below_background($element, $args)
  {
    $this->background_image_settings_helper(
      $element,
      $args,
      'Optimize Background',
      'advanced_bg',
      'TAB_ADVANCED',
      'large',
      'large',
      'large'
    );
  }

  // Settings Background Image Section
  public function background_image_settings__section_widget($element, $args)
  {
    $this->background_image_settings_helper(
      $element,
      $args,
      'Image Optimization',
      'section',
      'TAB_STYLE',
      '1536x1536',
      'large',
      'large'
    );
  }

  // Settings Background Overlay Image Section
  public function background_overlay_image_settings__section_widget($element, $args)
  {
    $this->background_image_settings_helper(
      $element,
      $args,
      'Overlay Image Optimization',
      'section_overlay',
      'TAB_STYLE',
      '1536x1536',
      'large',
      'large'
    );
  }

  // Settings Background Image Column
  public function background_image_settings__column_widget($element, $args)
  {
    $this->background_image_settings_helper(
      $element,
      $args,
      'Image Optimization',
      'column',
      'TAB_STYLE',
      'large',
      'large',
      'large'
    );
  }






  public function background_image_settings_helper($element, $args, $controls_label, $widget, $tab_type, $default_desktop, $default_tablet, $default_mobile)
  {

    // Set WebP extension
    $webp_none = false;
    $image_jpg_webp = false;
    $image_webp = true;
    $webp_options = [];

    // Check if WooCommerce is active and add thumbnails
    if (!function_exists('is_plugin_active')) {
      include_once(ABSPATH . 'wp-admin/includes/plugin.php');
    }
    $is_woocommerce_active = is_plugin_active('woocommerce/woocommerce.php');

    $tab_const = '\Elementor\Controls_Manager::' . $tab_type;
    $element->start_controls_section(
      'ss_background_image_settings_' . $widget,
      [
        'tab' => constant($tab_const),
        'label' => esc_html__($controls_label, 'textdomain'),
      ]
    );

    $sizes = [
      'none' => __('none', 'elementor'),
      'thumbnail' => __('Thumbnail - 150 x 150', 'elementor'),
      'medium' => __('Medium - 300 x 300', 'elementor'),
      'medium_large' => __('Medium Large - 768 x 0', 'elementor'),
      'large' => __('Large - 1024 x 1024', 'elementor'),
      '1536x1536' => __('1536x1536 - 1536 x 1536', 'elementor'),
      '2048x2048' => __('2048x2048 - 2048 x 2048', 'elementor'),
    ];

    $woocommerce_options = [
      'woocommerce_thumbnail' => __('Woocommerce Thumbnail - 600 x 600', 'elementor'),
      'woocommerce_single' => __('Woocommerce Single - 600 x 0', 'elementor'),
      'woocommerce_gallery_thumbnail' => __('Woocommerce Gallery Thumbnail - 100 x 100', 'elementor'),
    ];

    $devices = [
      'desktop' => $default_desktop,
      'tablet' => $default_tablet,
      'mobile' => $default_mobile,
    ];

    foreach ($devices as $device => $default_size) {
      $size_options = $sizes;

      // Check if WooCommerce is active and merge options
      if ($is_woocommerce_active) {
        $size_options = array_merge($size_options, $woocommerce_options);
      }

      // Add the 'full' option
      $size_options['full'] = __('Full');

      // Add controls with the merged options
      $element->add_control(
        'ss_background_image_size_' . $device . '_' . $widget,
        [
          'label' => __('Size on ' . ucfirst($device), 'elementor'),
          'type' => \Elementor\Controls_Manager::SELECT,
          'options' => $size_options,
          'default' => $default_size,
          'description' => $device === 'mobile' ? __('Only full size images are used inside the editor.<br>View the page to see the resolution', 'elementor') : '',
        ]
      );
    }

    // WebP Extension
    if ($webp_none) {
      $webp_options['none'] = __('none', 'elementor');
    } elseif ($image_webp) {
      $webp_options['image.webp'] = __('image.webp', 'elementor');
    } elseif ($image_jpg_webp) {
      $webp_options['image.jpg.webp'] = __('image.jpg.webp', 'elementor');
    }

    // Determine default
    $default_webp = 'none'; // Default to 'none' if all are false
    if ($image_webp) {
      $default_webp = 'image.webp';
    } elseif ($image_jpg_webp) {
      $default_webp = 'image.jpg.webp';
    }

    $element->add_control(
      'ss_webp_file_extension_' . $widget,
      [
        'label' => __('WebP Extension', 'elementor'),
        'type' => \Elementor\Controls_Manager::SELECT,
        'options' => [
          'none' => __('none', 'elementor'),
          'image.webp' => __('image.webp', 'elementor'),
          'image.jpg.webp' => __('image.jpg.webp', 'elementor'),
        ],
        'default' => $default_webp,
        'description' => __('If there is a WebP version of this image, chose the extension that is used.', 'elementor'),
      ]
    );

    // Preload
    $element->add_control(
      'ss_preload_bg_image_' . $widget,
      [
        'label' => __('Preload Image', 'elementor'),
        'type' => \Elementor\Controls_Manager::SWITCHER,
        'label_on' => __('Yes', 'elementor'),
        'label_off' => __('No', 'elementor'),
        'return_value' => 'yes',
        'default' => 'no',
        'description' => __("Only preload one image per page. It should be the largest one at the top.", 'elementor'),
      ]
    );

    $element->end_controls_section();
  }






  //** FUNCTIONS **//



  // Functions For Advanced Tab Background
  public function background_image_functions__advanced_tab($element)
  {
    $widget = 'advanced_bg';
    $image_type = 'background_image';

    $css_template = '
    /*.elementor-element.elementor-element-{widget_id} > .elementor-widget-container*/

    [super-speed] .super-speed [data-id="{widget_id}"][data-element_type].elementor-element.elementor-element-{widget_id} {
      background-image: url({desktop_image_url});
    }
    @media screen and (max-width: 1023px) {
      [super-speed] .super-speed [data-id="{widget_id}"][data-element_type].elementor-element.elementor-element-{widget_id} {
          background-image: url({tablet_image_url});
      }
    }
    @media screen and (max-width: 766px) {
      [super-speed] .super-speed [data-id="{widget_id}"][data-element_type].elementor-element.elementor-element-{widget_id} {
          background-image: url({mobile_image_url});
      }
    }';
    $this->background_image_helper($element, $widget, $image_type, $css_template);
  }


  // Functions For Background Image Section
  public function background_image_functions__section_widget($element)
  {
    $widget = 'section';
    $image_type = 'background_image';

    $css_template = '
    [super-speed] .super-speed [data-id="{widget_id}"][data-element_type].elementor-element.elementor-element-{widget_id} {
      background-image: url({desktop_image_url});
    }
    @media screen and (max-width: 1023px) {
      [super-speed] .super-speed [data-id="{widget_id}"][data-element_type].elementor-element.elementor-element-{widget_id} {
          background-image: url({tablet_image_url});
      }
    }
    @media screen and (max-width: 766px) {
      [super-speed] .super-speed [data-id="{widget_id}"][data-element_type].elementor-element.elementor-element-{widget_id} {
          background-image: url({mobile_image_url});
      }
    }';
    $this->background_image_helper($element, $widget, $image_type, $css_template);
  }


  // Functions For Background Image Overlay Section
  public function background_image_overlay_functions__section_widget($element)
  {
    $widget = 'section_overlay';
    $image_type = 'background_overlay_image';

    $css_template = '
    [has_bg_image="super-speed"][data-id="{widget_id}"][data-element_type].elementor-element.elementor-element-{widget_id} > .elementor-background-overlay {
      background-image: url({desktop_image_url});
      }
      @media screen and (max-width: 1023px) {
        [has_bg_image="super-speed"][data-id="{widget_id}"][data-element_type].elementor-element.elementor-element-{widget_id} > .elementor-background-overlay {
          background-image: url({tablet_image_url});
        }
      }
      @media screen and (max-width: 766px) {
        [has_bg_image="super-speed"][data-id="{widget_id}"][data-element_type].elementor-element.elementor-element-{widget_id} > .elementor-background-overlay {
          background-image: url({mobile_image_url});
        }
      }';
    $this->background_image_helper($element, $widget, $image_type, $css_template);
  }





  // Functions For Background Image Column
  public function background_image_functions__column_widget($element)
  {
    $widget = 'column';
    $image_type = 'background_image';

    $css_template = '
      [data-id="{widget_id}"][data-element_type].elementor-element.elementor-element-{widget_id}:not(.elementor-motion-effects-element-type-background) > .elementor-widget-wrap {
        background-image: url({desktop_image_url});
      }
      @media screen and (max-width: 1023px) {
        [data-id="{widget_id}"][data-element_type].elementor-element.elementor-element-{widget_id}:not(.elementor-motion-effects-element-type-background) > .elementor-widget-wrap {
          background-image: url({tablet_image_url});
        }
      }
      @media screen and (max-width: 766px) {
        [data-id="{widget_id}"][data-element_type].elementor-element.elementor-element-{widget_id}:not(.elementor-motion-effects-element-type-background) > .elementor-widget-wrap {
          background-image: url({mobile_image_url});
        }
      }';
    $this->background_image_helper($element, $widget, $image_type, $css_template);
  }


  // Helper function
  public function background_image_helper($element, $widget, $image_type, $css_template)
  {
    $settings = $element->get_settings_for_display();

    // If the background is set from the Advanced tab, get the name of the current widget
    if ($widget === 'advanced_bg') {
      $widget = $element->get_name();
    }

    $desktop_key = 'ss_background_image_size_desktop_' . $widget;
    $tablet_key = 'ss_background_image_size_tablet_' . $widget;
    $mobile_key = 'ss_background_image_size_mobile_' . $widget;

    $device_sizes = [
      'desktop' => $settings[$desktop_key],
      'tablet' => $settings[$tablet_key],
      'mobile' => $settings[$mobile_key]
    ];

    $attachment_id = attachment_url_to_postid($settings[$image_type]['url']);

    // Early exit if no background image URL or sizes are set for any of the devices
    if (
      !$attachment_id ||
      (empty($device_sizes['desktop']) &&
        empty($device_sizes['tablet']) &&
        empty($device_sizes['mobile']))
    ) {
      return;
    }

    // Add attribute mainly to be used to add a gray placeholder, untill the image is loaded
    if ($attachment_id) {
      $element->add_render_attribute('_wrapper', 'has_bg_image', 'super-speed');
    }


    // Standard sizes
    $sizes_priority = ['thumbnail', 'medium', 'medium_large', 'large', '1536x1536', '2048x2048', 'full'];

    // Ensure the is_plugin_active function is available
    if (!function_exists('is_plugin_active')) {
      include_once(ABSPATH . 'wp-admin/includes/plugin.php');
    }

    // Check if WooCommerce is active and merge sizes
    if (is_plugin_active('woocommerce/woocommerce.php')) {
      $woocommerce_sizes = ['woocommerce_thumbnail', 'woocommerce_single', 'woocommerce_gallery_thumbnail'];
      $sizes_priority = array_merge($sizes_priority, $woocommerce_sizes);
    }

    // Get URLs based on the selected thumbnail
    foreach ($device_sizes as $device => &$size) {
      // Set an empty string, if the user selected 'none'
      if ($size === 'none') {
        $size = '';
      } else {
        foreach ($sizes_priority as $priority_size) {
          if ($priority_size === $size || $priority_size === 'none') {
            $size = wp_get_attachment_image_src($attachment_id, $priority_size)[0];
            if ($size) break;
          }
        }
      }
    }

    // Extract the individual URLs for the devices
    $desktop_image_url = $device_sizes['desktop'];
    $tablet_image_url = $device_sizes['tablet'];
    $mobile_image_url = $device_sizes['mobile'];

    // WebP handling
    $sizes = ['desktop', 'tablet', 'mobile'];
    $pattern = '/\.(jpeg|jpg|png|gif|bmp)$/i';

    foreach ($sizes as $size) {
      $image_url_var_name = $size . '_image_url';

      // Skip if the image is already WebP
      if (substr($image_url_var_name, -5) !== '.webp' && isset($settings['ss_webp_file_extension_' . $widget])) {
        // This should be set in the plugin settings, and only have a switch in the widget
        if ($settings['ss_webp_file_extension_' . $widget] === 'image.jpg.webp') {
          $image_url_var_name .= '.webp';
        } elseif ($settings['ss_webp_file_extension_' . $widget] === 'image.webp') {
          $image_url_var_name = preg_replace($pattern, '.webp', $image_url_var_name);
        }

        // Revert to original image, if the WebP version doesn't exist
        $potential_webp_path = str_replace(get_site_url(), ABSPATH, $image_url_var_name);
        if (!file_exists($potential_webp_path)) {
          $image_url_var_name = ${$size . '_image_url_original'};
        }
      }
    }



    if (isset($settings['ss_preload_bg_image_' . $widget]) && $settings['ss_preload_bg_image_' . $widget] == 'yes') {

      // Add class
      $element->add_render_attribute('_wrapper', 'class', 'ss-bg-preloaded');

      // Add attribute
      $element->add_render_attribute('_wrapper', 'bg_preloaded', 'super-speed');

      if (!empty($mobile_image_url) && $device_sizes['mobile'] !== 'none') {
        $preload .= "\n" . '<link rel="preload" href="' . $mobile_image_url . '" as="image" fetchpriority="high" media="(max-width: 767px)">' . "\n";
      }

      if (!empty($tablet_image_url) && $device_sizes['tablet'] !== 'none') {
        $preload .= '<link rel="preload" href="' . $tablet_image_url . '" as="image" fetchpriority="high" media="(min-width: 990px) and (max-width: 1024px)">' . "\n";
      }

      if (!empty($desktop_image_url) && $device_sizes['desktop'] !== 'none') {
        $preload .= '<link rel="preload" href="' . $desktop_image_url . '" as="image" fetchpriority="high" media="(min-width: 1024.1px)">' . "\n";
      }


      // Add the HTML to the global variable
      global $ss_global_variable_for_preloading_images;
      $ss_global_variable_for_preloading_images .= $preload;
    }

    // Set the CSS or our custom thumbnails
    $widget_id = $element->get_id();
    $set_css = '<style id="ss-background-image-' . $widget_id . '">';
    $set_css .= str_replace(
      ['{widget_id}', '{desktop_image_url}', '{tablet_image_url}', '{mobile_image_url}'],
      [$widget_id, $desktop_image_url, $tablet_image_url, $mobile_image_url],
      $css_template
    );
    $set_css .= '</style>' . "\n";

    // Add the styles to the global variable
    global $ss_global_variable_for_inlining_css;
    $ss_global_variable_for_inlining_css .= $set_css;
  }
}
