<?php

namespace SuperSpeed;

use SuperSpeed\Utility;

class StyleHandler
{

  // Store CSS repeater filed data in a variable
  // private $repeater_data;

  // Cache the result of checking which page we are on, before removing stylsheets in critical_css_delay_external_files
  private $buffering_started_critical_css = false;

  // cache the result of checking which page we are on, before removing stylsheets
  private $buffering_started_remove_css = false;

  // Delay CSS cache the result of checking which page we are on
  private $buffering_started_delay_css = false;

  // Cache used CSS file URL
  private $used_css_file_template = false;

  // Used CSS cache the result of checking which page we are on
  private $used_css_should_run_on_page = false;

  // Chache the result if the Used CSS file exists
  private $used_css_file_exists = false;


  public function __construct()
  {
    // Exit if we could not get the current URL. Skipping static resource because it is not a page.
    if (!Utility::get_current_page_url()) {
      return;
    }

    // Fetch repeater data once in the constructor
    // $this->repeater_data = $this->get_repeater_data();

    // Cache the result of the function
    $this->used_css_should_run_on_page = Utility::getting_used_css_on_page();


    // Find all CPTs, so that they can be used in the repeater fields dropdown, when called by Fetch API, in the file critical-css-ui.js
    add_action('wp_ajax_get_public_post_types', array($this, 'get_public_post_types_callback'));

    // AJAX actions for saving Used CSS
    add_action('wp_ajax_save_used_css', array($this, 'get_used_css_by_fetch_api_ajax'));
    add_action('wp_ajax_nopriv_save_used_css', array($this, 'get_used_css_by_fetch_api_ajax'));

    // When the post is edited, retrieve and delete the file
    add_action('save_post', array($this, 'if_post_is_edited_remove_used_css_file'));

    // Delete the Used CSS file from the page we are on, for the specific page/template
    add_action('wp_ajax_delete_used_css', array($this, 'handle_delete_used_css'));

    // Delete all Used CSS files from the frontend
    add_action('wp_ajax_delete_all_used_css', array($this, 'handle_delete_all_used_css'));


    // Dynamically load files JS of frontend tools for CSS
    add_action('rest_api_init', [$this, 'register_rest_routes']);
    // Loading get-used-css.js before get-images-data.js
    add_action('wp_footer', [$this, 'enqueue_scripts_dynamically'], -1 * PHP_INT_MAX);

    // If it's a frontend HTML page
    if (!Utility::is_page_editor_or_wp_backend()) {

      // Add critical css to the <head>
      // add_action('wp_head', array($this, 'add_critical_css'), -1 * PHP_INT_MAX);

      // Delay Repeater
      // add_action('template_redirect', array($this, 'buffer_start_crictical_css_repater'), 9999);
      // add_action('shutdown', array($this, 'buffer_end_crictical_css_repater'), PHP_INT_MAX);

      // Place template name in the critical css frontend UI
      // add_action('wp_footer', array($this, 'get_and_echo_page_type'), 9999);

      // Place url / page slug in the critical css frontend UI
      // add_action('wp_footer', array($this, 'get_and_echo_page_slug'), 9999);

      // Preload links to create asynchronous loading
      add_filter('style_loader_tag', array($this, 'add_preload_link_for_async_loading'), 10, 2);

      // Remove stylesheets
      add_action('template_redirect', array($this, 'buffer_start_remove_styles'), 9999);
      add_action('shutdown', array($this, 'buffer_end_remove_styles'), PHP_INT_MAX);

      // Delay CSS global
      add_action('template_redirect', array($this, 'buffer_start_global_delay'), 9999);
      add_action('shutdown', array($this, 'buffer_end_global_delay'), PHP_INT_MAX);

      // Used CSS
      add_action('wp_head', array($this, 'html_elements_for_used_css'), 999);

      // Cache used CSS file URL. Check if the file exists. Set meta if the page is edited.
      add_action('wp', array($this, 'cache_template_type_and_if_file_exists_and_set_meta_if_edited'));
    }
  }


  /*** CREATE CUSTOM FUNCTIONS TO CHECK WHICH PAGE WE ARE ON ***/

  // Custom is_single_page(). Using short-circuit evaluation
  public function ss_is_single_page()
  {
    // Directly return false for any admin, WooCommerce-specific pages, or special WordPress pages
    if (is_admin() || is_home() || is_front_page() || is_search() || is_archive() || is_404()) {
      return false;
    }

    // Check WooCommerce conditions only if WooCommerce is active
    if (class_exists('WooCommerce')) {
      if (
        is_shop() || is_product_category() || is_product_tag() || is_product() ||
        is_cart() || is_checkout() || is_account_page() || is_wc_endpoint_url()
      ) {
        return false;
      }
    }

    // If it's a singular 'page', and none of the above conditions are met, return true
    return is_singular('page');
  }


  /*  // Custom is_single_page(). Using short-circuit evaluation
  public function ss_is_single_page()
  {
    return is_singular('page') && // This checks if it's a singular page
      !is_home() &&
      !is_front_page() &&
      !is_search() &&
      !is_archive() &&
      !is_404() &&
      !is_admin() &&
      !(class_exists('WooCommerce') && (is_shop() ||
        is_product_category() ||
        is_product_tag() ||
        is_product() ||
        is_cart() ||
        is_checkout() ||
        is_account_page() ||
        is_wc_endpoint_url()
      ));
  }
*/

  // Cutom is_single('post') for the defult blog
  public function ss_is_single_blog_post()
  {
    global $post;
    if (!is_a($post, 'WP_Post')) {
      return;
    }
    global $wp_query;
    $post_type = get_post_type($wp_query->post->ID);
    return is_single() && $post_type == 'post';
  }

  // Is single custom post type check
  public function ss_is_single_custom($post_type_name)
  {
    global $wp_query;
    $post_type = get_post_type($wp_query->post->ID);
    return is_single() && $post_type == $post_type_name;
  }

  /*** INJECT CRITICAL REPEATER DROPDOWN LISTS WITH CPT PAGE CHECKS, FOR HTML AND JS FILES ***/

  // Find all CPTs, for the repeater fields dropdown, when called by Fetch API, in the file critical-css-ui.js
  function get_public_post_types_callback()
  {
    // Get public custom post types
    $public_post_types = get_post_types(array('_builtin' => false, 'public' => true), 'objects');

    // Define an array of post types to exclude
    $exclude_post_types = array('product'); // WooCommerce already has a function for the single product

    $post_type_options = array();

    // check if there are any public custom post types
    if (!empty($public_post_types)) {
      foreach ($public_post_types as $post_type) {
        // Skip post types without archives or those that are excluded
        if (!$post_type->has_archive || in_array($post_type->name, $exclude_post_types)) {
          continue;
        }

        // The string function name, e.g., 'ss_is_single_product'
        $function_name = 'ss_single_cpt_' . $post_type->name;

        // The human readable function name, e.g., 'Single Product'
        $display_name = 'Single ' . $post_type->labels->singular_name;

        // add to options array
        $post_type_options[$function_name] = $display_name;
      }
    }

    echo json_encode($post_type_options);

    wp_die(); // this is required to terminate immediately and return a proper response
  }

  // Find all CPTs, to be used in 'get_and_echo_page_type', and in the repeater fields dropdown, in the file css.php
  public function get_custom_post_type_options()
  {

    // Get public custom post types
    $public_post_types = get_post_types(array('_builtin' => false, 'public' => true), 'objects');

    // Define an array of post types to exclude
    $exclude_post_types = array('product'); // WooCommerce already has a function for the single product

    $options = array();

    // Check if there are any public custom post types
    if (!empty($public_post_types)) {
      foreach ($public_post_types as $post_type) {
        // Skip post types without archives or those that are excluded
        if (!$post_type->has_archive || in_array($post_type->name, $exclude_post_types)) {
          continue;
        }

        // The string function name, e.g., 'ss_is_single_product'
        $function_name = 'ss_single_cpt_' . $post_type->name;

        // The human readable function name, e.g., 'Single Product'
        $display_name = 'Single ' . $post_type->labels->singular_name;

        // add to options array
        $options[$function_name] = $display_name;
      }
    }

    return $options;
  }


  /*** FETCH REPEATER DATA FROM THE DATABASE ***/
  /*
  private function get_repeater_data()
  {

    global $wpdb;
    $table_name = $wpdb->prefix . 'critical_css_repeater_fields';
    $repeater_data = $wpdb->get_results(
      "SELECT critical_css_page_templates, critical_css_page_url, critical_css, critical_css_radio, critical_exclude_links, critical_delay_specific_links FROM {$table_name}",
      ARRAY_A
    );
    // If there was a database error, log it
    if ($repeater_data === null) {
      error_log('Database error when fetching repeater data: ' . $wpdb->last_error);
      return [];
    }

    return $repeater_data;
  }
*/

  /*** PLACE PAGE TEMPLATE AND URL SLUG IN THE CRITICAL CSS UI ***/
  /*
  // Place template name in the critical css frontend UI
  function get_and_echo_page_type()
  {
    $options = array(
      'is_front_page' => 'Front Page',
      'is_home' => 'Posts Page',
      'ss_is_single_blog_post' => 'Single Blog Post',
      'ss_is_single_page' => 'Single Page',
      'is_archive' => 'Archive Page',
      'is_category' => 'Category Archive Page',
      'is_tag' => 'Tag Archive Page',
      'is_tax' => 'Custom Taxonomy Archive Page',
      'is_author' => 'Author Archive Page', // line 225
      'is_search' => 'Search Results Page',
      'is_404' => '404 Not Found Page',
      'is_post_type_archive' => 'Post Type Archive Page',
    );

    // check if WooCommerce is active
    if (class_exists('WooCommerce')) {
      // add WooCommerce related options
      $options['is_shop'] = 'WooCommerce Shop Page';
      $options['is_product_category'] = 'WooCommerce Product Category Page';
      $options['is_product_tag'] = 'WooCommerce Product Tag Page';
      $options['is_product'] = 'WooCommerce Single Product Page';
      $options['is_cart'] = 'WooCommerce Cart Page';
      $options['is_checkout'] = 'WooCommerce Checkout Page';
      $options['is_account_page'] = 'WooCommerce Account Page';
    }

    // Get custom post type options
    $custom_post_type_options = $this->get_custom_post_type_options();

    // Merge the custom post type options with the existing options
    $options = array_merge($options, $custom_post_type_options);

    $matches = array();
    foreach ($options as $function => $description) {
      if (stripos($function, 'ss_single_cpt_') === 0) {
        $post_type_name = str_replace('ss_single_cpt_', '', $function);
        if ($this->ss_is_single_custom($post_type_name)) {
          $matches[] = $description;
        }
      } elseif (method_exists($this, $function)) {
        if ($this->$function() === true) {
          $matches[] = $description;
        }
      } elseif (function_exists($function)) {
        if ($function() === true) {
          $matches[] = $description;
        }
      }
    }

    if (count($matches) > 0) {
      echo '<span id="ss-page-template" style="display:none">Page Template: 
          <span class="ss-slug-n-template">' . implode('<br>', $matches) . '</span>
          </span>';
    } else {
      echo "Unknown page type.";
    }
  }
*/
  /*
  // Place URL slug in the critical CSS frontend UI
  public function get_and_echo_page_slug()
  {
    // Prevent presenting /home/ as the slug for the front page
    if (is_home() || is_front_page()) {
      echo '<span id="ss-page-slug" style="display:none">Part of The URL: 
                <span class="ss-slug-n-template">*the Home page doesn&#39;t have a URL slug*</span>
              </span>';
    } else {
      $obj = get_queried_object();
      if (isset($obj->post_name)) {
        $post_type = $obj->post_type;
        if ($post_type == 'page') {
          echo '<span id="ss-page-slug" style="display:none">Part of The URL: 
                  <span class="ss-slug-n-template">/' . $obj->post_name . '/</span>
                </span>';
        } else {
          echo '<span id="ss-page-slug" style="display:none">Part of The URL: 
                  <span class="ss-slug-n-template">/' . $post_type . '/' . $obj->post_name . '/</span>
                </span>';
        }
      }
    }
  }
*/

  /*** ADD CRITICAL CSS TO THE HEAD ***/
  /*
  public function add_critical_css()
  {

    if (get_option('critical_css_enable') !== '1') {
      return;
    }

    $current_path = parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH);

    // Iterate through repeater field data
    foreach ($this->repeater_data as $data) {
      $should_add_css = false;

      // Check if dropdown option is not empty
      if (!empty($data['critical_css_page_templates'])) {
        // Check if it's a native WordPress function or a method of this class
        if (function_exists($data['critical_css_page_templates']) || method_exists($this, $data['critical_css_page_templates'])) {
          $should_add_css = method_exists($this, $data['critical_css_page_templates']) ?
            call_user_func(array($this, $data['critical_css_page_templates'])) :
            call_user_func($data['critical_css_page_templates']);
        }
        // Check if it's a dynamically created function for a custom post type
        elseif (stripos($data['critical_css_page_templates'], 'ss_single_cpt_') === 0) {
          $post_type = str_replace('ss_single_cpt_', '', $data['critical_css_page_templates']);
          $should_add_css = $this->ss_is_single_custom($post_type);
        }
      }

      // If dropdown option is not provided, check if URL contains the specified string
      elseif (empty($data['critical_css_page_templates']) && !empty($data['critical_css_page_url'])) {
        $string = $data['critical_css_page_url'];
        $should_add_css = stripos($current_path, $string) !== false;
      }

      // If either condition is met, add CSS to the head
      if ($should_add_css && !empty($data['critical_css'])) {
        echo '<style id="ss-critical-css">' . $data['critical_css'] . '</style>';
      }
    }
  }
*/

  /*** CRITICAL CSS BUFFER FOR REPACING 'HREF' ATTRIBUTES WITH 'DELAY-HREF' ***/
  /*
  // Start buffer
  public function buffer_start_crictical_css_repater()
  {
    if (get_option('critical_css_enable') !== '1') {
      return;
    }

    // If the user has enabled auto-setting LCP images, if the page is new, edited, or the data is missing
    // Exit if `$post` is not a WP_Post object.
    global $post;
    if (!is_a($post, 'WP_Post')) {
      return;
    }

    // If the user has enabled auto-setting LCP images
    // If the data is missing
    // If the page is new or edited
    // And if we should be getting image data on this page
    if (Utility::getting_images_data()) {
      return;
    }

    // If the user has enabled auto-setting Used CSS
    // If the CSS file is missing
    // If the page is new or edited
    // And if we should be getting Used CSS on this page
    if (Utility::getting_used_css_data()) {
      return;
    }

    if ($_SERVER['REQUEST_METHOD'] !== 'GET') {
      return;
    }

    global $wpdb;
    $table_name = $wpdb->prefix . 'critical_css_repeater_fields';

    $repeater_data = $wpdb->get_results(
      "SELECT * FROM {$table_name}",
      ARRAY_A
    );

    $current_path = parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH);

    foreach ($repeater_data as $data) {
      $should_start_buffer = false;

      // We have 3 types of functions that check what page we are on: 
      // WP native, custom from this class, and dynamically created for CPTs

      // Check if dropdown option is not empty
      if (!empty($data['critical_css_page_templates'])) {
        // Check if it's a native WordPress function or a method of this class
        if (function_exists($data['critical_css_page_templates']) || method_exists($this, $data['critical_css_page_templates'])) {
          $should_start_buffer = method_exists($this, $data['critical_css_page_templates']) ?
            call_user_func(array($this, $data['critical_css_page_templates'])) :
            call_user_func($data['critical_css_page_templates']);
        }
        // Check if it's a dynamically created function for a custom post type
        elseif (stripos($data['critical_css_page_templates'], 'ss_single_cpt_') === 0) {
          $post_type = str_replace('ss_single_cpt_', '', $data['critical_css_page_templates']);
          $should_start_buffer = $this->ss_is_single_custom($post_type);
        }
      }

      // If dropdown option is not provided, check if URL path matches the specified string
      elseif (empty($data['critical_css_page_templates']) && !empty($data['critical_css_page_url'])) {
        $string = $data['critical_css_page_url'];
        $should_start_buffer = stripos($current_path, $string) !== false;
      }

      if ($should_start_buffer) {
        ob_start(array($this, "critical_css_delay_external_files"));
        $this->buffering_started_critical_css = true;
        break;
      }
    }
  }

  // Delay stylesheets for each repeater based on the selections
  public function critical_css_delay_external_files($html)
  {

    $dom = Utility::processHTML($html);

    // Check if the returned value is an instance of DOMDocument
    if (!$dom instanceof \DOMDocument) {
      return $html;
    }

    $always_exclude_list = array(
      '/wp-includes/css/dashicons.css',
      '/wp-includes/css/admin-bar.css',
    );

    global $wpdb;
    $table_name = $wpdb->prefix . 'critical_css_repeater_fields';

    $repeater_data = $wpdb->get_results(
      "SELECT * FROM {$table_name}",
      ARRAY_A
    );

    foreach ($repeater_data as $data) {
      $valid_radio_values = ['critical_delay_all_links_css', 'critical_delay_specific_links_css', 'critical_delay_exclude_links_css'];
      if (!in_array($data['critical_css_radio'], $valid_radio_values)) {
        continue;
      }

      $delay_all = $data['critical_css_radio'] == 'critical_delay_all_links_css' ? true : false;
      $delay_specific = $data['critical_css_radio'] == 'critical_delay_specific_links_css' ? true : false;
      $delay_exclude = $data['critical_css_radio'] == 'critical_delay_exclude_links_css' ? true : false;

      $exclude_list = array_map('trim', explode("\n", $data['critical_exclude_links']));
      $include_list = array_map('trim', explode("\n", $data['critical_delay_specific_links']));

      // Iterate over each link tag
      foreach ($dom->getElementsByTagName('link') as $link) {
        if ($link->getAttribute('rel') !== 'stylesheet') {
          continue; // Skip non-stylesheet link tags
        }

        $href = $link->getAttribute('href');

        foreach (array_keys($always_exclude_list) as $item) {
          if (stripos($href, $item) !== false) {
            continue 2;
          }
        }

        if ($delay_specific) {
          foreach ($include_list as $item) {
            if (stripos($href, $item) !== false) {
              if ($link->hasAttribute('href')) {
                $link->setAttribute("delay-href", $href);
                $link->removeAttribute("href");
              }
            }
          }
        }

        if ($delay_exclude) {
          foreach ($exclude_list as $item) {
            if (stripos($href, $item) !== false) {
              continue 2;  // Skip to the next link if current one is in the exclude list
            }
          }
          // Only set the delay-href if the link is not in the exclude list
          if ($link->hasAttribute('href')) {
            $link->setAttribute("delay-href", $href);
            $link->removeAttribute("href");
          }
        }

        if ($delay_all) {
          if ($link->hasAttribute('href')) {
            $link->setAttribute("delay-href", $href);
            $link->removeAttribute("href");
          }
        }
      }

      return $dom->saveHTML();
    }
  }

  // Already performed the check if we are on the right page, and stored it in the variable '$this->buffering_started_critical_css'
  public function buffer_end_crictical_css_repater()
  {
    if ($this->buffering_started_critical_css) {
      if (ob_get_length() !== false) {
        ob_end_flush();
      }
      $this->buffering_started_critical_css = false;
    }
  }
*/

  /*** PRELOAD LINKS TO CREATE ASYNCHRONOUS LOADING ***/

  public function add_preload_link_for_async_loading($html)
  {

    if (get_option('preload_styles') !== '1') {
      return $html;
    }

    // If we are removing or delaying CSS, don't preload the CSS files
    if (
      $this->buffering_started_remove_css !== false
      || $this->buffering_started_critical_css !== false
      || $this->buffering_started_delay_css !== false
    ) {
      return $html;
    }

    // The media attribute of the original link element.
    $media = 'all';
    if (preg_match("/media='(.+?)'/", $html, $matches)) {
      $media = $matches[1];
    }

    // The href attribute of the original link element.
    $href = '';
    if (preg_match("/href='(.+?)'/", $html, $matches)) {
      $href = $matches[1];
    }

    // The HTML link element for preloading the style sheet.
    $preload = '<link rel="preload" href="' . $href . '" as="style" media="' . $media . '" fetchpriority="high">';

    return $preload . "\n" . $html;
  }


  /*** REMOVE FRONTEND STYLES ***/

  // Start buffer
  public function buffer_start_remove_styles()
  {
    if ($_SERVER['REQUEST_METHOD'] !== 'GET') {
      return;
    }

    if (get_option('enable_dequeue_styles') !== '1') {
      return;
    }

    // Exit if `$post` is not a WP_Post object.
    global $post;
    if (!is_a($post, 'WP_Post')) {
      return;
    }

    // If the user has enabled auto-setting LCP images
    // If the data is missing
    // If the page is new or edited
    // And if we should be getting image data on this page
    if (Utility::getting_images_data()) {
      return;
    }

    // If the user has enabled auto-setting Used CSS
    // If the CSS file is missing
    // If the page is new or edited
    // And if we should be getting Used CSS on this page
    if (Utility::getting_used_css_data()) {
      return;
    }

    // PAGES - Get radio buttons and text areas
    $checkedDeqCSSPagesRadio = get_option('remove_pages_css_radio_checked');
    $exclude_list = get_option('dequeue_exclude_pages_css_textarea') ? array_map('trim', explode("\n", get_option('dequeue_exclude_pages_css_textarea'))) : [];
    $include_list = get_option('dequeue_spacific_pages_css_textarea') ? array_map('trim', explode("\n", get_option('dequeue_spacific_pages_css_textarea'))) : [];

    // Add home page settings checkboxses
    $exclude_home = get_option('dequeue_css_exclude_home') === '1';
    $include_home = get_option('dequeue_css_include_home') === '1';

    $all_pages = $checkedDeqCSSPagesRadio === 'dequeue_all_pages_css';
    $exclude_pages = $checkedDeqCSSPagesRadio === 'exclude_remove_pages_css';
    $include_pages = $checkedDeqCSSPagesRadio === 'dequeue_specific_pages_css';

    $current_URL = '';
    $is_home = '';

    if (!$all_pages) {
      $is_home = Utility::is_homepage();
    }

    // Run on all pages
    if ($all_pages) {
      ob_start(array($this, "remove_external_stylesheets"));
      $this->buffering_started_remove_css = true;
      return;
    }

    // Run on all except these excluded pages
    elseif ($exclude_pages) {
      if ($is_home && $exclude_home) {
        // Don't run on the homepage
        $this->buffering_started_remove_css = false;
        return;
      } else {
        // Don't run on pages from the list
        if (!empty($exclude_list)) {
          foreach ($exclude_list as $string) {
            $normalizedString = Utility::normalizeUrlString($string);
            if (stripos($current_URL, $normalizedString) !== false) {
              $this->buffering_started_remove_css = false;
              return;
            }
          }
        }
      }

      // Run if we are not on an excluded page
      $this->buffering_started_remove_css = true;
      ob_start(array($this, "remove_external_stylesheets"));
      return;
    }

    // Run only on specified pages
    elseif ($include_pages) {
      if ($is_home && $include_home) {
        // Run on Home page
        if ($include_home && Utility::is_homepage()) {
          ob_start(array($this, "remove_external_stylesheets"));
          $this->buffering_started_remove_css = true;
          return;
        }
      } else {
        foreach ($include_list as $string) {
          if (!empty($include_list)) {
            $normalizedString = Utility::normalizeUrlString($string);
            if (stripos($current_URL, $normalizedString) !== false) {
              ob_start(array($this, "remove_external_stylesheets"));
              $this->buffering_started_remove_css = true;
              return;
            }
          }
        }
      }
    }
  }

  // Remove stylesheets for each repeater based on the selections
  public function remove_external_stylesheets($html)
  {

    $dom = Utility::processHTML($html);

    // Check if the returned value is an instance of DOMDocument
    if (!$dom instanceof \DOMDocument) {
      return $html;
    }

    // Remove all styles
    $xpath = new \DOMXPath($dom);

    // LINKS - Get radio buttons and text areas
    $checkedDeqLinksCSSRadio = get_option('dequeue_css_links_radio_checked');
    $exclude_list = get_option('dequeue_exclude_links_css_textarea') ? array_map('trim', explode("\n", get_option('dequeue_exclude_links_css_textarea'))) : [];
    $include_list = get_option('dequeue_spacific_links_css_textarea') ? array_map('trim', explode("\n", get_option('dequeue_spacific_links_css_textarea'))) : [];
    $exclude_3rd_party = get_option('dequeue_css_exclude_3rd_party') === '1';
    $exclude_google_fonts = get_option('used_css_exclude_google') === '1';

    // Initialize the array with elements that are never removed
    $never_remove_list = [];

    // Check if the user is logged in
    if (is_user_logged_in()) {
      $current_user = wp_get_current_user();
      $editable_roles = ['administrator', 'editor', 'author', 'contributor'];

      // Convert editable roles to an associative array for faster lookup
      $editable_roles_lookup = array_flip($editable_roles);

      // Check if any of the current user's roles match the editable roles
      foreach ($current_user->roles as $role) {
        if (isset($editable_roles_lookup[$role])) {
          // If the user's role is in the editable roles, set the never remove list with necessary files
          $never_remove_list = [
            '/wp-includes/css/dashicons',
            '/wp-includes/css/admin-bar',
          ];
          break;  // Break the loop once a match is found
        }
      }
    }

    // Conditionally add elements to the array
    if ($exclude_google_fonts) {
      $never_remove_list[] = 'fonts.googleapis.com/css';
    }

    // Exclude 3rd party styles
    if ($checkedDeqLinksCSSRadio === 'dequeue_exclude_links_css' && $exclude_3rd_party) {

      // Get the site URL and parse it to get components
      $parsed_url = parse_url(get_site_url());
      $base_path = $parsed_url['scheme'] . '://' . $parsed_url['host'];

      // Work only with stylesheets that start with the domain
      // Target delayed links as well
      $nodes = $xpath->query("//link[@rel='stylesheet' and (@href[starts-with(., '{$base_path}')] or @delay-href[starts-with(., '{$base_path}')])]");
    } else {
      // Select <link rel="stylesheet">
      $nodes = $xpath->query("//link[@rel='stylesheet']");
    }

    foreach ($nodes as $node) {

      $href = $node->getAttribute('href');
      $delay_href = $node->getAttribute('delay-href');
      $id = $node->getAttribute('id');
      $combinedAttributes = $delay_href . ' ' . $href . ' ' . $id; // Combine href and id into a single string

      $shouldSkip = false; // Flag to check if current node should be skipped

      // Early skip if the string is in the never remove list
      foreach ($never_remove_list as $never_remove_item) {
        if (stripos($combinedAttributes, trim($never_remove_item)) !== false) {
          $shouldSkip = true;
        }
      }

      if ($shouldSkip) {
        continue; // Skip to the next node in the outer loop
      }

      $remove = false; // Flag to determine if the node should be removed

      if ($checkedDeqLinksCSSRadio === 'dequeue_all_links_css') {
        $remove = true;
      }

      // Exclude links from removal if the handle or src matches any value in list
      elseif ($checkedDeqLinksCSSRadio === 'dequeue_exclude_links_css') {
        $remove = true; // Assume removal unless an exclusion match is found
        if (!empty($exclude_list)) {
          foreach ($exclude_list as $exclude) {
            if (stripos($combinedAttributes, trim($exclude)) !== false) {
              $remove = false;
            }
          }
        }
      }

      // If the handle or src matches any value in list, remove the style
      elseif ($checkedDeqLinksCSSRadio === 'dequeue_specific_links_css') {
        if (!empty($include_list)) {
          foreach ($include_list as $include) {
            if (stripos($combinedAttributes, trim($include)) !== false) {
              $remove = true;
            }
          }
        }
      }

      if ($remove) {
        $node->parentNode->removeChild($node);
      }
    }

    return $dom->saveHTML();
  }

  // End buffer for removing styles
  public function buffer_end_remove_styles()
  {

    if ($this->buffering_started_remove_css) {
      if (ob_get_length() !== false) {
        ob_end_flush();
      }
      $this->buffering_started_remove_css = false;
    }
  }



  /*** DELAY CSS GLOBAL ***/

  // Delay Extarnal Styles
  public function buffer_start_global_delay()
  {

    if (get_option('enable_ext_delay_css') !== '1') {
      return;
    }

    // If the user has enabled auto-setting LCP images, if the page is new, edited, or the data is missing
    // Exit if `$post` is not a WP_Post object.
    global $post;
    if (!is_a($post, 'WP_Post')) {
      return;
    }

    // If the user has enabled auto-setting LCP images
    // If the data is missing
    // If the page is new or edited
    // And if we should be getting image data on this page
    if (Utility::getting_images_data()) {
      return;
    }

    // If the user has enabled auto-setting Used CSS
    // If the CSS file is missing
    // If the page is new or edited
    // And if we should be getting Used CSS on this page
    if (Utility::getting_used_css_data()) {
      return;
    }

    if ($_SERVER['REQUEST_METHOD'] !== 'GET') {
      return;
    }

    // PAGES - Get radio buttons and text areas
    $checkedCSSDelayPagesRadio = get_option('delay_css_pages_radio_checked');
    $exclude_list = get_option('delay_exclude_pages_css_textarea') ? array_map('trim', explode("\n", get_option('delay_exclude_pages_css_textarea'))) : [];
    $include_list = get_option('delay_specific_pages_css_textarea') ? array_map('trim', explode("\n", get_option('delay_specific_pages_css_textarea'))) : [];

    // Add home page settings checkboxses
    $exclude_home = get_option('delay_css_exclude_home') === '1';
    $include_home = get_option('delay_css_include_home') === '1';

    $all_pages = $checkedCSSDelayPagesRadio === 'delay_all_pages_css_radio';
    $exclude_pages = $checkedCSSDelayPagesRadio === 'delay_exclude_pages_css_radio';
    $include_pages = $checkedCSSDelayPagesRadio === 'delay_spacific_pages_css_radio';

    $current_URL = '';
    $is_home = '';

    if (!$all_pages) {
      $is_home = Utility::is_homepage();
    }

    // Run on all pages
    if ($all_pages) {
      ob_start(array($this, "delay_external_stylesheets"));
      $this->buffering_started_delay_css = true;
      return;
    }

    // Run on all except these excluded pages
    elseif ($exclude_pages) {
      if ($is_home && $exclude_home) {
        // Don't run on the homepage
        $this->buffering_started_delay_css = false;
        return;
      } else {
        // Don't run on pages from the list
        if (!empty($exclude_list)) {
          foreach ($exclude_list as $string) {
            $normalizedString = Utility::normalizeUrlString($string);
            if (stripos($current_URL, $normalizedString) !== false) {
              $this->buffering_started_delay_css = false;
              return;
            }
          }
        }
      }

      // Run if we are not on an excluded page
      $this->buffering_started_delay_css = true;
      ob_start(array($this, "delay_external_stylesheets"));
      return;
    }

    // Run only on specified pages
    elseif ($include_pages) {
      if ($is_home && $include_home) {
        // Run on Home page
        ob_start(array($this, "delay_external_stylesheets"));
        $this->buffering_started_delay_css = true;
        return;
      } else {
        if (!empty($include_list)) {
          foreach ($include_list as $string) {
            $normalizedString = Utility::normalizeUrlString($string);
            if (stripos($current_URL, $normalizedString) !== false) {
              ob_start(array($this, "delay_external_stylesheets"));
              $this->buffering_started_delay_css = true;
              return;
            }
          }
        }
      }
    }
  }

  // Delay stylesheets for each repeater based on the selections
  public function delay_external_stylesheets($html)
  {
    $dom = Utility::processHTML($html);

    // Check if the returned value is an instance of DOMDocument
    if (!$dom instanceof \DOMDocument) {
      return $html;
    }

    // We do have IDs 'super-speed-*' that can be delayed
    $never_remove_list = array(
      'superspeed',
      '/wp-includes/css/dashicons',
      '/wp-includes/css/admin-bar',
    );

    // LINKS - Get radio buttons and text areas
    $checkedCSSDelayLinksRadio = get_option('delay_css_links_radio_checked');
    $exclude_list = get_option('delay_exclude_links_css_textarea') ? array_map('trim', explode("\n", get_option('delay_exclude_links_css_textarea'))) : [];
    $include_list = get_option('delay_spacific_links_css_textarea') ? array_map('trim', explode("\n", get_option('delay_spacific_links_css_textarea'))) : [];

    // Delay all styles
    $xpath = new \DOMXPath($dom);
    $nodes = $xpath->query("//link[@rel='stylesheet']");

    foreach ($nodes as $node) {

      $href = $node->getAttribute('href');
      $id = $node->getAttribute('id');
      $combinedAttributes = $href . ' ' . $id; // Combine href and id into a single string

      $shouldSkip = false; // Flag to check if current node should be skipped

      // Early skip if the string is in the never remove list
      foreach ($never_remove_list as $never_remove_item) {
        if (stripos($combinedAttributes, trim($never_remove_item)) !== false) {
          $shouldSkip = true;
        }
      }

      if ($shouldSkip) {
        continue; // Skip to the next node in the outer loop
      }

      $delay = false; // Flag to determine if the node should be removed

      if ($checkedCSSDelayLinksRadio === 'delay_all_links_css') {
        $delay = true;
      }

      // Exclude links from removal if the handle or src matches any value in list
      elseif ($checkedCSSDelayLinksRadio === 'delay_exclude_links_css') {
        $delay = true; // Assume removal unless an exclusion match is found
        if (!empty($exclude_list)) {
          foreach ($exclude_list as $exclude) {
            if (stripos($combinedAttributes, trim($exclude)) !== false) {
              $delay = false;
            }
          }
        }
      }

      // If the handle or src matches any value in list, delay the style
      elseif ($checkedCSSDelayLinksRadio === 'delay_specific_links_css') {
        if (!empty($include_list)) {
          foreach ($include_list as $include) {
            if (stripos($combinedAttributes, trim($include)) !== false) {
              $delay = true;
            }
          }
        }
      }

      if ($delay) {
        $node->setAttribute("delay-href", $href);
        $node->removeAttribute("href");
      }
    }

    return $dom->saveHTML();
  }

  // Already performed the check if we are on the right page, and stored it in the variable '$this->buffering_started_critical_css'
  public function buffer_end_global_delay()
  {
    if ($this->buffering_started_delay_css) {
      if (ob_get_length() !== false) {
        ob_end_flush();
      }
      $this->buffering_started_delay_css = false;
    }
  }




  /*** USED CSS ***/

  // Cache used CSS file URL. Check if the file exists. Set meta if the page is edited.
  public function cache_template_type_and_if_file_exists_and_set_meta_if_edited()
  {
    // Main enable
    if (!Utility::get_used_css_enabled()) {
      return;
    }

    // If we should not run on this page
    if (!$this->used_css_should_run_on_page) {
      return;
    }

    $used_css_different_post_layout = get_option('used_css_different_post_layout') === '1';
    $used_css_different_cpt_layout = get_option('used_css_different_cpt_layout') === '1';

    $template_type = null;

    if (is_home()) {
      $template_type = 'is_home';
    } elseif (is_front_page()) {
      $template_type = 'is_front_page';
    } elseif (is_search()) {
      $template_type = 'is_search';
    } elseif (is_category()) {
      $template_type = 'is_category';
    } elseif (is_tag()) {
      $template_type = 'is_tag';
    } elseif (is_tax()) {
      $template_type = 'is_tax';
    } elseif (is_author()) {
      $template_type = 'is_author';
    } elseif (is_date()) {
      $template_type = 'is_date';
    } elseif (is_year()) {
      $template_type = 'is_year';
    } elseif (is_month()) {
      $template_type = 'is_month';
    } elseif (is_day()) {
      $template_type = 'is_day';
    } elseif (is_post_type_archive()) {
      $template_type = 'is_post_type_archive';
    } elseif (is_404()) {
      $template_type = 'is_404';
    } elseif (is_archive()) {
      $template_type = 'is_archive';
    }


    if (class_exists('WooCommerce')) {
      if (is_shop()) {
        $template_type = 'is_shop';
      } elseif (is_product_category()) {
        $template_type = 'is_product_category';
      } elseif (is_product_tag()) {
        $template_type = 'is_product_tag';
      } elseif (is_product()) {
        $template_type = 'is_product';
      } elseif (is_cart()) {
        $template_type = 'is_cart';
      } elseif (is_checkout()) {
        $template_type = 'is_checkout';
      } elseif (is_account_page()) {
        $template_type = 'is_account_page';
      } elseif (is_wc_endpoint_url()) {
        $template_type = 'is_wc_endpoint_url';
      }
    }

    if ($template_type === null) {

      // Remove any query string from the URI
      $request_path = parse_url($_SERVER['REQUEST_URI'], PHP_URL_PATH);
      $uri_segments = explode('/', trim($request_path, '/'));

      $uri_suffix = implode('-', $uri_segments);

      if (is_singular('page')) {
        $template_type = 'page-' . $uri_suffix;
      } elseif (is_singular('post')) {

        // If blog posts have a different layout
        if ($used_css_different_post_layout) {
          $template_type = 'post-' . $uri_suffix;
        } else {
          $template_type = 'post';
        }
      } elseif (is_singular()) {
        // Dynamically get the current singular post type
        $current_post_type = get_post_type();

        // If CPT posts have a different layout
        if ($used_css_different_cpt_layout) {
          // Check if the uri_suffix already starts with the post type and avoid duplication
          if (strpos($uri_suffix, $current_post_type) === 0) {
            // If the uri_suffix starts with the current_post_type, use it directly
            $template_type = $uri_suffix;
          } else {
            // If not, prepend the post type to the suffix
            $template_type = $current_post_type . '-' . $uri_suffix;
          }
        } else {
          $template_type = $current_post_type;
        }
      }
    }

    // Cache the template type
    $this->used_css_file_template = $template_type;

    // Save the $template_type as post meta so that we know if the post has been edited
    update_post_meta(get_the_ID(), 'post_edited_used_css', $template_type);

    // Check if the file exists
    $upload_dir = wp_upload_dir();
    $file_path = $upload_dir['basedir'] . '/superspeed-used-css/' . $template_type . '.css';
    if (file_exists($file_path)) {
      $this->used_css_file_exists = true;
    }
  }

  // Used CSS HTML elements
  public function html_elements_for_used_css()
  {

    // Main enable
    if (!Utility::get_used_css_enabled()) {
      return;
    }

    // If we should not run on this page
    if (!$this->used_css_should_run_on_page) {
      return;
    }

    $used_css_display_info = get_option('used_css_file_info') === '1';
    $file_link = '';

    // Get the Upload directory
    $upload_dir = wp_upload_dir();
    $file_path = $upload_dir['baseurl'] . '/superspeed-used-css/' . $this->used_css_file_template . '.css';

    // CREATE THE HTML ELEMENT THAT JS FUNCTIONS WILL USE TO GET THE URL

    if ($this->used_css_file_template) {

      // Echo the element with data attributes. The template name is used in get-used-css.js and refresh-used-css.js
      echo '<style 
        id="used-css-info" 
        data-template-or-slug="' . esc_attr($this->used_css_file_template) . '" 
        data-used-css-file-location="' . $file_path . '" >
        </style>';
    }


    // PRELOAD THE CSS FILE

    // If the user has enabled auto-setting Used CSS
    // If the CSS file is missing
    // If the page is new or edited
    // And if we should be getting Used CSS on this page
    if (!Utility::getting_used_css_data()) { // Do not load the file if we are generating a new file

      if ($this->used_css_file_exists) {
        // Generate a random number for fallback
        $random_number = mt_rand();

        // Get the uploads directory info in an array
        $uploads = wp_upload_dir();

        // Extract the path from the uploads URL by removing the domain and protocol
        $pattern = '/https?:\/\/[^\/]+/i';
        $relative_path = preg_replace($pattern, '', $uploads['baseurl']);

        // Ensure there are no leading slashes
        $relative_path = ltrim($relative_path, '/');

        // Define the base directory for CSS files using the relative path
        $css_file_path = $relative_path . '/superspeed-used-css/' . $this->used_css_file_template . '.css';
        $absolute_file_path = ABSPATH . $css_file_path;
        $file_version = file_exists($absolute_file_path) ? filemtime($absolute_file_path) : $random_number;

        // Construct the relative URL with versioning
        $file_url = '/' . $css_file_path . '?ver=' . $file_version;

        $preloadUsedCss = '<link rel="preload" id="preload-superspeed-used-css" href="' . esc_url($file_url) . '" as="style" onload="this.onload=null;this.rel=\'stylesheet\'">
                           <noscript><link rel="stylesheet" href="' . esc_url($file_url) . '"></noscript>';

        // Link the CSS file for use. Exclude from dequeue and delay.
        $callUsedCss = '<link rel="stylesheet" id="superspeed-used-css" href="' . esc_url($file_url) . '">';

        global $ss_global_variable_for_preloading_images;
        $ss_global_variable_for_preloading_images .= "\n" . $preloadUsedCss . "\n" . $callUsedCss;

        // Construct the full link to the Used CSS file to display on the frontend
        if ($used_css_display_info) {
          $site_url = site_url();
          $file_link = $site_url . $file_url;
        }
      }
    }

    // USED CSS INFO ADDED TO THE PAGE USING JS
    if ($used_css_display_info) {
      if (current_user_can('edit_others_posts') && $this->used_css_file_exists && $this->used_css_file_template) {
?>
        <script id="super-speed-used_css_info_on_page" defer>
          (function() {
            function initializeScript() {
              const isEditor = document.querySelector('body.can-edit');
              const pageInfo = document.getElementById('superspeed-page-info');
              if (isEditor && pageInfo && document.getElementById('preload-superspeed-used-css')) {
                pageInfo.insertAdjacentHTML('beforeend', '<div class="ss-page-info-title page-info-item" style="font-size: 1.25em; font-weight: 500;">Used CSS is Preloaded</div>');
                pageInfo.insertAdjacentHTML('beforeend', '<a class="page-info-list-item" href="<?php echo esc_url($file_link); ?>" style="margin-bottom: 1em; margin-left: 1em;" target="_blank">File URL: <?php echo esc_url($file_link); ?></a>');
              }
            }

            document.addEventListener('DOMContentLoaded', function() {
              if (document.readyState === 'interactive' || document.readyState === 'complete') {
                initializeScript();
              } else {
                window.addEventListener('load', initializeScript);
              }
            });
          })();
        </script>
    <?php
      }
    }
  }


  // Dynamically load files JS of frontend tools for CSS
  public function register_rest_routes()
  {

    // Define a common permission callback for all routes
    $permission_callback = function () {
      return current_user_can('manage_options');
    };

    // An array to define all the routes in one place
    $routes = [
      '/delete-css/' => 'handle_delete_used_css',
      '/delete-all-css/' => 'handle_delete_all_used_css',
      '/get-used-css/' => 'get_used_css_by_fetch_api_ajax',
    ];

    // Register each route with a loop
    foreach ($routes as $route => $callback) {
      register_rest_route('super-speed/v1', $route, [
        'methods' => 'POST',
        'callback' => [$this, $callback],
        'permission_callback' => $permission_callback,
      ]);
    }
  }


  // Dynamically load files JS of frontend tools for CSS
  public function enqueue_scripts_dynamically()
  {

    if (!current_user_can('manage_options')) {
      return;
    }

    // Main enable
    if (!Utility::get_used_css_enabled()) {
      return;
    }

    // If we shouold not run on this page
    if (!$this->used_css_should_run_on_page) {
      return;
    }

    global $post;
    // Exit if `$post` is not a WP_Post object.
    if (!is_a($post, 'WP_Post')) {
      return;
    }

    // Server paths to the JS files
    $script_base_path = plugin_dir_path(dirname(__FILE__, 2)) . 'assets/js/';

    global $ss_global_variable_for_version_control;

    $get_used_css_script_version = $ss_global_variable_for_version_control;
    $refresh_css_script_version = $ss_global_variable_for_version_control;

    // Get Used CSS
    if (!$this->used_css_file_exists) {
      if (!$ss_global_variable_for_version_control) {
        $get_used_css_script_path = $script_base_path . 'get-used-css.js';
        $get_used_css_script_version = filemtime($get_used_css_script_path);
      }
    }

    // Refresh Used CSS
    if (!$ss_global_variable_for_version_control) {
      $refresh_css_script_path = $script_base_path . 'refresh-used-css.js';
      $refresh_css_script_version = filemtime($refresh_css_script_path);
    }

    // Base URL for the scripts
    $script_base_url = plugins_url('assets/js/', dirname(__FILE__, 2));
    ?>

    <script id="super-speed-unique-global-var-for-sync">
      // Set up a global variable to check for, so that 'get-used-css.js' runs before 'get-images-data.js'
      // The variable is defined in 'get-used-css.js'
      if (typeof window.UniqueScriptExecutionState === "undefined") {
        window.UniqueScriptExecutionState = {};
      }
    </script>


    <script id="super-speed-data-for-dynamically-loaded-css-scripts">
      window.dynamicallyLoadedFilesDataCss = {
        deleteCssUrl: '<?php echo esc_js(rest_url('super-speed/v1/delete-css/')); ?>',
        deleteAllCssUrl: '<?php echo esc_js(rest_url('super-speed/v1/delete-all-css/')); ?>',
        getUsedCssUrl: '<?php echo esc_js(rest_url('super-speed/v1/get-used-css/')); ?>',
        cssNonce: '<?php echo wp_create_nonce('wp_rest'); ?>',
      };
    </script>

    <script id="super-speed-enqueue-css-scripts-dynamically" defer>
      (function() {
        function initializeScript() {

          // Don't load in an iframe, which is used for getting image data
          const isInIframe = (window !== window.top);
          if (isInIframe) {
            return;
          }

          // Conditions check
          if (!document.body.classList.contains('can-edit') || !document.getElementById('wpadminbar')) {
            return;
          }

          const scriptBaseUrl = '<?php echo $script_base_url; ?>';

          const refreshUsesCssFileVersion = '<?php echo $refresh_css_script_version; ?>';
          var refreshCssScriptSrc = scriptBaseUrl + 'refresh-used-css.js?ver=' + refreshUsesCssFileVersion;

          const usedCssFileExists = <?php echo $this->used_css_file_exists ? 'true' : 'false'; ?>;

          function getUsedCSS() {
            if (!usedCssFileExists) {
              const getUsesCssFileVersion = '<?php echo $get_used_css_script_version; ?>';
              var getUsedCssScriptSrc = scriptBaseUrl + 'get-used-css.js?ver=' + getUsesCssFileVersion;
              loadScript(getUsedCssScriptSrc);
            } else {}
          }

          function loadScript(src) {
            var script = document.createElement('script');
            script.src = src;
            document.body.appendChild(script);
          }

          loadScript(refreshCssScriptSrc);
          getUsedCSS();
        }

        if (document.readyState === 'complete') {
          initializeScript();
        } else {
          window.addEventListener('load', initializeScript);
        }
      })();
    </script>
<?php
  }


  // AJAX actions for saving Used CSS
  public function get_used_css_by_fetch_api_ajax(\WP_REST_Request $request)
  {
    $nonce = $request->get_header('X-WP-Nonce');
    if (!wp_verify_nonce($nonce, 'wp_rest')) {
      return new \WP_Error('nonce_verification_failed', 'Nonce verification failed', ['status' => 403]);
    }

    // Since the data is sent as JSON, use $request->get_json_params() instead of $_POST
    $params = $request->get_json_params();
    $cssData = $params['css'] ?? '';
    $templateType = $params['template_type'] ?? '';

    // Use $templateType to generate the filename
    $filename = sanitize_title($templateType) . '.css';

    $upload_dir = wp_upload_dir();
    $custom_dir = '/superspeed-used-css/';
    $dir_path = $upload_dir['basedir'] . $custom_dir;

    if (!file_exists($dir_path)) {
      if (!wp_mkdir_p($dir_path)) {
        error_log("Unable to create directory at: {$dir_path}");
        wp_send_json_error(['message' => 'Failed to create directory for CSS file.']);
        wp_die();
      }
    }

    $file_path = $dir_path . '/' . $filename;

    // Save the CSS data to the file
    if (file_put_contents($file_path, $cssData)) {
      wp_send_json_success(['message' => 'CSS file saved successfully.']);
    } else {
      error_log("Failed to save CSS data to file at: {$file_path}");
      wp_send_json_error(['message' => 'Failed to save CSS file.']);
    }

    wp_die(); // Terminate AJAX request
  }


  // If the post is edited, delete the Used CSS file
  public function if_post_is_edited_remove_used_css_file($post_id)
  {
    $template_type = get_post_meta($post_id, 'post_edited_used_css', true);

    if ($template_type) {
      $upload_dir = wp_upload_dir();
      $file_path = $upload_dir['basedir'] . '/superspeed-used-css/' . $template_type . '.css';

      if (file_exists($file_path)) {
        wp_delete_file($file_path);
        delete_post_meta($post_id, 'post_edited_used_css');
      }
    }
  }


  // Delete the CSS file from the page we are on, for the specific page/template
  public function handle_delete_used_css(\WP_REST_Request $request)
  {
    // Explicit nonce verification for clarity and security
    $nonce = $request->get_header('X-WP-Nonce');
    if (!wp_verify_nonce($nonce, 'wp_rest')) {
      return new \WP_Error('nonce_verification_failed', 'Nonce verification failed', ['status' => 403]);
    }

    $cssFileUrl = $request->get_json_params()['css_file_url'] ?? '';
    if (!$cssFileUrl) {
      return new \WP_Error('missing_css_file_url', 'CSS file URL is required.', ['status' => 400]);
    }

    // Convert URL to a file path on the server
    $upload_dir = wp_upload_dir();
    $basePath = wp_make_link_relative($upload_dir['baseurl']);
    $filePath = str_replace($basePath, $upload_dir['basedir'], wp_make_link_relative($cssFileUrl));

    // Prevent directory traversal
    $filePath = realpath($filePath);

    // Check if the file is within the uploads directory and exists
    if (!$filePath || strpos($filePath, $upload_dir['basedir']) !== 0 || !file_exists($filePath)) {
      return new \WP_Error('invalid_file_path_or_not_found', 'Invalid file path or file does not exist.', ['status' => 400]);
    }

    if (!unlink($filePath)) { // Attempt to delete the file and check the outcome in one step
      error_log("Failed to delete CSS file at path: $filePath"); // Log error for server-side debugging
      return new \WP_Error('file_deletion_failed', 'Failed to delete CSS file.', ['status' => 500]);
    }

    return new \WP_REST_Response(['message' => 'CSS file deleted successfully'], 200);
  }

  // Delete all CSS files from the frontend
  public function handle_delete_all_used_css(\WP_REST_Request $request)
  {

    // Explicit nonce verification for clarity and security
    $nonce = $request->get_header('X-WP-Nonce');
    if (!wp_verify_nonce($nonce, 'wp_rest')) {
      return new \WP_Error('nonce_verification_failed', 'Nonce verification failed', ['status' => 403]);
    }

    // Define the directory path
    $upload_dir = wp_upload_dir();
    $dir_path = $upload_dir['basedir'] . '/superspeed-used-css/';

    // Check if directory exists
    if (!is_dir($dir_path)) {
      wp_send_json_error('CSS directory does not exist');
      return;
    }

    $dir = new \DirectoryIterator($dir_path);
    foreach ($dir as $fileinfo) {
      if (!$fileinfo->isDot() && strtolower($fileinfo->getExtension()) === 'css') {
        $filePath = $dir_path . $fileinfo->getFilename();
        if (!unlink($filePath)) {
          // Log failure without altering the response to the client
          error_log("Failed to delete CSS file at path: $filePath");
        }
      }
    }

    // Return a success message without specifying the number of files deleted
    wp_send_json_success('All applicable CSS files processed for deletion.');
  }
}
