<?php

/**
 * Plugin Name: SuperSpeed
 * Plugin URI: https://WPSuperSpeed.com
 * Description: Massively improve the speed of your website with SuperSpeed - a system of advanced tools, designed to drastically reduce page load times.
 * Version: 1.0.6
 * Author: Jovan Ralić
 * Author URI: https://www.facebook.com/groups/wpsuperspeed
 * License: Proprietary
 * License URI: https://WPSuperSpeed.com/license.txt
 * Text Domain: wpssxtdomain
 * Domain Path: /languages
 *
 * @package WP_SS
 *
 * COPYRIGHT 2024 JOVAN RALIĆ. ALL RIGHTS RESERVED.
 * 
 * RESTRICTIONS: This software is bound by a proprietary license and is exclusively distributed by Jovan Ralić.
 * Unauthorized use, duplication, modification, distribution, or disclosure in whole or in part is strictly prohibited 
 * under the terms of the license, except as explicitly authorized by a licensing agreement signed by Jovan Ralić.
 * 
 * LEGAL PROTECTIONS: This software and the associated intellectual property rights are protected by national and
 * international copyright and patent laws. Violation of these rights will lead to legal enforcement and penalties.
 * 
 * LICENSE DETAILS: For full license terms and conditions, refer to the license agreement available at:
 * https://WPSuperSpeed.com/license.txt.
 */

// Exit if accessed directly
if (!defined('ABSPATH')) {
  exit;
}

// Plugin activation
register_activation_hook(__FILE__, 'superspeed_activate');

// Add to the database radio button values. Inputs may seem active, but are not added if not interacted with.
add_action('admin_init', 'superspeed_activate');
function superspeed_activate()
{
  // String settings: Radio button and dropdown states
  $default_string_settings  = [
    // Radio button => default value
    // CSS
    'load_used_css_on_pages_radio' => 'used_css_load_all_pages',
    'dequeue_css_links_radio_checked' => 'dequeue_all_links_css',
    'remove_pages_css_radio_checked' => 'dequeue_all_pages_css',
    'delay_css_links_radio_checked' => 'delay_all_links_css',
    'delay_css_pages_radio_checked' => 'delay_all_pages_css_radio',
    //JS
    'ss_dequeue_js_links_radio_checked' => 'dequeue_all_links_js',
    'ss_deq_radio_checked' => 'dequeue_all_pages',
    'ss_defer_radio_checked' => 'defer_all_scripts',
    'ss_delay_js_links_radio_checked' => 'delay_ext_all_links_radio',
    'ss_delay_ext_pages_radio_checked' => 'delay_ext_all_pages_radio',
    // Media
    'imagesPlaceholderRadio' => 'lazy_image_placeholder_gray',
    'img_first_x_select_pages_radio' => 'img_first_x_all_pages',
    'back_img_placeholder_radio' => 'back_img_placeholder_gray',
    // Prefetch
    'preload_links_color_coding_visibility' => 'editing_permissions',

  ];

  // Numeric settings: Values typically from input fields requiring validation
  $default_numeric_settings = [

    // Dropdown Select => default value
    'lazy_img_preload_first_x_select' => 1,
    'lazy_img_fadein_duration' => 1.0,
    'lazy_bg_fadein_duration' => 1.0,

    'preload_links_mobile_width' => 800.0,
    'preload_links_max_retries' => 3.0,
    'preload_links_retry_delay' => 1.0,
    'preload_links_fetch_timeout_desktop' => 5.0,
    'preload_links_server_response_desktop' => 250.0,
    'preload_links_initial_delay_mobile' => 5.0,
    'preload_links_next_file_delay_mobile' => 5.0,
    'preload_links_fetch_timeout_mobile' => 7.0,
    'preload_links_server_response_mobile' => 250.0,

    'image_auto_set_fixed_pixels_desktop' => 1.5,
    'image_auto_set_fixed_pixels_tablet' => 2.0,
    'image_auto_set_fixed_pixels_mobile' => 2.5,

    'backgound_auto_set_fixed_pixels_desktop' => 1.5,
    'backgound_auto_set_fixed_pixels_tablet' => 2.0,
    'backgound_auto_set_fixed_pixels_mobile' => 2.5,

  ];

  // Combine both string and numeric settings
  $default_settings = $default_string_settings + $default_numeric_settings;

  foreach ($default_settings as $key => $value) {
    if (!get_option($key)) {
      update_option($key, $value);
    }
  }
}


// Plugin temporary deactivation
register_deactivation_hook(__FILE__, 'superspeed_deactivate');
function superspeed_deactivate()
{
  // Your deactivation logic here
  // This might include clearing cache files, removing scheduled events, etc.
}

// Require necessary files
require_once plugin_dir_path(__FILE__) . 'includes/class/Utility.php';
require_once plugin_dir_path(__FILE__) . 'includes/class/Main.php';
require_once plugin_dir_path(__FILE__) . 'includes/class/DB.php';
require_once plugin_dir_path(__FILE__) . 'includes/class/Config.php';
require_once plugin_dir_path(__FILE__) . 'admin/templates/class/Settings.php';
require_once plugin_dir_path(__FILE__) . 'includes/class/ScriptHandler.php';
require_once plugin_dir_path(__FILE__) . 'includes/class/StyleHandler.php';
require_once plugin_dir_path(__FILE__) . 'includes/class/Media.php';
require_once plugin_dir_path(__FILE__) . 'includes/class/PreloadLinks.php';
require_once plugin_dir_path(__FILE__) . 'includes/class/FixThemesAndPlugins.php';
//require_once plugin_dir_path(__FILE__) . 'includes/class/SSElementor.php';

// Register the function to be called upon plugin activation
register_activation_hook(__FILE__, array('\SuperSpeed\DB', 'create_wp_critical_css_repeater_fields_table'));

// Initialize class functionalies
new SuperSpeed\DB();
new SuperSpeed\Config();
SuperSpeed\Utility::init(); // Static method call
new SuperSpeed\Main();
new SuperSpeed\ScriptHandler();
new SuperSpeed\Media();
new SuperSpeed\PreloadLinks();
new SuperSpeed\FixThemesAndPlugins();
new SuperSpeed\Admin\Settings(__FILE__);

//new SuperSpeed\SSElementor(); // Temporarely disabled

// Handeling StyleHandler differerently, because functions and variables are used outside the class to construct the repeater field dropdown selection
// Save the new instance of StyleHandler in a variable
$styleHandlerInstance = new SuperSpeed\StyleHandler();

// Make the StyleHandler instance global
$GLOBALS['styleHandler'] = $styleHandlerInstance;
