<?php
if (!defined('WP_UNINSTALL_PLUGIN')) {
  die; // Exit if accessed directly
}

if (!current_user_can('activate_plugins')) {
  return; // Check for sufficient permissions
}

// Remove Used CSS files and directory
$upload_dir = wp_upload_dir();
$css_dir = $upload_dir['basedir'] . '/superspeed-used-css';
if (is_dir($css_dir)) {
  // Delete all files in directory
  $files = glob($css_dir . '/*'); // Get all file names
  foreach ($files as $file) { // Iterate files
    if (is_file($file)) {
      unlink($file); // Delete file
    }
  }
  rmdir($css_dir); // Delete directory after emptying
}

// List of options to remove
$options = [
  /// JS ///

  // Dequeue Scripts
  'ss_enable_dequeue',
  'ss_dequeue_js_links_radio_checked',
  'ss_dequeue_exclude_links_js_textarea',
  'ss_dequeue_spacific_links_js_textarea',
  'ss_deq_radio_checked',
  'ss_exc_deq_js_textarea',
  'ss_deq_spec_js_textarea',
  'ss_dequeue_js_exclude_home',
  'ss_dequeue_js_include_home',
  'dequeue_scripts_exclude_3rd_party',

  // Defer Scripts
  'ss_enable_defer_ext_js',
  'ss_defer_radio_checked',
  'ss_exc_defer_textarea',
  'ss_defer_specific_textarea',
  'ss_defer_exclude_dependent',
  'ss_defer_exclude_support',

  // Delay Scripts
  'ss_enable_delay_ext_js',
  'ss_delay_js_links_radio_checked',
  'ss_delay_ext_exclude_links_textarea',
  'ss_delay_ext_specific_links_textarea',
  'ss_delay_ext_pages_radio_checked',
  'ss_delay_ext_excl_pages_textarea',
  'ss_delay_ext_specific_pages_textarea',
  'ss_delay_js_exclude_home',
  'ss_delay_js_include_home',

  /// CSS ///

  // Preload CSS
  'preload_styles',

  // Used CSS
  'enable_used_css',
  'used_css_file_info',
  'used_css_different_post_layout',
  'used_css_different_cpt_layout',
  'load_used_css_on_pages_radio',
  'used_css_exclude_home',
  'used_css_exclude_pages_textarea',
  'used_css_include_home',
  'used_css_spacific_pages_textarea',

  // Dequeue CSS
  'enable_dequeue_styles',
  'dequeue_css_links_radio_checked',
  'dequeue_exclude_links_css_textarea',
  'dequeue_spacific_links_css_textarea',
  'remove_pages_css_radio_checked',
  'dequeue_exclude_pages_css_textarea',
  'dequeue_spacific_pages_css_textarea',
  'dequeue_css_exclude_home',
  'dequeue_css_include_home',
  'dequeue_css_exclude_3rd_party',
  'used_css_exclude_google',

  // Delay CSS
  'enable_ext_delay_css',
  'delay_css_links_radio_checked',
  'delay_exclude_links_css_textarea',
  'delay_spacific_links_css_textarea',
  'delay_css_pages_radio_checked',
  'delay_exclude_pages_css_textarea',
  'delay_specific_pages_css_textarea',
  'delay_css_exclude_home',
  'delay_css_include_home',

  // Critical CSS
  'critical_css_enable',

  /// IMAGE DATA COLLECTION ///
  'image_data_collection_exclude_home',
  'image_data_collection_exclude_pages_cbx',
  'image_data_collection_exclude_pages_textarea',
  'image_data_collection_exclude_posts',
  'image_data_collection_exclude_products',
  'image_data_collection_exclude_cpts',
  'image_data_collection_exclude_cpts_textarea',

  /// LAZY LOAD ///

  // Image Data Collection
  'image_data_collection',

  // Images
  'lazy_load_images',
  'lazy_load_images_checkbox',
  'lazyloaded_img_visually_mark',
  'lazy_img_fadein_exclude_selector',
  'lazy_img_fadein_exclude_selector_textarea',
  'lazy_img_fadein_exclude_parent_selector',
  'lazy_img_fadein_exclude_parent_selector_textarea',
  'img_first_x_exlude_selector',
  'img_first_x_exlude_selector_textarea',
  'img_first_x_exlude_parent_selector',
  'img_first_x_exlude_selector_parent_textarea',
  'preloaded_img_visually_mark',
  'lazy_img_exclude_home_cbx',
  'lazy_img_include_home_cbx',
  'img_first_x_select_pages_radio',
  'img_first_x_exlude_pages_textarea',
  'img_first_x_include_pages_textarea',
  'lazy_img_fadein',
  'lazy_img_fadein_duration',
  'image_auto_set_thumbnails',
  'image_auto_set_fixed_pixels_desktop',
  'image_auto_set_fixed_pixels_tablet',
  'image_auto_set_fixed_pixels_mobile',
  'image_auto_set_exclude_selector',
  'image_auto_set_exclude_selector_textarea',
  'image_auto_set_exclude_parent_selector',
  'image_auto_set_exclude_parent_selector_textarea',
  'image_auto_set_thumbs_info',
  'lazy_image_exclude_home',
  'lazy_image_exclude_pages_cbx',
  'lazy_image_exclude_pages_textarea',
  'lazy_image_exclude_selector',
  'lazy_image_exclude_selector_textarea',
  'lazy_image_exclude_parent_selector',
  'lazy_image_exclude_parent_selector_textarea',
  'imagesPlaceholderRadio',
  'lazy_img_placeholder_exclude_selector',
  'lazy_img_placeholder_exclude_selector_textarea',
  'lazy_img_placeholder_exclude_parent_selector',
  'lazy_img_placeholder_exclude_parent_selector_textarea',
  'img_load_smallest_thumbnail_exclude_selector',
  'img_load_smallest_thumbnail_exclude_selector_textarea',
  'img_load_smallest_thumbnail_exclude_parent_selector',
  'img_load_smallest_thumbnail_exclude_parent_selector_textarea',
  'lazy_img_preload_first_enable',
  'lazy_img_preload_first_x_select',
  'lazy_img_preload_selector_enable',
  'preload_featured_img_blog',
  'preload_featured_img_product',
  'lazy_img_preload_selector',
  'lazy_img_selector_textarea',
  'lazy_img_preload_parent_selector',
  'lazy_img_parent_selector_textarea',

  /// Background Images ///
  'lazy_load_background',
  'lazy_background_checkbox',
  'lazy_background_exclude_selector',
  'lazy_background_exclude_selector_textarea',
  'lazy_background_exclude_parent_selector',
  'lazy_background_exclude_parent_selector_textarea',
  'lazy_background_exclude_pages_cbx',
  'lazy_background_exclude_pages_textarea',
  'lazy_background_exclude_home',
  'lazy_bg_fadein',
  'lazy_bg_fadein_duration',
  'preloaded_bg_visually_mark',
  'preloaded_bg_lazyloaded_mark',
  'preload_background_images',
  'preload_background_selector',
  'preload_background_selector_textarea',
  'preload_background_parent_selector',
  'preload_background_parent_selector_textarea',
  'back_img_placeholder_radio',
  'background_auto_set_thumbnails',
  'background_auto_set_thumbs_info',
  'image_custom_pixel_density_desktop',
  'image_custom_pixel_density_tablet',
  'image_custom_pixel_density_mobile',
  'background_custom_pixel_density_tablet',
  'background_custom_pixel_density_mobile',
  'backgound_auto_set_fixed_pixels_desktop',
  'backgound_auto_set_fixed_pixels_tablet',
  'backgound_auto_set_fixed_pixels_mobile',
  'backgound_auto_set_exclude_selector',
  'backgound_auto_set_exclude_selector_textarea',
  'backgound_auto_set_exclude_parent_selector',
  'backgound_auto_set_exclude_parent_selector_textarea',

  /// LCP ///
  'lcp_enable',
  'visually_mark_lcp',
  'display_lcp_info',
  'auto_preload_lcp',
  'lcp_prelod_exclude_selector',
  'lcp_prelod_exclude_selector_textarea',
  'lcp_prelod_exclude_parent_selector',
  'lcp_prelod_exclude_parent_selector_textarea',
  'lcp_prelod_exclude_home_cbx',
  'lcp_prelod_exclude_pages_cbx',
  'lcp_prelod_exclude_pages_textarea',
  'lcp_prelod_exclude_posts',
  'lcp_prelod_exclude_products',
  'lcp_prelod_exclude_cpts',
  'lcp_prelod_exclude_cpts_textarea',

  /// iFrames ///
  'lazy_load_iframes',
  'lazy_iframe_exclude_selector',
  'lazy_iframe_exclude_selector_textarea',
  'lazy_iframe_exclude_parent_selector',
  'lazy_iframe_exclude_parent_selector_textarea',
  'lazy_iframe_exclude_pages_cbx',
  'lazy_iframe_exclude_pages_textarea',
  'lazy_iframe_placeholder',
  'lazy_iframe_exclude_home_cbx',
  'lazyloaded_iframe_visually_mark',

  /// Videos ///
  'enable_lazy_load_videos',
  'lazy_video_thumb_preload_selector',
  'lazy_video_thumb_preload_selector_textarea',
  'lazy_video_thumb_preload_parent_selector',
  'lazy_video_thumb_preload_parent_selector_textarea',
  'lazy_video_exclude_pages_cbx',
  'lazy_video_exclude_pages_textarea',
  'lazy_video_exclude_home_cbx',
  'lazy_video_exclude_selector',
  'lazy_video_exclude_selector_textarea',
  'lazy_video_exclude_parent_selector',
  'lazy_video_exclude_parent_selector_textarea',
  'visually_mark_non_embeded_yt',
  'preloaded_thumbnail_visually_mark',
  'lazyloaded_video_visually_mark_js',

  // Preload Links
  'enable_preload_links',
  'preload_links_enable_mobile',
  'preload_links_mobile_width',
  'preload_links_max_retries',
  'preload_links_retry_delay',
  'preload_stylesheets_desktop',
  'preload_javaScript_desktop',
  'preload_images_desktop',
  'preload_links_fetch_timeout_desktop',
  'preload_links_server_response_desktop',
  'preload_links_for_logged_in_users',
  'preload_links_test_mode',
  'preload_stylesheets_mobile',
  'preload_javaScript_mobile',
  'preload_images_mobile',
  'preload_links_initial_delay_mobile',
  'preload_links_next_file_delay_mobile',
  'preload_links_fetch_timeout_mobile',
  'preload_links_server_response_mobile',
  'preload_links_color_coding_enable',
  'preload_links_color_coding_visibility'
];

// Delete options
foreach ($options as $option) {
  delete_option($option);
}
